@@ -1,6 +1,7 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2003 Andreas Beckermann (b_mann@gmx.de)
+    Copyright (C) 2003 Rivo Laks (rivolaks@hot.ee)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,6 +20,7 @@
 
 #include "bocamera.h"
 
+#include "../bomemory/bodummymemory.h"
 #include "boautocamera.h"
 #include "defines.h"
 #include "bosonconfig.h"
@@ -30,25 +32,20 @@
 #include <qdom.h>
 
 // Camera limits
-// These are min/max height of the camera (between camera's _position_, not
-//  lookat point, and ground)
-#define CAMERA_MIN_Z 2
-#define CAMERA_MAX_Z 100
-#define CAMERA_MAX_RADIUS 70
-// This is tangens of the minimum camera angle allowed (if angle is 90, camera
-//  is looking straight at the ground from top; if angle is 0, camera is looking
-//  at the horizon)
-// This is tan(30 degrees) atm, i.e. min camera angle is 30 degrees
-//#define MIN_CAMERA_ANGLE_TAN 0.57735027
-// 30 degrees limit is fatal to performance atm, so we use 45 degrees instead
-#define MIN_CAMERA_ANGLE_TAN 1.0
-// This helps to avoid tiny rounding errors which would be causing infinite loops
-#define CAMERA_Z_ROUNDING_ERROR 0.001
+// These are min/max distances between camera's position and lookat point.
+// Note that min distance MUST be > 0
+#define CAMERA_MIN_DISTANCE 2
+#define CAMERA_MAX_DISTANCE 200
+// Maximum allowed angle between z-axis and camera (0 = )
+#define CAMERA_MAX_XROTATION 50
 
-#define NEW_Z_CALCULATIONS
+// Height of the lookat point (from the ground)
+#define CAMERA_LOOKAT_HEIGHT 1
+// Minimum allowed z-distance between camera and ground
+#define CAMERA_MIN_HEIGHT 0.5
 
 
-#include <GL/glu.h>
+#include <bogl.h>
 
 #include <math.h>
 
@@ -98,7 +95,7 @@ void BoCamera::setAutoCamera(BoAutoCamera* a)
   }
 }
 
-void BoCamera::setGluLookAt(const BoVector3& cameraPos, const BoVector3& lookAt, const BoVector3& up)
+void BoCamera::setGluLookAt(const BoVector3Float& cameraPos, const BoVector3Float& lookAt, const BoVector3Float& up)
 {
   mLookAt = lookAt;
   mCameraPos = cameraPos;
@@ -106,17 +103,17 @@ void BoCamera::setGluLookAt(const BoVector3& cameraPos, const BoVector3& lookAt,
   setCameraChanged(true);
 }
 
-void BoCamera::changeLookAt(const BoVector3& diff)
+void BoCamera::changeLookAt(const BoVector3Float& diff)
 {
   setLookAt(lookAt() + diff);
 }
 
-void BoCamera::changeCameraPos(const BoVector3& diff)
+void BoCamera::changeCameraPos(const BoVector3Float& diff)
 {
   setCameraPos(cameraPos() + diff);
 }
 
-void BoCamera::changeUp(const BoVector3& diff)
+void BoCamera::changeUp(const BoVector3Float& diff)
 {
   setUp(up() + diff);
 }
@@ -200,8 +197,8 @@ bool BoCamera::saveAsXML(QDomElement& root)
   root.setAttribute("LookAtZ", lookAt().z());
 
   root.setAttribute("CameraPosX", cameraPos().x());
-  root.setAttribute("CameraPosX", cameraPos().y());
-  root.setAttribute("CameraPosX", cameraPos().z());
+  root.setAttribute("CameraPosY", cameraPos().y());
+  root.setAttribute("CameraPosZ", cameraPos().z());
 
   root.setAttribute("UpX", up().x());
   root.setAttribute("UpY", up().y());
@@ -238,31 +235,31 @@ void BoCamera::applyCameraToScene()
   setCameraChanged(false);
 }
 
-const BoVector3& BoCamera::cameraPos()
+const BoVector3Float& BoCamera::cameraPos()
 {
   return mCameraPos;
 }
 
-const BoVector3& BoCamera::up()
+const BoVector3Float& BoCamera::up()
 {
   return mUp;
 }
 
-void BoCamera::setLookAt(const BoVector3& pos)
+void BoCamera::setLookAt(const BoVector3Float& pos)
 {
   mLookAt = pos;
   setPositionDirty();
   setCameraChanged(true);
 }
 
-void BoCamera::setCameraPos(const BoVector3& pos)
+void BoCamera::setCameraPos(const BoVector3Float& pos)
 {
   mCameraPos = pos;
   setPositionDirty();
   setCameraChanged(true);
 }
 
-void BoCamera::setUp(const BoVector3& up)
+void BoCamera::setUp(const BoVector3Float& up)
 {
   mUp = up;
   setPositionDirty();
@@ -289,11 +286,11 @@ BoGameCamera::BoGameCamera(const BosonCanvas* canvas)
 BoGameCamera& BoGameCamera::operator=(const BoGameCamera& c)
 {
   BoCamera::operator=(c);
-  mPosZ = c.mPosZ;
+  mDistance = c.mDistance;
   mRotation = c.mRotation;
-  mRadius = c.mRadius;
+  mXRotation = c.mXRotation;
 
-  mWantedAbsoluteZ = c.mWantedAbsoluteZ;
+  mActualDistance = c.mActualDistance;
 
   mFree = c.mFree;
   mLimits = c.mLimits;
@@ -307,42 +304,16 @@ BoGameCamera& BoGameCamera::operator=(const BoGameCamera& c)
 void BoGameCamera::init()
 {
   setAutoCamera(new BoAutoGameCamera(this));
-  initStatic();
   mCanvas = 0;
-  mPosZ = 8.0f;
+
+  mDistance = 8.0f;
+  mActualDistance = 8.0f;
   mRotation = 0.0f;
-  mRadius = 5.0f;
-  mWantedAbsoluteZ = mPosZ;
+  mXRotation = 30.0f;
   mFree = false;
   mLimits = true;
 }
 
-void BoGameCamera::initStatic()
-{
-  static bool initialized = false;
-  if(initialized)
-  {
-    return;
-  }
-  initialized = true;
-  boConfig->addDynamicEntry(new BoConfigDoubleEntry(boConfig, "CameraMinZ",
-      CAMERA_MIN_Z));
-  boConfig->addDynamicEntry(new BoConfigDoubleEntry(boConfig, "CameraMaxZ",
-      CAMERA_MAX_Z));
-}
-
-float BoGameCamera::minCameraZ()
-{
-  initStatic();
-  return (float)boConfig->doubleValue("CameraMinZ");
-}
-
-float BoGameCamera::maxCameraZ()
-{
-  initStatic();
-  return (float)boConfig->doubleValue("CameraMaxZ");
-}
-
 void BoGameCamera::updateCamera()
 {
   if(mFree)
@@ -351,19 +322,22 @@ void BoGameCamera::updateCamera()
     // are set directly, so return here
     return;
   }
-  float diffX = 0.0f;
-  float diffY = 0.0f;
-  float radius = this->radius();
-  if(radius <= 0.02f)
-  {
-    // If radius is 0, up vector will be wrong so we change it
-    radius = 0.02f;
-  }
-  Bo3dTools::pointByRotation(&diffX, &diffY, rotation(), radius);
+  BoMatrix matrix;
+  matrix.rotate(mRotation, 0, 0, 1);
+  matrix.rotate(mXRotation, 1, 0, 0);
+  matrix.translate(0, 0, mActualDistance);
+  BoVector3Float in(0, 0, 0), out;
+  matrix.transform(&out, &in);
+  float diffX, diffY;
+  Bo3dTools::pointByRotation(&diffX, &diffY, 0, 0);
+
 
   // We don't want any validation here (we'd get infinite loops otherwise)
-  BoCamera::setCameraPos(BoVector3(lookAt().x() + diffX, lookAt().y() + diffY, lookAt().z() + z()));
-  BoCamera::setUp(BoVector3(-diffX, -diffY, 0.0f));
+  BoCamera::setCameraPos(lookAt() + out);
+
+  BoVector3Float upvector(0, 1, 0);
+  matrix.transform(&out, &upvector);
+  BoCamera::setUp(out);
 
   setPositionDirty(false);
   setCameraChanged(true);
@@ -391,7 +365,7 @@ void BoGameCamera::checkLookAtPosition()
   // We must first update lookAt position by calling
 //  bool dirty = positionDirty();
 //  setPositionDirty(false);
-  BoVector3 lookAt_ = lookAt();
+  BoVector3Float lookAt_ = lookAt();
 //  setPositionDirty(dirty);
 
   if(lookAt_.x() < 0.0f)
@@ -435,48 +409,40 @@ void BoGameCamera::checkCameraPosition()
   //  lookAt.z() + z()  (which should always be equal to just cameraPos.z())
   if(mCanvas)
   {
-    float camposx = cameraPos().x() * BO_TILE_SIZE;
-    float camposy = -cameraPos().y() * BO_TILE_SIZE;
-    camposx = QMAX(0, QMIN(camposx, mCanvas->mapWidth() * BO_TILE_SIZE));
-    camposy = QMAX(0, QMIN(camposy, mCanvas->mapHeight() * BO_TILE_SIZE));
+    float camposx = cameraPos().x();
+    float camposy = -cameraPos().y();
+    camposx = QMAX(0.0f, QMIN(camposx, (float)mCanvas->mapWidth()));
+    camposy = QMAX(0.0f, QMIN(camposy, (float)mCanvas->mapHeight()));
     float groundz = mCanvas->heightAtPoint(camposx, camposy);
-    // FIXME: this assumes that lookAt.z() == 0, which may not always be the case
-    // TODO: maybe also change radius to keep camera's angle constant
-#ifdef NEW_Z_CALCULATIONS
-    if(mWantedAbsoluteZ < groundz + minCameraZ())
+    if(cameraPos().z() < groundz + CAMERA_MIN_HEIGHT)
     {
-      // We may have tiny rounding errors here, so we set z to little bigger
-      //  number than the limit in order to avoid infinite loop
-      //  (otherwise we may have  groundz + minCameraZ() = 0.0000001, which will
-      //  be rounded to 0 when calling setZ(), and then setZ() sets z to 0 and
-      //  this function compares values and finds out that 0 < 0.0000001, so it
-      //  calls setZ() ... )
-      setZ(groundz + minCameraZ() + CAMERA_Z_ROUNDING_ERROR - lookAt().z());
+      // Shorten the distance from the lookat point until the camera is enough
+      //  above the ground.
+      const int iterations = 25;
+      const float step = 1 / (float)iterations;
+      for(float factor = 1 - step; factor >= 0; factor -= step)
+      {
+        mActualDistance = QMAX(CAMERA_MIN_DISTANCE, mDistance * factor);
+        // Recalc camera pos
+        setPositionDirty();
+        BoVector3Float newcamerapos = cameraPos();
+        // Find out ground height at the new camera pos
+        camposx = cameraPos().x();
+        camposy = -cameraPos().y();
+        camposx = QMAX(0.0f, QMIN(camposx, (float)mCanvas->mapWidth()));
+        camposy = QMAX(0.0f, QMIN(camposy, (float)mCanvas->mapHeight()));
+        groundz = mCanvas->heightAtPoint(camposx, camposy);
+        if(cameraPos().z() >= groundz + CAMERA_MIN_HEIGHT)
+        {
+          // Valid point found.
+          break;
+        }
+      }
     }
-    else if(mWantedAbsoluteZ > groundz + maxCameraZ())
+    else
     {
-      setZ(groundz + maxCameraZ() - CAMERA_Z_ROUNDING_ERROR - lookAt().z());
+      mActualDistance = mDistance;
     }
-    else if(QABS(mWantedAbsoluteZ - lookAt().z() - z()) > CAMERA_Z_ROUNDING_ERROR)
-    {
-      changeZ((mWantedAbsoluteZ - lookAt().z()) - z());
-    }
-#else
-    if(cameraPos().z() < groundz + minCameraZ())
-    {
-      // We may have tiny rounding errors here, so we set z to little bigger
-      //  number than the limit in order to avoid infinite loop
-      //  (otherwise we may have  groundz + minCameraZ() = 0.0000001, which will
-      //  be rounded to 0 when calling setZ(), and then setZ() sets z to 0 and
-      //  this function compares values and finds out that 0 < 0.0000001, so it
-      //  calls setZ() ... )
-      setZ(groundz + minCameraZ() + CAMERA_Z_ROUNDING_ERROR);
-    }
-    else if(cameraPos().z() > groundz + maxCameraZ())
-    {
-      setZ(groundz + maxCameraZ() - CAMERA_Z_ROUNDING_ERROR);
-    }
-#endif
   }
 }
 
@@ -497,31 +463,23 @@ void BoGameCamera::checkRotation()
   }
 }
 
-void BoGameCamera::checkRadius()
+void BoGameCamera::checkXRotation()
 {
   if(mFree || !mLimits)
   {
     // No restrictions
     return;
   }
-  // Make sure radius is within limits
-  if(radius() < 0.0f)
-  {
-    setRadius(0.0f);
-  }
-  else if(radius() * MIN_CAMERA_ANGLE_TAN > z() && z() > 0)
-  {
-    setRadius(z() / MIN_CAMERA_ANGLE_TAN);
-  }
+  mXRotation = QMAX(0, QMIN(CAMERA_MAX_XROTATION, mXRotation));
 }
 
+
 bool BoGameCamera::saveAsXML(QDomElement& root)
 {
  BoCamera::saveAsXML(root);
- root.setAttribute("PosZ", z());
+ root.setAttribute("Distance", distance());
  root.setAttribute("Rotation", rotation());
- root.setAttribute("Radius", radius());
- // TODO: save diffs too?
+ root.setAttribute("XRotation", xRotation());
  return true;
 }
 
@@ -530,31 +488,39 @@ bool BoGameCamera::loadFromXML(const QDomElement& root)
   bool ret = BoCamera::loadFromXML(root);
 
   bool ok;
-  mPosZ = root.attribute("PosZ").toFloat(&ok);
-  if(!ok)
-  {
-    boError(230) << k_funcinfo << "Invalid value for PosZ tag" << endl;
-    mPosZ = 0.0f;
-  }
-  float rotation = root.attribute("Rotation").toFloat(&ok);
+  mRotation = root.attribute("Rotation").toFloat(&ok);
   if(!ok)
   {
     boError(230) << k_funcinfo << "Invalid value for Rotation tag" << endl;
-    rotation = 0.0f;
+    mRotation = 0.0f;
   }
-  mRotation = rotation;
-  float radius = root.attribute("Radius").toFloat(&ok);
-  if(!ok)
+  if(root.hasAttribute("Distance"))
   {
-    boError(230) << k_funcinfo << "Invalid value for Radius tag" << endl;
-    radius = 0.0f;
+    mDistance = root.attribute("Distance").toFloat(&ok);
+    if(!ok)
+    {
+      boError(230) << k_funcinfo << "Invalid value for Distance tag" << endl;
+      mDistance = 8.0f;
+    }
+    mXRotation = root.attribute("XRotation").toFloat(&ok);
+    if(!ok)
+    {
+      boError(230) << k_funcinfo << "Invalid value for XRotation tag" << endl;
+      mXRotation = 30.0f;
+    }
   }
-  mRadius = radius;
+  else
+  {
+    // Revert to defaults
+    mDistance = 8.0f;
+    mXRotation = 30.0f;
+  }
+
   setPositionDirty();
   return ret;
 }
 
-const BoVector3& BoGameCamera::cameraPos()
+const BoVector3Float& BoGameCamera::cameraPos()
 {
   if(positionDirty())
   {
@@ -563,7 +529,7 @@ const BoVector3& BoGameCamera::cameraPos()
   return BoCamera::cameraPos();
 }
 
-const BoVector3& BoGameCamera::up()
+const BoVector3Float& BoGameCamera::up()
 {
   if(positionDirty())
   {
@@ -572,42 +538,14 @@ const BoVector3& BoGameCamera::up()
   return BoCamera::up();
 }
 
-void BoGameCamera::changeZ(GLfloat diff)
+void BoGameCamera::changeDistance(GLfloat diff)
 {
-  // Calculate new z
-  float oldz = mPosZ;
-  float newz = oldz + diff;
-  float factor = newz / oldz;
-
-  // Set new z coordinate (also validates camera's position (i.e. makes sure
-  //  it's not inside the terrain))
-  setZ(newz);
-
-  if(mPosZ != newz)
-  {
-    if(mPosZ == oldz)
-    {
-      // Special case - z coordinate didn't change. Probably camera is at it's
-      //  max/min height. Return.
-      return;
-    }
-    // checkCameraPosition changed z coordinate (it was set to too low/high).
-    // recalculate z change factor
-    factor = mPosZ / oldz;
-  }
-
-  // Change radius too, so that camera angle will remain same
-  // TODO: maybe provide another method for that, e.g. changeZWithRadius().
-  //  Then changeZ() would change _only_ z
-  setRadius(mRadius * factor);
+  setDistance(mDistance + diff);
 }
 
-void BoGameCamera::changeRadius(GLfloat diff)
+void BoGameCamera::changeXRotation(GLfloat diff)
 {
-  // How much radius is changed depends on z position
-  float radius = this->radius() + mPosZ / CAMERA_MAX_RADIUS * diff;
-  // Update radius
-  setRadius(radius);
+  setXRotation(mXRotation + diff);
 }
 
 void BoGameCamera::changeRotation(GLfloat diff)
@@ -615,75 +553,49 @@ void BoGameCamera::changeRotation(GLfloat diff)
   setRotation(rotation() + diff);
 }
 
-void BoGameCamera::setLookAt(const BoVector3& pos)
+void BoGameCamera::setLookAt(const BoVector3Float& pos)
 {
-#ifdef NEW_Z_CALCULATIONS
+  BO_CHECK_NULL_RET(mCanvas);
   // We must set lookAt point in 2 passes: first we set it using setLookAt(),
   //  then we validate it (this may change lookAt) and finally we calculate
   //  ground height (z) at validated lookAt point.
   BoCamera::setLookAt(pos);
-  // Validate lookat point (this may change lookat)
-  checkLookAtPosition();
-
-  // Calculate new z
-  float groundz = mCanvas->heightAtPoint(lookAt().x() * BO_TILE_SIZE, -lookAt().y() * BO_TILE_SIZE);
-  // Set lookat again. Note that this time, we don't do any validation, because
-  //  only z-coordinate changed.
-  BoCamera::setLookAt(BoVector3(lookAt().x(), lookAt().y(), groundz));
-
-  // Set the z value (height from ground) to new value, so that
-  //  mWantedAbsoluteZ remains same (this code is from changeZ())
-  float oldz = mPosZ;
-  float newz = mWantedAbsoluteZ - lookAt().z();
-  float zfactor = newz / oldz;
-  mPosZ = newz;
-
-  // Validate lookat point and camera's position
-  checkCameraPosition();
-
-  // We must also update radius to keep camera's angle constant
-  if(mPosZ != newz)
+  if(mFree || !mLimits)
   {
-    if(mPosZ == oldz)
-    {
-      // Special case - z coordinate didn't change. Probably camera is at it's
-      //  max/min height. Return.
-      return;
-    }
-    // checkCameraPosition changed z coordinate (it was set to too low/high).
-    // recalculate z change factor
-    zfactor = mPosZ / oldz;
+    // No restrictions
+    return;
   }
 
-  // Change radius too, so that camera angle will remain same
-  setRadius(mRadius * zfactor);
-#else
-  BoCamera::setLookAt(pos);
-
-  // Validate lookat point and camera's position
+  // Validate lookat point (this may change lookat)
   checkLookAtPosition();
+  // Calculate new z
+  float groundz = mCanvas->heightAtPoint(lookAt().x(), -lookAt().y());
+  // Set lookat again. Note that this time, we don't do any validation, because
+  //  only z-coordinate changed.
+  BoCamera::setLookAt(BoVector3Float(lookAt().x(), lookAt().y(), groundz + CAMERA_LOOKAT_HEIGHT));
   checkCameraPosition();
-#endif
+  setCameraChanged(true);
 }
 
-void BoGameCamera::setCameraPos(const BoVector3& pos)
+void BoGameCamera::setCameraPos(const BoVector3Float& pos)
 {
   BoCamera::setCameraPos(pos);
 
   // Validate position
   checkCameraPosition();
+  setCameraChanged(true);
 }
 
-void BoGameCamera::setRadius(GLfloat r)
+void BoGameCamera::setXRotation(GLfloat r)
 {
-  mRadius = r;
+  mXRotation = QMAX(0, QMIN(CAMERA_MAX_XROTATION, r));
   setPositionDirty();
 
-  // Make sure new radius is valid
-  checkRadius();
+  checkXRotation();
 
   // Validate camera's position
   checkCameraPosition();
+  setCameraChanged(true);
 }
 
 void BoGameCamera::setRotation(GLfloat r)
@@ -696,16 +608,19 @@ void BoGameCamera::setRotation(GLfloat r)
 
   // Validates camera's position
   checkCameraPosition();
+  setCameraChanged(true);
 }
 
-void BoGameCamera::setZ(GLfloat z)
+void BoGameCamera::setDistance(GLfloat dist)
 {
-  mPosZ = z;
-  mWantedAbsoluteZ = lookAt().z() + z;
+  mDistance = QMIN(CAMERA_MAX_DISTANCE, QMAX(CAMERA_MIN_DISTANCE, dist));
+  mActualDistance = mDistance;
+
   setPositionDirty();
 
   // Validate camera pos
   checkCameraPosition();
+  setCameraChanged(true);
 }
 
 void BoGameCamera::setFreeMovement(bool free)
@@ -726,6 +641,7 @@ void BoGameCamera::setFreeMovement(bool free)
     //  to higher values than allowed, and when camera comes out of free mode,
     //  those values won't be checked for.
     updateCamera();
+    setCameraChanged(true);
   }
 }
 
@@ -741,9 +657,11 @@ void BoGameCamera::setUseLimits(bool use)
   if(mLimits)
   {
     // Check for limits
-    checkRadius();
+    checkRotation();
+    checkXRotation();
     checkLookAtPosition();
     checkCameraPosition();
+    setCameraChanged(true);
   }
 }
 
@@ -768,12 +686,12 @@ BoLightCamera::~BoLightCamera()
 {
 }
 
-void BoLightCamera::setLightPos(const BoVector3& pos)
+void BoLightCamera::setLightPos(const BoVector3Float& pos)
 {
-  setGluLookAt(pos, BoVector3(), BoVector3());
+  setGluLookAt(pos, BoVector3Float(), BoVector3Float());
 }
 
-void BoLightCamera::setGluLookAt(const BoVector3& c, const BoVector3& l, const BoVector3& u)
+void BoLightCamera::setGluLookAt(const BoVector3Float& c, const BoVector3Float& l, const BoVector3Float& u)
 {
   BoCamera::setGluLookAt(c, l, u);
   if(!mContext)
