@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2003 Andreas Beckermann (b_mann@gmx.de)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,6 +19,8 @@
 
 #include "boitemlisthandler.h"
 #include "boitemlisthandler.moc"
+
+#include "../bomemory/bodummymemory.h"
 #include "boitemlist.h"
 #include "boglobal.h"
 #include "bodebug.h"
@@ -53,6 +55,7 @@ BoItemListHandler::BoItemListHandler() : QObject(0)
 
 BoItemListHandler::~BoItemListHandler()
 {
+ slotDeleteLists();
  delete d;
 }
 
