@@ -1,6 +1,8 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 1999-2000,2001-2004 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 1999-2000 Thomas Capricelli (capricel@email.enst.fr)
+    Copyright (C) 2001-2005 Andreas Beckermann (b_mann@gmx.de)
+    Copyright (C) 2001-2005 Rivo Laks (rivolaks@hot.ee)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,8 +22,10 @@
 #define BOSON_H
 
 #include <kgame/kgame.h>
+#include <sys/time.h>
 
 class Player;
+class PlayerIO;
 class Unit;
 class ProductionPlugin;
 class BosonCanvas;
@@ -31,8 +35,17 @@ class QDomDocument;
 class QDataStream;
 class QTextStream;
 class BosonSaveLoad;
-class BosonStarting;
 class BoMessage;
+class BoEvent;
+class BoEventManager;
+class BoAdvanceMessageTimes;
+class BoDebugMessage;
+class bofixed;
+template<class T> class BoVector2;
+template<class T> class BoRect;
+typedef BoVector2<bofixed> BoVector2Fixed;
+typedef BoRect<bofixed> BoRectFixed;
+class BosonMessageEditorMove;
 
 #define boGame Boson::boson()
 
@@ -99,16 +112,8 @@ public:
 	void setPlayField(BosonPlayField*);
 	BosonPlayField* playField() const;
 
-	/**
-	 * Set the object that will get used to start the game. Some starting
-	 * relevant network messages could get forwarded directly to this object
-	 * and we will store some data from network messages there.
-	 *
-	 * Note that a NULL @p starting parameter will unset the object and
-	 * therefore disable game starting. This can be done once the game has
-	 * been started, in order to avoid starting it twice.
-	 **/
-	void setStartingObject(BosonStarting* starting);
+	PlayerIO* findPlayerIO(Q_UINT32 id) const;
+	PlayerIO* playerIOAt(unsigned int playerIndex) const;
 
 	/**
 	 * Initialize a @ref BosonSaveLoad object with the relevant data.
@@ -117,6 +122,8 @@ public:
 
 	void quitGame();
 
+	static int advanceMessageInterval();
+
 	int gameSpeed() const;
 	bool gamePaused() const;
 
@@ -164,10 +171,9 @@ public:
 	 * @param factory Where the unit is being build
 	 * @param unitType which type of unit being buid. See @ref
 	 * UnitProperties::typeId
-	 * @param x The x-coordinate of the new unit.
-	 * @param y The y-coordinate of the new unit.
+	 * @param pos The position of the new unit.
 	 **/
-	bool buildProducedUnit(ProductionPlugin* factory, unsigned long int unitType, int x, int y);
+	bool buildProducedUnit(ProductionPlugin* factory, unsigned long int unitType, BoVector2Fixed pos);
 
 	virtual void networkTransmission(QDataStream&, int msgid, Q_UINT32 receiver, Q_UINT32 sender, Q_UINT32 clientID);
 
@@ -200,7 +206,7 @@ public:
 	 * for editor mode.
 	 **/
 	void setGameMode(bool isGame = true) { mGameMode = isGame; }
-	
+
 	/**
 	 * Note: not yet fully supported!
 	 * @return TRUE if we are in game mode or FALSE if we are in editor mode
@@ -216,6 +222,9 @@ public:
 	virtual bool savegame(QDataStream& stream, bool network, bool saveplayers = true);
 	virtual bool loadgame(QDataStream& stream, bool network, bool reset);
 
+	virtual bool event(QEvent* event);
+	virtual bool eventFilter(QObject* o, QEvent* event);
+
 	/**
 	 * @return See @ref BosonSaveLoad::LoadingStatus
 	 **/
@@ -268,13 +277,19 @@ public:
 	void writeGameLog(QTextStream& log);
 	bool saveGameLogs(const QString& prefix);
 
+	void syncNetwork();
+	void clearDelayedMessages();
+	void forcePauseGame();
 
-	/**
-	 * Start the scenario. This must not be called anymore, once a game is
-	 * started, therefore it is allowed from a @ref BosonStarting object
-	 * only.
-	 **/
-	void startScenario(BosonStarting*);
+	void queueEvent(BoEvent* event);
+
+	BoEventManager* eventManager() const;
+
+	bool loadCanvasConditions(const QDomElement& root);
+	bool saveCanvasConditions(QDomElement& root) const;
+
+	// for debugging
+	const QPtrList<BoAdvanceMessageTimes>& advanceMessageTimes() const;
 
 public: // small KGame extenstions for boson
 	/**
@@ -339,12 +354,7 @@ public slots:
 	void slotSetGameSpeed(int speed);
 	void slotTogglePause();
 
-	void slotAdvanceComputerPlayers(unsigned int advanceCount, bool advanceFlag);
-
-	/**
-	 * Emits @ref signalUpdateProductionOptions
-	 **/
-	void slotUpdateProductionOptions();
+	void slotAdvanceComputerPlayers(unsigned int advanceCallsCount, bool advanceFlag);
 
 	/**
 	 * Add a system message for the local player only. Note that this does
@@ -387,15 +397,32 @@ signals:
 	 **/
 	void signalStartNewGame();
 
+	/**
+	 * Emitted when the game is about to be started. The @p data contains
+	 * all data necessary for starting the game and should be taken by a
+	 * slot in @ref BosonStarting.
+	 *
+	 * The slot should set @p taken to TRUE, to let this object know that
+	 * the data has been received.
+	 **/
+	void signalSetNewGameData(const QByteArray& data, bool* taken);
+
+	/**
+	 * Emitted when a client has sent a message that he completed starting
+	 * (i.e. data loading). The game (i.e. advance messages) should start
+	 * when all clients completed starting.
+	 **/
+	void signalStartingCompletedReceived(const QByteArray& message, Q_UINT32 sender);
+
 	/**
 	 * Order the canvas to call @ref BosonCanvas::slotAdvance
-	 * @param advanceCount The number of this advance call. This is used to
+	 * @param advanceCallsCount The number of this advance call. This is used to
 	 * decide what should be done - e.g. there is no need to check for new
 	 * enemies every advance call. This value is increased after the signal
-	 * was emitted and reset to 0 when a certain value is reached.
+	 * was emitted
 	 * @param advanceFlag see @ref advanceFlag
 	 **/
-	void signalAdvance(unsigned int advanceCount, bool advanceFlag);
+	void signalAdvance(unsigned int advanceCallsCount, bool advanceFlag);
 
 	/**
 	 * Emitted when the editor is meant to change the map.
@@ -428,20 +455,12 @@ signals:
 	void signalGameStarted();
 
 	/**
-	 * This signal is emitted whenever the production list of factory
-	 * changes. This means whenever you add a unit to the queue (see @ref
-	 * Factory::productionList), pause a production (using @ref
-	 * Unit::setWork(Unit::WorkNone)) or continue a production
+	 * Emitted when the game is officially over and the winners are known.
 	 *
-	 * This is also emitted once a production has been completed.
+	 * When this signal is emitted, no more advance calls will be processed.
+	 * You most likely want to end the game now, e.g. call @ref quitGame
 	 **/
-	void signalUpdateProduction(Unit* factory);
-
-	/**
-	 * Emitted when unit's construction or when technology research has been
-	 * completed to ensure that new production options become available.
-	 **/
-	void signalUpdateProductionOptions();
+	void signalGameOver();
 
 	/**
 	 * Tell the map to change @ref BosonMap::texMap at coordinates @p x, @p
@@ -450,11 +469,10 @@ signals:
 	 * This is for editor use only!
 	 **/
 	void signalChangeTexMap(int x, int y, unsigned int textureCount, unsigned int* textures, unsigned char* alpha);
+	void signalChangeHeight(int x, int y, float height);
 
 	void signalAddChatSystemMessage(const QString& fromName, const QString& text, const Player* forPlayer);
 
-	void signalLoadExternalStuff(QDataStream& stream);
-	void signalSaveExternalStuff(QDataStream& stream);
 	void signalLoadExternalStuffFromXML(const QDomElement& root);
 	void signalSaveExternalStuffAsXML(QDomElement& root);
 
@@ -470,23 +488,24 @@ signals:
 	 **/
 	void signalPlayerKilled(Player*);
 
+	void signalEditorClearUndoStack();
+	void signalEditorClearRedoStack();
+	void signalEditorNewUndoMessage(const BosonMessageEditorMove&, bool fromRedo);
+	void signalEditorNewRedoMessage(const BosonMessageEditorMove&);
+
 protected:
 	virtual bool playerInput(QDataStream& stream, KPlayer* player);
 
-	/**
-	 * Load the XML file in @p xml into @p doc and display an error message
-	 * if an error occured.
-	 * @return TRUE on success
-	 **/
-	bool loadXMLDoc(QDomDocument* doc, const QString& xml);
-
 	/**
 	 * Create a game log (see @ref writeGameLog) and store it for later use (see
 	 * @ref saveGameLogs).
 	 **/
 	void makeGameLog();
+	void makeUnitLog();
 
-	bool loadFromLogFile();
+	bool loadFromLogFile(const QString& file);
+
+	void clearUndoStacks();
 
 protected slots:
 	/**
@@ -497,6 +516,8 @@ protected slots:
 	 **/
 	void slotNetworkData(int msgid, const QByteArray& buffer, Q_UINT32 receiver, Q_UINT32 sender);
 
+	void slotClientLeftGame(int clientId, int oldgamestatus, KGame*);
+
 	/**
 	 * Send an advance message. When this is received by the clients @ref
 	 * BosonCanvasCanvas::slotAdvance ist called.
@@ -514,7 +535,15 @@ protected slots:
 
 	void slotReceiveAdvance();
 
-	void slotDebugOutput(const QString& area, const char* data, int level);
+	void slotBoDebugOutput(const BoDebugMessage&);
+	void slotBoDebugWarning(const BoDebugMessage&);
+	void slotBoDebugError(const BoDebugMessage&);
+
+	/**
+	 * Called when the winning conditions are fullfilled. See @ref
+	 * BosonCanvas::signalGameOver.
+	 **/
+	void slotGameOver();
 
 private:
 	friend class BoAdvance;
@@ -527,5 +556,20 @@ private:
 	static Boson* mBoson;
 };
 
+struct timeval;
+class BoAdvanceMessageTimes
+{
+public:
+	BoAdvanceMessageTimes(int gameSpeed);
+	~BoAdvanceMessageTimes();
+
+	void receiveAdvanceCall();
+
+	struct timeval mAdvanceMessage;
+	struct timeval* mAdvanceCalls;
+	int mGameSpeed;
+	int mCurrentCall;
+};
+
 #endif
 
