@@ -1,6 +1,7 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 1999-2000,2001-2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 1999-2000 Thomas Capricelli (capricel@email.enst.fr)
+    Copyright (C) 2001-2005 Andreas Beckermann (b_mann@gmx.de)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,10 +18,13 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 #include "bosonconfig.h"
+
 #include "defines.h"
+#include "../bomemory/bodummymemory.h"
 #include "boglobal.h"
 #include "bodebug.h"
 #include "bo3dtools.h"
+#include <bogl.h>
 
 #include <qptrlist.h>
 #include <qdict.h>
@@ -29,7 +33,6 @@
 #include <kapplication.h>
 #include <klocale.h>
 
-#include <GL/gl.h>
 #include <stdlib.h>
 
 class BoGlobalConfigObject : public BoGlobalObject<BosonConfig>
@@ -65,9 +68,10 @@ private:
 static BoGlobalConfigObject globalConfig;
 
 
-BoConfigEntry::BoConfigEntry(BosonConfig* parent, const QString& key)
+BoConfigEntry::BoConfigEntry(BosonConfig* parent, const QString& key, bool saveConfig)
 {
  mKey = key;
+ mSaveConfig = saveConfig;
  parent->addConfigEntry(this);
 }
 
@@ -80,9 +84,10 @@ void BoConfigEntry::activate(KConfig* conf)
  conf->setGroup("Boson");
 }
 
-BoConfigBoolEntry::BoConfigBoolEntry(BosonConfig* parent, const QString& key, bool defaultValue)
-		: BoConfigEntry(parent, key)
+BoConfigBoolEntry::BoConfigBoolEntry(BosonConfig* parent, const QString& key, bool defaultValue, bool saveConfig)
+		: BoConfigEntry(parent, key, saveConfig)
 {
+ mDefaultValue = defaultValue;
  mValue = defaultValue;
 }
 
@@ -98,9 +103,10 @@ void BoConfigBoolEntry::load(KConfig* conf)
  mValue = conf->readBoolEntry(key(), mValue);
 }
 
-BoConfigIntEntry::BoConfigIntEntry(BosonConfig* parent, const QString& key, int defaultValue)
-		: BoConfigEntry(parent, key)
+BoConfigIntEntry::BoConfigIntEntry(BosonConfig* parent, const QString& key, int defaultValue, bool saveConfig)
+		: BoConfigEntry(parent, key, saveConfig)
 {
+ mDefaultValue = defaultValue;
  mValue = defaultValue;
 }
 
@@ -116,9 +122,10 @@ void BoConfigIntEntry::load(KConfig* conf)
  mValue = conf->readNumEntry(key(), mValue);
 }
 
-BoConfigUIntEntry::BoConfigUIntEntry(BosonConfig* parent, const QString& key, unsigned int defaultValue)
-		: BoConfigEntry(parent, key)
+BoConfigUIntEntry::BoConfigUIntEntry(BosonConfig* parent, const QString& key, unsigned int defaultValue, bool saveConfig)
+		: BoConfigEntry(parent, key, saveConfig)
 {
+ mDefaultValue = defaultValue;
  mValue = defaultValue;
 }
 
@@ -134,9 +141,10 @@ void BoConfigUIntEntry::load(KConfig* conf)
  mValue = conf->readUnsignedNumEntry(key(), mValue);
 }
 
-BoConfigDoubleEntry::BoConfigDoubleEntry(BosonConfig* parent, const QString& key, double defaultValue)
-		: BoConfigEntry(parent, key)
+BoConfigDoubleEntry::BoConfigDoubleEntry(BosonConfig* parent, const QString& key, double defaultValue, bool saveConfig)
+		: BoConfigEntry(parent, key, saveConfig)
 {
+ mDefaultValue = defaultValue;
  mValue = defaultValue;
 }
 
@@ -152,9 +160,10 @@ void BoConfigDoubleEntry::load(KConfig* conf)
  mValue = conf->readDoubleNumEntry(key(), mValue);
 }
 
-BoConfigStringEntry::BoConfigStringEntry(BosonConfig* parent, const QString& key, QString defaultValue)
-		: BoConfigEntry(parent, key)
+BoConfigStringEntry::BoConfigStringEntry(BosonConfig* parent, const QString& key, QString defaultValue, bool saveConfig)
+		: BoConfigEntry(parent, key, saveConfig)
 {
+ mDefaultValue = defaultValue;
  mValue = defaultValue;
 }
 
@@ -173,8 +182,10 @@ void BoConfigStringEntry::load(KConfig* conf)
 class BoConfigIntListEntry : public BoConfigEntry
 {
 public:
-	BoConfigIntListEntry(BosonConfig* parent, const QString& key, QValueList<int> defaultValue) : BoConfigEntry(parent, key)
+	BoConfigIntListEntry(BosonConfig* parent, const QString& key, QValueList<int> defaultValue, bool saveConfig = true)
+		: BoConfigEntry(parent, key, saveConfig)
 	{
+		mDefaultValue = defaultValue;
 		mValue = defaultValue;
 	}
 	~BoConfigIntListEntry() {}
@@ -194,6 +205,7 @@ public:
 
 	void setValue(QValueList<int> list) { mValue = list; }
 	QValueList<int> value() const { return mValue; }
+	QValueList<int> defaultValue() const { return mDefaultValue; }
 
 	void append(int e)
 	{
@@ -206,11 +218,13 @@ public:
 
 private:
 	QValueList<int> mValue;
+	QValueList<int> mDefaultValue;
 };
 
-BoConfigColorEntry::BoConfigColorEntry(BosonConfig* parent, const QString& key, const QColor& defaultValue)
-		: BoConfigEntry(parent, key)
+BoConfigColorEntry::BoConfigColorEntry(BosonConfig* parent, const QString& key, const QColor& defaultValue, bool saveConfig)
+		: BoConfigEntry(parent, key, saveConfig)
 {
+ mDefaultRGBValue = defaultValue.rgb();
  mRGBValue = defaultValue.rgb();
 }
 
@@ -237,12 +251,18 @@ QColor BoConfigColorEntry::value() const
  return QColor(mRGBValue);
 }
 
+QColor BoConfigColorEntry::defaultValue() const
+{
+ return QColor(mDefaultRGBValue);
+}
+
 
 
 class BosonConfig::BosonConfigPrivate
 {
 public:
 	BosonConfigPrivate()
+		: mDynamicEntries(QDict<BoConfigEntry>(101))
 	{
 	}
 
@@ -258,59 +278,89 @@ BosonConfig::BosonConfig(KConfig* conf)
  // there
  d->mDynamicEntries.setAutoDelete(false);
 
- mSound = new BoConfigBoolEntry(this, "Sound", DEFAULT_SOUND);
- mMusic = new BoConfigBoolEntry(this, "Music", DEFAULT_MUSIC);
- mMMBMove = new BoConfigBoolEntry(this, "MMBMove", DEFAULT_USE_MMB_MOVE);
- mRMBMove = new BoConfigBoolEntry(this, "RMBMove", DEFAULT_USE_RMB_MOVE);
- mShowMenubarInGame = new BoConfigBoolEntry(this, "ShowMenubarInGame", true);
- mShowMenubarOnStartup = new BoConfigBoolEntry(this, "ShowMenubarOnStartup", false);
- mArrowKeyStep = new BoConfigUIntEntry(this, "ArrowKeyStep", DEFAULT_ARROW_SCROLL_SPEED);
- mCursorEdgeSensity = new BoConfigUIntEntry(this, "CursorEdgeSensity", DEFAULT_CURSOR_EDGE_SENSITY);
- mUpdateInterval = new BoConfigUIntEntry(this, "GLUpdateInterval", DEFAULT_UPDATE_INTERVAL);
- mMiniMapScale = new BoConfigDoubleEntry(this, "MiniMapScale", DEFAULT_MINIMAP_SCALE);
- mMiniMapZoom = new BoConfigDoubleEntry(this, "MiniMapZoom", DEFAULT_MINIMAP_ZOOM);
- mChatScreenRemoveTime = new BoConfigUIntEntry(this, "ChatScreenRemoveTime", DEFAULT_CHAT_SCREEN_REMOVE_TIME);
- mChatScreenMaxItems = new BoConfigIntEntry(this, "ChatScreenMaxItems", DEFAULT_CHAT_SCREEN_MAX_ITEMS);
- mModelTexturesMipmaps = new BoConfigBoolEntry(this, "ModelTexturesMipmaps", DEFAULT_USE_MIPMAPS_FOR_MODELS);
- mUnitSoundsDeactivated = new BoConfigIntListEntry(this, "DeactivateUnitSounds", QValueList<int>());
- mMagnificationFilter = new BoConfigIntEntry(this, "MagnificationFilter", DEFAULT_MAGNIFICATION_FILTER);
- mMinificationFilter = new BoConfigIntEntry(this, "MinificationFilter", DEFAULT_MINIFICATION_FILTER);
- mMipmapMinificationFilter = new BoConfigIntEntry(this, "MipmapMinificationFilter", DEFAULT_MIPMAP_MINIFICATION_FILTER);
- mAlignSelectionBoxes = new BoConfigBoolEntry(this, "AlignSelectionBoxes", DEFAULT_ALIGN_SELECTION_BOXES);
- mRMBMovesWithAttack = new BoConfigBoolEntry(this, "RMBMovesWithAttack", DEFAULT_RMB_MOVES_WITH_ATTACK);
- mMouseWheelAction = new BoConfigIntEntry(this, "MouseWheelAction", DEFAULT_MOUSE_WHEEL_ACTION);
- mMouseWheelShiftAction = new BoConfigIntEntry(this, "MouseWheelShiftAction", DEFAULT_MOUSE_WHEEL_SHIFT_ACTION);
- mDeactivateWeaponSounds = new BoConfigBoolEntry(this, "DeactivateWeaponSounds", DEFAULT_DEACTIVATE_WEAPON_SOUNDS);
- mUseLight = new BoConfigBoolEntry(this, "UseLight", DEFAULT_USE_LIGHT);
- mUseMaterials = new BoConfigBoolEntry(this, "UseMaterials", DEFAULT_USE_MATERIALS);
- mCursorMode = new BoConfigIntEntry(this, "CursorMode", (int)DEFAULT_CURSOR);
- mCursorDir = new BoConfigStringEntry(this, "CursorDir", DEFAULT_CURSOR_DIR);
- mToolTipUpdatePeriod = new BoConfigIntEntry(this, "ToolTipUpdatePeriod", DEFAULT_TOOLTIP_UPDATE_PERIOD);
- mToolTipCreator = new BoConfigIntEntry(this, "ToolTipCreator", DEFAULT_TOOLTIP_CREATOR);
- mGameLogInterval = new BoConfigIntEntry(this, "GameLogInterval", 10);
- mUseLOD = new BoConfigBoolEntry(this, "UseLOD", DEFAULT_USE_LOD);
 
- mDebugMode = DebugNormal;
+ addDynamicEntryBool("Sound", false);
+ addDynamicEntryBool("Music", true);
+ addDynamicEntryBool("MMBMove", DEFAULT_USE_MMB_MOVE);
+ addDynamicEntryBool("RMBMove", DEFAULT_USE_RMB_MOVE);
+ addDynamicEntryUInt("ArrowKeyStep", DEFAULT_ARROW_SCROLL_SPEED);
+ addDynamicEntryUInt("CursorEdgeSensity", DEFAULT_CURSOR_EDGE_SENSITY);
+ addDynamicEntryUInt("GLUpdateInterval", DEFAULT_UPDATE_INTERVAL);
+ addDynamicEntryDouble("MiniMapScale", DEFAULT_MINIMAP_SCALE);
+ addDynamicEntryDouble("MiniMapZoom", DEFAULT_MINIMAP_ZOOM);
+ addDynamicEntryUInt("ChatScreenRemoveTime", DEFAULT_CHAT_SCREEN_REMOVE_TIME);
+ addDynamicEntryInt("ChatScreenMaxItems", DEFAULT_CHAT_SCREEN_MAX_ITEMS);
+ addDynamicEntryIntList("DeactivateUnitSounds", QValueList<int>());
+ addDynamicEntryBool("AlignSelectionBoxes", DEFAULT_ALIGN_SELECTION_BOXES);
+ addDynamicEntryBool("RMBMovesWithAttack", DEFAULT_RMB_MOVES_WITH_ATTACK);
+ addDynamicEntryInt("MouseWheelAction", DEFAULT_MOUSE_WHEEL_ACTION);
+ addDynamicEntryInt("MouseWheelShiftAction", DEFAULT_MOUSE_WHEEL_SHIFT_ACTION);
+ addDynamicEntryBool("DeactivateWeaponSounds", DEFAULT_DEACTIVATE_WEAPON_SOUNDS);
+ addDynamicEntryBool("UseLight", DEFAULT_USE_LIGHT);
+ addDynamicEntryBool("UseMaterials", DEFAULT_USE_MATERIALS);
+ addDynamicEntryInt("CursorMode", (int)DEFAULT_CURSOR);
+ addDynamicEntryString("CursorDir", DEFAULT_CURSOR_DIR);
+ addDynamicEntryInt("ToolTipUpdatePeriod", DEFAULT_TOOLTIP_UPDATE_PERIOD);
+ addDynamicEntryInt("ToolTipCreator", DEFAULT_TOOLTIP_CREATOR);
+ addDynamicEntryInt("GameLogInterval", 10);
+ addDynamicEntryBool("UseLOD", DEFAULT_USE_LOD);
+ addDynamicEntryBool("UseVBO", DEFAULT_USE_VBO);
+ addDynamicEntryBool("WaterShaders", DEFAULT_WATER_SHADERS);
+ addDynamicEntryBool("WaterReflections", DEFAULT_WATER_REFLECTIONS);
+ addDynamicEntryBool("WaterTranslucency", DEFAULT_WATER_TRANSLUCENCY);
+ addDynamicEntryBool("WaterBumpmapping", DEFAULT_WATER_BUMPMAPPING);
+ addDynamicEntryBool("WaterAnimatedBumpmaps", DEFAULT_WATER_ANIMATED_BUMPMAPS);
+ addDynamicEntryInt("TextureFilter", DEFAULT_TEXTURE_FILTER);
+ addDynamicEntryBool("TextureCompression", DEFAULT_TEXTURE_COMPRESSION);
+ addDynamicEntryBool("TextureColorMipmaps", false);
+ addDynamicEntryInt("TextureAnisotropy", 1);
+ addDynamicEntryUInt("MaxProfilingEntriesGL", 1000);
+ addDynamicEntryUInt("MaxProfilingEntriesAdvance", 200);
+ addDynamicEntryUInt("MaxProfilingEntries", 1000);
+ addDynamicEntryBool("UseGroundShaders", false);
 
- mDisableSound = true; // disabled by default, to make bounit and other non-sound applications work correctly, without calling setDisableSound(true) explicitly.
- mAIDelay = 3.0;
- mWantDirect = true;
- mWireFrames = false;
- mDebugOpenGLMatrices = false;
- mDebugMapCoordinates = false;
- mDebugShowCellGrid = false;
- mDebugItemWorkStatistics = false;
- mDebugOpenGLCamera = false;
- mDebugRenderCounts = false;
- mDebugBoundingBoxes = false;
- mDebugFPS = false;
- mShowResources = true;
-
- // these are dynamic entries. usually they are added in the class where they
- // get used, but sometimes it is also handy to add them here (e.g. when it
- // isn't 100% clear which class will reference it first).
- addDynamicEntry(new BoConfigUIntEntry(this, "GroundRenderer", DEFAULT_GROUND_RENDERER));
+ addDynamicEntry(new BoConfigUIntEntry(this, "GroundRenderer", 0)); // obsolete
  addDynamicEntry(new BoConfigUIntEntry(this, "DefaultLOD", 0));
+ addDynamicEntry(new BoConfigBoolEntry(this, "EnableATIDepthWorkaround", false));
+ addDynamicEntry(new BoConfigDoubleEntry(this, "ATIDepthWorkaroundValue", 0.00390625));
+ addDynamicEntry(new BoConfigStringEntry(this, "GLFont", QString::null));
+ addDynamicEntry(new BoConfigBoolEntry(this, "SmoothShading", true));
+ addDynamicEntry(new BoConfigStringEntry(this, "MeshRenderer", "BoMeshRendererVertexArray"));
+ addDynamicEntry(new BoConfigStringEntry(this, "GroundRendererClass", DEFAULT_GROUND_RENDERER));
+ addDynamicEntry(new BoConfigStringEntry(this, "GameViewPlugin", "BosonGameViewDefault"));
+ addDynamicEntry(new BoConfigBoolEntry(this, "EditorShowRandomMapGenerationWidget", false));
+ addDynamicEntry(new BoConfigBoolEntry(this, "ShowUnitDebugWidget", false));
+
+
+ // the following are NOT stored into the config file
+ addDynamicEntryBool("debug_fps", false, false);
+
+ // sound is disabled by default, to make bounit and other non-sound applications work correctly, without calling setDisableSound(true) explicitly.
+ addDynamicEntryBool("ForceDisableSound", true, false); // command line arg! do NOT save to config
+
+ addDynamicEntryInt("DebugMode", (int)DebugNormal, false);
+ addDynamicEntryDouble("AIDelay", 3.0, false);
+ addDynamicEntryBool("ForceWantDirect", true, false); // command line arg! do NOT save to config
+ addDynamicEntryBool("debug_wireframes", false, false);
+ addDynamicEntryBool("debug_matrices", false, false);
+ addDynamicEntryBool("debug_map_coordinates", false, false);
+ addDynamicEntryBool("debug_pf_data", false, false);
+ addDynamicEntryBool("debug_cell_grid", false, false);
+ addDynamicEntryBool("debug_works", false, false);
+ addDynamicEntryBool("debug_camera", false, false);
+ addDynamicEntryBool("debug_rendercounts", false, false);
+ addDynamicEntryBool("debug_boundingboxes", false, false);
+ addDynamicEntryBool("debug_advance_calls", false, false);
+ addDynamicEntryBool("debug_texture_memory", false, false);
+ addDynamicEntryBool("debug_memory_usage", false, false);
+ addDynamicEntryBool("debug_memory_vmdata_only", false, false);
+ addDynamicEntryBool("debug_cpu_usage", false, false);
+ addDynamicEntryBool("show_resources", true, false);
+ addDynamicEntryBool("debug_profiling_graph", false, false);
+ addDynamicEntryBool("ForceDisableModelLoading", false, false); // command line arg! do NOT save to config
+ addDynamicEntryBool("ForceDisableTextureCompression", false, false); // command line arg! do NOT save to config
+ addDynamicEntryBool("TextureFOW", true, false);
+ addDynamicEntryInt("DefaultLodCount", 5, false);
 
  // load from config
  reset(conf);
@@ -336,7 +386,7 @@ void BosonConfig::setPostInitFunction(void (*func)())
  globalConfig.setPostInitFunction(func);
 }
 
-BoVector3 BosonConfig::readBoVector3Entry(const KConfig* cfg, const QString& key, const BoVector3& aDefault)
+BoVector3Float BosonConfig::readBoVector3FloatEntry(const KConfig* cfg, const QString& key, const BoVector3Float& aDefault)
 {
  QValueList<float> list = BosonConfig::readFloatNumList(cfg, key);
  if (list.count() != 3) {
@@ -347,20 +397,39 @@ BoVector3 BosonConfig::readBoVector3Entry(const KConfig* cfg, const QString& key
 	}
 	return aDefault;
  }
- return BoVector3(list[0], list[1], list[2]);
+ return BoVector3Float(list[0], list[1], list[2]);
 }
 
-BoVector3 BosonConfig::readBoVector3Entry(const KConfig* cfg, const QString& key)
+BoVector3Float BosonConfig::readBoVector3FloatEntry(const KConfig* cfg, const QString& key)
 {
- return readBoVector3Entry(cfg, key, BoVector3());
+ return readBoVector3FloatEntry(cfg, key, BoVector3Float());
 }
 
-BoVector4 BosonConfig::readBoVector4Entry(const KConfig* cfg, const QString& key)
+BoVector3Fixed BosonConfig::readBoVector3FixedEntry(const KConfig* cfg, const QString& key, const BoVector3Fixed& aDefault)
 {
- return readBoVector4Entry(cfg, key, BoVector4());
+ QValueList<float> list = BosonConfig::readFloatNumList(cfg, key);
+ if (list.count() != 3) {
+	if (list.count() != 0) {
+		boError() << k_funcinfo
+				<< "BoVector3 entry must have 3 floats, not "
+				<< list.count() << endl;
+	}
+	return aDefault;
+ }
+ return BoVector3Fixed(list[0], list[1], list[2]);
 }
 
-BoVector4 BosonConfig::readBoVector4Entry(const KConfig* cfg, const QString& key, const BoVector4& aDefault)
+BoVector3Fixed BosonConfig::readBoVector3FixedEntry(const KConfig* cfg, const QString& key)
+{
+ return readBoVector3FixedEntry(cfg, key, BoVector3Fixed());
+}
+
+BoVector4Float BosonConfig::readBoVector4FloatEntry(const KConfig* cfg, const QString& key)
+{
+ return readBoVector4FloatEntry(cfg, key, BoVector4Float());
+}
+
+BoVector4Float BosonConfig::readBoVector4FloatEntry(const KConfig* cfg, const QString& key, const BoVector4Float& aDefault)
 {
  QValueList<float> list = BosonConfig::readFloatNumList(cfg, key);
  if (list.count() != 4) {
@@ -371,10 +440,29 @@ BoVector4 BosonConfig::readBoVector4Entry(const KConfig* cfg, const QString& key
 	}
 	return aDefault;
  }
- return BoVector4(list[0], list[1], list[2], list[3]);
+ return BoVector4Float(list[0], list[1], list[2], list[3]);
 }
 
-void BosonConfig::writeEntry(KConfig* cfg, const QString& key, const BoVector3& value)
+BoVector4Fixed BosonConfig::readBoVector4FixedEntry(const KConfig* cfg, const QString& key)
+{
+ return readBoVector4FixedEntry(cfg, key, BoVector4Fixed());
+}
+
+BoVector4Fixed BosonConfig::readBoVector4FixedEntry(const KConfig* cfg, const QString& key, const BoVector4Fixed& aDefault)
+{
+ QValueList<float> list = BosonConfig::readFloatNumList(cfg, key);
+ if (list.count() != 4) {
+	if (list.count() != 0) {
+		boError() << k_funcinfo
+				<< "BoVector4 entry must have 4 floats, not "
+				<< list.count() << endl;
+	}
+	return aDefault;
+ }
+ return BoVector4Fixed(list[0], list[1], list[2], list[3]);
+}
+
+void BosonConfig::writeEntry(KConfig* cfg, const QString& key, const BoVector3Float& value)
 {
   QValueList<float> list;
   list.append(value[0]);
@@ -383,7 +471,26 @@ void BosonConfig::writeEntry(KConfig* cfg, const QString& key, const BoVector3&
   BosonConfig::writeFloatNumList(list, cfg, key);
 }
 
-void BosonConfig::writeEntry(KConfig* cfg, const QString& key, const BoVector4& value)
+void BosonConfig::writeEntry(KConfig* cfg, const QString& key, const BoVector3Fixed& value)
+{
+  QValueList<float> list;
+  list.append(value[0]);
+  list.append(value[1]);
+  list.append(value[2]);
+  BosonConfig::writeFloatNumList(list, cfg, key);
+}
+
+void BosonConfig::writeEntry(KConfig* cfg, const QString& key, const BoVector4Float& value)
+{
+  QValueList<float> list;
+  list.append(value[0]);
+  list.append(value[1]);
+  list.append(value[2]);
+  list.append(value[3]);
+  BosonConfig::writeFloatNumList(list, cfg, key);
+}
+
+void BosonConfig::writeEntry(KConfig* cfg, const QString& key, const BoVector4Fixed& value)
 {
   QValueList<float> list;
   list.append(value[0]);
@@ -572,7 +679,9 @@ void BosonConfig::reset(KConfig* conf)
  // read function
  QPtrListIterator<BoConfigEntry> it(d->mConfigEntries);
  for (; it.current(); ++it) {
-	it.current()->load(conf);
+	if (it.current()->saveConfig()) {
+		it.current()->load(conf);
+	}
  }
 
  conf->setGroup(oldGroup);
@@ -588,7 +697,9 @@ void BosonConfig::save(bool /*editor*/, KConfig* conf)
  // save function
  QPtrListIterator<BoConfigEntry> it(d->mConfigEntries);
  for (; it.current(); ++it) {
-	it.current()->save(conf);
+	if (it.current()->saveConfig()) {
+		it.current()->save(conf);
+	}
  }
 
  conf->setGroup(oldGroup);
@@ -596,16 +707,18 @@ void BosonConfig::save(bool /*editor*/, KConfig* conf)
 
 void BosonConfig::setUnitSoundActivated(UnitSoundEvent e, bool activated)
 {
+ QValueList<int> l = intListValue("DeactivateUnitSounds");
  if (activated) {
-	mUnitSoundsDeactivated->remove((int)e);
+	l.remove((int)e);
  } else {
-	mUnitSoundsDeactivated->append((int)e);
+	l.append((int)e);
  }
+ setIntListValue("DeactivateUnitSounds", l);
 }
 
 bool BosonConfig::unitSoundActivated(UnitSoundEvent e) const
 {
- return !mUnitSoundsDeactivated->contains((int)e);
+ return !intListValue("DeactivateUnitSounds").contains((int)e);
 }
 
 QValueList<unsigned long int> BosonConfig::readUnsignedLongNumList(const KConfig* cfg, const QString key)
@@ -669,6 +782,55 @@ void BosonConfig::addDynamicEntry(BoConfigEntry* entry, KConfig* conf)
  entry->load(conf);
 }
 
+BoConfigBoolEntry* BosonConfig::addDynamicEntryBool(const QString& configKey, bool defaultValue, bool saveConfig)
+{
+ BoConfigBoolEntry* e = new BoConfigBoolEntry(this, configKey, defaultValue, saveConfig);
+ addDynamicEntry(e);
+ return e;
+}
+
+BoConfigIntEntry* BosonConfig::addDynamicEntryInt(const QString& configKey, int defaultValue, bool saveConfig)
+{
+ BoConfigIntEntry* e = new BoConfigIntEntry(this, configKey, defaultValue, saveConfig);
+ addDynamicEntry(e);
+ return e;
+}
+
+BoConfigUIntEntry* BosonConfig::addDynamicEntryUInt(const QString& configKey, unsigned int defaultValue, bool saveConfig)
+{
+ BoConfigUIntEntry* e = new BoConfigUIntEntry(this, configKey, defaultValue, saveConfig);
+ addDynamicEntry(e);
+ return e;
+}
+
+BoConfigDoubleEntry* BosonConfig::addDynamicEntryDouble(const QString& configKey, double defaultValue, bool saveConfig)
+{
+ BoConfigDoubleEntry* e = new BoConfigDoubleEntry(this, configKey, defaultValue, saveConfig);
+ addDynamicEntry(e);
+ return e;
+}
+
+BoConfigStringEntry* BosonConfig::addDynamicEntryString(const QString& configKey, QString defaultValue, bool saveConfig)
+{
+ BoConfigStringEntry* e = new BoConfigStringEntry(this, configKey, defaultValue, saveConfig);
+ addDynamicEntry(e);
+ return e;
+}
+
+BoConfigColorEntry* BosonConfig::addDynamicEntryColor(const QString& configKey, QColor defaultValue, bool saveConfig)
+{
+ BoConfigColorEntry* e = new BoConfigColorEntry(this, configKey, defaultValue, saveConfig);
+ addDynamicEntry(e);
+ return e;
+}
+
+BoConfigIntListEntry* BosonConfig::addDynamicEntryIntList(const QString& configKey, const QValueList<int>& defaultValue, bool saveConfig)
+{
+ BoConfigIntListEntry* e = new BoConfigIntListEntry(this, configKey, defaultValue, saveConfig);
+ addDynamicEntry(e);
+ return e;
+}
+
 bool BosonConfig::hasKey(const QString& key) const
 {
  return (d->mDynamicEntries.find(key) != 0);
@@ -745,6 +907,17 @@ void BosonConfig::setColorValue(const QString& key, const QColor& v)
  ((BoConfigColorEntry*)entry)->setValue(v);
 }
 
+void BosonConfig::setIntListValue(const QString& key, const QValueList<int>& v)
+{
+ BoConfigEntry* entry = value(key);
+ BO_CHECK_NULL_RET(entry);
+ if (entry->type() != BoConfigEntry::IntList) {
+	boError() << k_funcinfo << key << "is not an IntList entry. type=" << entry->type() << endl;
+	return;
+ }
+ ((BoConfigIntListEntry*)entry)->setValue(v);
+}
+
 bool BosonConfig::boolValue(const QString& key, bool _default) const
 {
  BoConfigEntry* entry = value(key);
@@ -834,3 +1007,130 @@ QColor BosonConfig::colorValue(const QString& key, const QColor& _default) const
  return ((BoConfigColorEntry*)entry)->value();
 }
 
+QValueList<int> BosonConfig::intListValue(const QString& key) const
+{
+ return intListValue(key, QValueList<int>());
+}
+
+QValueList<int> BosonConfig::intListValue(const QString& key, const QValueList<int>& _default) const
+{
+ BoConfigEntry* entry = value(key);
+ if (!entry) {
+	boError() << k_funcinfo << "no key " << key << endl;
+	return _default;
+ }
+ if (entry->type() != BoConfigEntry::IntList) {
+	boError() << k_funcinfo << key << "is not a IntList entry. type=" << entry->type() << endl;
+	return _default;
+ }
+ return ((BoConfigIntListEntry*)entry)->value();
+}
+
+bool BosonConfig::boolDefaultValue(const QString& key, bool _default) const
+{
+ BoConfigEntry* entry = value(key);
+ if (!entry) {
+	boError() << k_funcinfo << "no key " << key << endl;
+	return _default;
+ }
+ if (entry->type() != BoConfigEntry::Bool) {
+	boError() << k_funcinfo << key << "is not a bool entry. type=" << entry->type() << endl;
+	return _default;
+ }
+ return ((BoConfigBoolEntry*)entry)->defaultValue();
+}
+
+int BosonConfig::intDefaultValue(const QString& key, int _default) const
+{
+ BoConfigEntry* entry = value(key);
+ if (!entry) {
+	boError() << k_funcinfo << "no key " << key << endl;
+	return _default;
+ }
+ if (entry->type() != BoConfigEntry::Int) {
+	boError() << k_funcinfo << key << "is not an integer entry. type=" << entry->type() << endl;
+	return _default;
+ }
+ return ((BoConfigIntEntry*)entry)->defaultValue();
+}
+
+unsigned int BosonConfig::uintDefaultValue(const QString& key, unsigned int _default) const
+{
+ BoConfigEntry* entry = value(key);
+ if (!entry) {
+	boError() << k_funcinfo << "no key " << key << endl;
+	return _default;
+ }
+ if (entry->type() != BoConfigEntry::UInt) {
+	boError() << k_funcinfo << key << "is not an unsigned integer entry. type=" << entry->type() << endl;
+	return _default;
+ }
+ return ((BoConfigUIntEntry*)entry)->defaultValue();
+}
+
+const QString&BosonConfig::stringDefaultValue(const QString& key, const QString& _default) const
+{
+ BoConfigEntry* entry = value(key);
+ if (!entry) {
+	boError() << k_funcinfo << "no key " << key << endl;
+	return _default;
+ }
+ if (entry->type() != BoConfigEntry::String) {
+	boError() << k_funcinfo << key << "is not a string entry. type=" << entry->type() << endl;
+	return _default;
+ }
+ return ((BoConfigStringEntry*)entry)->defaultValue();
+}
+
+double BosonConfig::doubleDefaultValue(const QString& key, double _default) const
+{
+ BoConfigEntry* entry = value(key);
+ if (!entry) {
+	boError() << k_funcinfo << "no key " << key << endl;
+	return _default;
+ }
+ if (entry->type() != BoConfigEntry::Double) {
+	boError() << k_funcinfo << key << "is not a double entry. type=" << entry->type() << endl;
+	return _default;
+ }
+ return ((BoConfigDoubleEntry*)entry)->defaultValue();
+}
+
+QColor BosonConfig::colorDefaultValue(const QString& key) const
+{
+ return colorDefaultValue(key, Qt::black);
+}
+
+QColor BosonConfig::colorDefaultValue(const QString& key, const QColor& _default) const
+{
+ BoConfigEntry* entry = value(key);
+ if (!entry) {
+	boError() << k_funcinfo << "no key " << key << endl;
+	return _default;
+ }
+ if (entry->type() != BoConfigEntry::Color) {
+	boError() << k_funcinfo << key << "is not a color entry. type=" << entry->type() << endl;
+	return _default;
+ }
+ return ((BoConfigColorEntry*)entry)->defaultValue();
+}
+
+QValueList<int> BosonConfig::intListDefaultValue(const QString& key) const
+{
+ return intListDefaultValue(key, QValueList<int>());
+}
+
+QValueList<int> BosonConfig::intListDefaultValue(const QString& key, const QValueList<int>& _default) const
+{
+ BoConfigEntry* entry = value(key);
+ if (!entry) {
+	boError() << k_funcinfo << "no key " << key << endl;
+	return _default;
+ }
+ if (entry->type() != BoConfigEntry::IntList) {
+	boError() << k_funcinfo << key << "is not a IntList entry. type=" << entry->type() << endl;
+	return _default;
+ }
+ return ((BoConfigIntListEntry*)entry)->defaultValue();
+}
+
