@@ -1,6 +1,7 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 1999-2000,2001-2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 1999-2000 Thomas Capricelli (capricel@email.enst.fr)
+    Copyright (C) 2001-2005 Andreas Beckermann (b_mann@gmx.de)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,9 +26,17 @@
 class QColor;
 class KConfig;
 class BosonConfig;
-class BoVector3;
-class BoVector4;
+class bofixed;
 template<class T> class QValueList;
+template<class T> class BoVector2;
+template<class T> class BoVector3;
+template<class T> class BoVector4;
+typedef BoVector2<float> BoVector2Float;
+typedef BoVector2<bofixed> BoVector2Fixed;
+typedef BoVector3<float> BoVector3Float;
+typedef BoVector3<bofixed> BoVector3Fixed;
+typedef BoVector4<float> BoVector4Float;
+typedef BoVector4<bofixed> BoVector4Fixed;
 
 #define boConfig BosonConfig::bosonConfig()
 
@@ -44,7 +53,12 @@ public:
 		Color = 6
 	};
 public:
-	BoConfigEntry(BosonConfig* parent, const QString& key);
+	/**
+	 * @param saveConfig If TRUE (the default) this value will be saved by
+	 * @ref BosonConfig::save to the config file. If it is FALSE, the value
+	 * will not be saved (may be useful for debugging options).
+	 **/
+	BoConfigEntry(BosonConfig* parent, const QString& key, bool saveConfig = true);
 	virtual ~BoConfigEntry();
 
 	/**
@@ -63,19 +77,26 @@ public:
 
 	const QString& key() const { return mKey; }
 
+	bool saveConfig() const
+	{
+		return mSaveConfig;
+	}
+
 private:
 	QString mKey;
+	bool mSaveConfig;
 };
 
 //AB: we can't use a template because of KConfig::readEntry()
 class BoConfigBoolEntry : public BoConfigEntry
 {
 public:
-	BoConfigBoolEntry(BosonConfig* parent, const QString& key, bool defaultValue);
+	BoConfigBoolEntry(BosonConfig* parent, const QString& key, bool defaultValue, bool saveConfig = true);
 	virtual ~BoConfigBoolEntry() {}
 
 	bool value() const { return mValue; }
 	void setValue(bool v) { mValue = v; }
+	bool defaultValue() const { return mDefaultValue; }
 
 	virtual void save(KConfig* conf);
 	virtual void load(KConfig* conf);
@@ -84,16 +105,18 @@ public:
 
 private:
 	bool mValue;
+	bool mDefaultValue;
 };
 
 class BoConfigIntEntry : public BoConfigEntry
 {
 public:
-	BoConfigIntEntry(BosonConfig* parent, const QString& key, int defaultValue);
+	BoConfigIntEntry(BosonConfig* parent, const QString& key, int defaultValue, bool saveConfig = true);
 	virtual ~BoConfigIntEntry() {}
 
-	unsigned int value() const { return mValue; }
+	int value() const { return mValue; }
 	void setValue(int v) { mValue = v; }
+	int defaultValue() const { return mDefaultValue; }
 
 	virtual void save(KConfig* conf);
 	virtual void load(KConfig* conf);
@@ -102,17 +125,18 @@ public:
 
 private:
 	int mValue;
+	int mDefaultValue;
 };
 
-
 class BoConfigUIntEntry : public BoConfigEntry
 {
 public:
-	BoConfigUIntEntry(BosonConfig* parent, const QString& key, unsigned int defaultValue);
+	BoConfigUIntEntry(BosonConfig* parent, const QString& key, unsigned int defaultValue, bool saveConfig = true);
 	virtual ~BoConfigUIntEntry() {}
 
 	unsigned int value() const { return mValue; }
 	void setValue(unsigned int v) { mValue = v; }
+	unsigned int defaultValue() const { return mDefaultValue; }
 
 	virtual void save(KConfig* conf);
 	virtual void load(KConfig* conf);
@@ -120,17 +144,19 @@ public:
 	virtual int type() const { return UInt; }
 
 private:
+	unsigned int mDefaultValue;
 	unsigned int mValue;
 };
 
 class BoConfigDoubleEntry : public BoConfigEntry
 {
 public:
-	BoConfigDoubleEntry(BosonConfig* parent, const QString& key, double defaultValue);
+	BoConfigDoubleEntry(BosonConfig* parent, const QString& key, double defaultValue, bool saveConfig = true);
 	virtual ~BoConfigDoubleEntry() {}
 
 	double value() const { return mValue; }
 	void setValue(double v) { mValue = v; }
+	double defaultValue() const { return mDefaultValue; }
 
 	virtual void save(KConfig* conf);
 	virtual void load(KConfig* conf);
@@ -138,17 +164,19 @@ public:
 	virtual int type() const { return Double; }
 
 private:
+	double mDefaultValue;
 	double mValue;
 };
 
 class BoConfigStringEntry : public BoConfigEntry
 {
 public:
-	BoConfigStringEntry(BosonConfig* parent, const QString& key, QString defaultValue);
+	BoConfigStringEntry(BosonConfig* parent, const QString& key, QString defaultValue, bool saveConfig = true);
 	virtual ~BoConfigStringEntry() {}
 
 	const QString& value() const { return mValue; }
 	void setValue(const QString& v) { mValue = v; }
+	const QString& defaultValue() const { return mDefaultValue; }
 
 	virtual void save(KConfig* conf);
 	virtual void load(KConfig* conf);
@@ -156,18 +184,20 @@ public:
 	virtual int type() const { return String; }
 
 private:
+	QString mDefaultValue;
 	QString mValue;
 };
 
 class BoConfigColorEntry : public BoConfigEntry
 {
 public:
-	BoConfigColorEntry(BosonConfig* parent, const QString& key, const QColor& defaultValue);
+	BoConfigColorEntry(BosonConfig* parent, const QString& key, const QColor& defaultValue, bool saveConfig = true);
 	virtual ~BoConfigColorEntry() {}
 
 	QColor value() const;
 	void setValue(unsigned int rgb ) { mRGBValue = rgb; }
 	void setValue(const QColor& v);
+	QColor defaultValue() const;
 
 	virtual void save(KConfig* conf);
 	virtual void load(KConfig* conf);
@@ -175,6 +205,7 @@ public:
 	virtual int type() const { return Color; }
 
 private:
+	unsigned int mDefaultRGBValue;
 	unsigned int mRGBValue;
 };
 
@@ -183,7 +214,7 @@ class BoConfigIntListEntry; // forwarding, since i dont want to #include <qvalue
 /**
  * Boson has two different types of config entries, you can find both of them in
  * BosonConfig.
- * 
+ *
  * The first type is stored in this class - you can read it with the usual KDE
  * get functions (like @ref sound and @ref music). These entries can be saved to
  * the config file using @ref save. As the value is stored in BosonConfig you
@@ -232,7 +263,8 @@ public:
 	 * Read and return a @ref BoVector3. Will return @p aDefault if @p key
 	 * is not found.
 	 **/
-	static BoVector3 readBoVector3Entry(const KConfig* cfg, const QString& key, const BoVector3& aDefault);
+	static BoVector3Float readBoVector3FloatEntry(const KConfig* cfg, const QString& key, const BoVector3Float& aDefault);
+	static BoVector3Fixed readBoVector3FixedEntry(const KConfig* cfg, const QString& key, const BoVector3Fixed& aDefault);
 
 	/**
 	 * @overload
@@ -241,13 +273,18 @@ public:
 	 * default. We don't use a default value in the parameter, as we would
 	 * have to include bo3dtools.h for that.
 	 **/
-	static BoVector3 readBoVector3Entry(const KConfig* cfg, const QString& key);
+	static BoVector3Float readBoVector3FloatEntry(const KConfig* cfg, const QString& key);
+	static BoVector3Fixed readBoVector3FixedEntry(const KConfig* cfg, const QString& key);
 
-	static BoVector4 readBoVector4Entry(const KConfig* cfg, const QString& key, const BoVector4& aDefault);
-	static BoVector4 readBoVector4Entry(const KConfig* cfg, const QString& key);
+	static BoVector4Float readBoVector4FloatEntry(const KConfig* cfg, const QString& key, const BoVector4Float& aDefault);
+	static BoVector4Float readBoVector4FloatEntry(const KConfig* cfg, const QString& key);
+	static BoVector4Fixed readBoVector4FixedEntry(const KConfig* cfg, const QString& key, const BoVector4Fixed& aDefault);
+	static BoVector4Fixed readBoVector4FixedEntry(const KConfig* cfg, const QString& key);
 
-	static void writeEntry(KConfig* cfg, const QString& key, const BoVector3& value);
-	static void writeEntry(KConfig* cfg, const QString& key, const BoVector4& value);
+	static void writeEntry(KConfig* cfg, const QString& key, const BoVector3Float& value);
+	static void writeEntry(KConfig* cfg, const QString& key, const BoVector3Fixed& value);
+	static void writeEntry(KConfig* cfg, const QString& key, const BoVector4Float& value);
+	static void writeEntry(KConfig* cfg, const QString& key, const BoVector4Fixed& value);
 
 
 	static QString readLocalPlayerName(KConfig* conf = 0);
@@ -274,87 +311,9 @@ public:
 // below we have config options that are stored in this class (and saved in
 // save())
 public:
-	void setSound(bool s) { mSound->setValue(s); }
-	bool sound() const { return mSound->value(); }
-	void setMusic(bool m) { mMusic->setValue(m); }
-	bool music() const { return mMusic->value(); }
-	void setRMBMove(bool m) { mRMBMove->setValue(m); }
-	bool rmbMove() const { return mRMBMove->value(); }
-	void setMMBMove(bool m) { mMMBMove->setValue(m); }
-	bool mmbMove() const { return mMMBMove->value(); }
-	void setShowMenubarInGame(bool s) { mShowMenubarInGame->setValue(s); }
-	bool showMenubarInGame() const { return mShowMenubarInGame->value(); }
-	void setShowMenubarOnStartup(bool s) { mShowMenubarOnStartup->setValue(s); }
-	bool showMenubarOnStartup() const { return mShowMenubarOnStartup->value(); }
-	void setArrowKeyStep(unsigned int k) { mArrowKeyStep->setValue(k); }
-	unsigned int arrowKeyStep() const { return mArrowKeyStep->value(); }
-	void setUpdateInterval(unsigned int i) { mUpdateInterval->setValue(i); }
-	unsigned int updateInterval() const { return mUpdateInterval->value(); }
-	void setMiniMapScale(double s) { mMiniMapScale->setValue(s); }
-	double miniMapScale() const { return mMiniMapScale->value(); }
-	void setMiniMapZoom(double z) { mMiniMapZoom->setValue(z); }
-	double miniMapZoom() const { return mMiniMapZoom->value(); }
-	void setChatScreenRemoveTime(unsigned int s) { mChatScreenRemoveTime->setValue(s); }
-	unsigned int chatScreenRemoveTime() const { return mChatScreenRemoveTime->value(); }
-	void setChatScreenMaxItems(int max) { mChatScreenMaxItems->setValue(max); }
-	int chatScreenMaxItems() const { return mChatScreenMaxItems->value(); }
-	void setModelTexturesMipmaps(bool enable) { mModelTexturesMipmaps->setValue(enable); }
-	bool modelTexturesMipmaps() const { return mModelTexturesMipmaps->value(); }
-	int magnificationFilter() const { return mMagnificationFilter->value(); }
-	void setMagnificationFilter(int f) { mMagnificationFilter->setValue(f); }
-	int minificationFilter() const { return mMinificationFilter->value(); }
-	void setMinificationFilter(int f) { mMinificationFilter->setValue(f); }
-	int mipmapMinificationFilter() const { return mMipmapMinificationFilter->value(); }
-	void setMipmapMinificationFilter(int f) { mMipmapMinificationFilter->setValue(f); }
-	void setAlignSelectionBoxes(bool enable) { mAlignSelectionBoxes->setValue(enable); }
-	bool alignSelectionBoxes() const { return mAlignSelectionBoxes->value(); }
-	void setRMBMovesWithAttack(bool attack) { mRMBMovesWithAttack->setValue(attack); }
-	bool RMBMovesWithAttack() const { return (mRMBMovesWithAttack->value()); }
-	void setMouseWheelAction(CameraAction action) { mMouseWheelAction->setValue((int)action); }
-	CameraAction mouseWheelAction() const { return (CameraAction)(mMouseWheelAction->value()); }
-	void setMouseWheelShiftAction(CameraAction action) { mMouseWheelShiftAction->setValue((int)action); }
-	CameraAction mouseWheelShiftAction() const { return (CameraAction)(mMouseWheelShiftAction->value()); }
-	void setDeactivateWeaponSounds(bool deactivate) { mDeactivateWeaponSounds->setValue(deactivate); }
-	bool deactivateWeaponSounds() const { return mDeactivateWeaponSounds->value(); }
-	void setUseLOD(bool use) { mUseLOD->setValue(use); }
-	bool useLOD() const { return mUseLOD->value(); }
-	void setUseVBO(bool use) { mUseVBO->setValue(use); }
-	bool useVBO() const { return mUseVBO->value(); }
-
 	void setUnitSoundActivated(UnitSoundEvent e, bool activated);
 	bool unitSoundActivated(UnitSoundEvent e) const;
 
-	/**
-	 * @param m How "sensitive" the edge is. I.e. the number the cursor must
-	 * be in range of m pixels to an edge of the window. 0 to disable
-	 **/
-	void setCursorEdgeSensity(unsigned int m) { mCursorEdgeSensity->setValue(m); }
-	unsigned int cursorEdgeSensity() const { return mCursorEdgeSensity->value(); }
-
-	bool useLight() const { return mUseLight->value(); }
-	void setUseLight(bool l) const { return mUseLight->setValue(l); }
-
-	bool useMaterials() const { return mUseMaterials->value(); }
-	void setUseMaterials(bool l) const { return mUseMaterials->setValue(l); }
-
-	void setCursorMode(int mode) { mCursorMode->setValue(mode); }
-	int cursorMode() const { return mCursorMode->value(); }
-
-	void setCursorDir(const QString& dir) { mCursorDir->setValue(dir); }
-	QString cursorDir() const { return mCursorDir->value(); }
-
-	void setToolTipUpdatePeriod(int ms) { mToolTipUpdatePeriod->setValue(ms); }
-	int toolTipUpdatePeriod() const { return mToolTipUpdatePeriod->value(); }
-
-	void setToolTipCreator(int type) { mToolTipCreator->setValue(type); }
-	int toolTipCreator() const { return mToolTipCreator->value(); }
-
-	void setGameLogInterval(int interval) { mGameLogInterval->setValue(interval); }
-	int gameLogInterval() const { return mGameLogInterval->value(); }
-
-
-
-
 
 	/**
 	 * Add a dynamic entry to BosonConfig. A dynamic entry does not have a
@@ -374,6 +333,17 @@ public:
 	 **/
 	void addDynamicEntry(BoConfigEntry* entry, KConfig* conf = 0);
 
+	/**
+	 * Convenience method for @ref addDynamicEntry
+	 **/
+	BoConfigBoolEntry* addDynamicEntryBool(const QString& configKey, bool defaultValue = false, bool saveConfig = true);
+	BoConfigIntEntry* addDynamicEntryInt(const QString& configKey, int defaultValue = 0, bool saveConfig = true);
+	BoConfigUIntEntry* addDynamicEntryUInt(const QString& configKey, unsigned int defaultValue = 0, bool saveConfig = true);
+	BoConfigDoubleEntry* addDynamicEntryDouble(const QString& configKey, double defaultValue = 0.0, bool saveConfig = true);
+	BoConfigStringEntry* addDynamicEntryString(const QString& configKey, QString defaultValue = QString::null, bool saveConfig = true);
+	BoConfigColorEntry* addDynamicEntryColor(const QString& configKey, QColor defaultValue, bool saveConfig = true);
+	BoConfigIntListEntry* addDynamicEntryIntList(const QString& configKey, const QValueList<int>& defaultValue, bool saveConfig = true);
+
 	/**
 	 * @return TRUE if a dynamic entry with @p key was added with @ref
 	 * addDynamicEntry, otherwise FALSE.
@@ -417,6 +387,17 @@ public:
 	 **/
 	void setColorValue(const QString& key, const QColor& value);
 
+	/**
+	 * Same as  @ref setBoolValue, but for @ref BoConfigIntListEntry objects.
+	 **/
+	void setIntListValue(const QString& key, const QValueList<int>& v);
+
+	/**
+	 * Set the value of the config entry @p key (see @ref value) to the
+	 * default value that was given to the constructor of that config entry.
+	 **/
+	void resetValueToDefault(const QString& key);
+
 	/**
 	 * @return The (dynamic) entry for @p key, or NULL if no such key was
 	 * ever added using @ref addDynamicEntry.
@@ -465,73 +446,32 @@ public:
 	 **/
 	QColor colorValue(const QString& key, const QColor& _default) const;
 
-
-// below we have config values that are *not* stored when quitting boson
-public:
-	DebugMode debugMode() const { return mDebugMode; }
+	/**
+	 * @return The value of the entry @p key, or an empty list if no entry
+	 * of the correct type exists.
+	 **/
+	QValueList<int> intListValue(const QString& key) const;
 
 	/**
-	 * Change the debugging mode. Note that this isn't saved to the config,
-	 * so this is lost on quit.
+	 * @return The value of the entry @p key, or @p _default if no entry of
+	 * the correct type exists.
 	 **/
-	void setDebugMode(DebugMode m) { mDebugMode = m; }
+	QValueList<int> intListValue(const QString& key, const QValueList<int>& _default) const;
 
 	/**
-	 * Disable sound loading and playing. Note that this value is 
-	 * <em>not</em> saved into the config file (it is a command line arg)!
+	 * @return The default value of the entry @p key, if existing, otherwise
+	 * @p _default
 	 **/
-	void setDisableSound(bool d) { mDisableSound = d; }
+	bool boolDefaultValue(const QString& key, bool _default = false) const;
+	int intDefaultValue(const QString& key, int _default = 0) const;
+	unsigned int uintDefaultValue(const QString& key, unsigned int _default = 0) const;
+	const QString& stringDefaultValue(const QString& key, const QString& _default = QString::null) const;
+	double doubleDefaultValue(const QString& key, double _default = 0.0) const;
+	QColor colorDefaultValue(const QString& key) const;
+	QColor colorDefaultValue(const QString& key, const QColor& _default) const;
+	QValueList<int> intListDefaultValue(const QString& key) const;
+	QValueList<int> intListDefaultValue(const QString& key, const QValueList<int>& _default) const;
 
-	/**
-	 * FALSE (default) if sound files should be loaded normally, otherwise
-	 * TRUE.
-	 **/
-	bool disableSound() const { return mDisableSound; }
-
-	void setAiDelay(float delay) { mAIDelay = delay; }
-	float aiDelay() const { return mAIDelay; }
-
-	void setWantDirect(bool w) { mWantDirect = w; }
-	bool wantDirect() const { return mWantDirect; }
-
-	void setWireFrames(bool on) { mWireFrames = on; }
-	bool wireFrames() const { return mWireFrames; }
-
-	void setDebugOpenGLMatrices(bool debug) { mDebugOpenGLMatrices = debug; }
-	bool debugOpenGLMatrices() const { return mDebugOpenGLMatrices; }
-
-	void setDebugMapCoordinates(bool debug) { mDebugMapCoordinates = debug; }
-	bool debugMapCoordinates() const { return mDebugMapCoordinates; }
-
-	void setDebugPFData(bool debug) { mDebugPFData = debug; }
-	bool debugPFData() const { return mDebugPFData; }
-
-	void setDebugShowCellGrid(bool debug) { mDebugShowCellGrid = debug; }
-	bool debugShowCellGrid() const { return mDebugShowCellGrid; }
-
-	void setDebugItemWorkStatistics(bool debug) { mDebugItemWorkStatistics = debug; }
-	bool debugItemWorkStatistics() const { return mDebugItemWorkStatistics; }
-
-	void setDebugOpenGLCamera(bool debug) { mDebugOpenGLCamera = debug; }
-	bool debugOpenGLCamera() const { return mDebugOpenGLCamera; }
-
-	void setDebugRenderCounts(bool debug) { mDebugRenderCounts = debug; }
-	bool debugRenderCounts() const { return mDebugRenderCounts; }
-
-	void setDebugBoundingBoxes(bool debug) { mDebugBoundingBoxes = debug; }
-	bool debugBoundingBoxes() const { return mDebugBoundingBoxes; }
-
-	unsigned int defaultLodCount() const { return mDefaultLodCount; }
-	void setDefaultLodCount(unsigned int c) { mDefaultLodCount = c; }
-
-	void setDebugFPS(bool debug) { mDebugFPS = debug; }
-	bool debugFPS() const { return mDebugFPS; }
-
-	void setShowResources(bool show) { mShowResources = show; }
-	bool showResources() const { return mShowResources; }
-
-	void setEnableColormap(bool enable) { mEnableColormap = enable; }
-	bool enableColormap() const { return mEnableColormap; }
 
 public:
 	void save(bool editor = false, KConfig* conf = 0);
@@ -556,60 +496,6 @@ public:
 private:
 	class BosonConfigPrivate;
 	BosonConfigPrivate* d;
-
-	// note that ALL BoConfigEntry objects must be new'ed in the
-	// BosonConfig c'tor !
-	BoConfigBoolEntry* mSound;
-	BoConfigBoolEntry* mMusic;
-	BoConfigBoolEntry* mMMBMove;
-	BoConfigBoolEntry* mRMBMove;
-	BoConfigBoolEntry* mShowMenubarInGame;
-	BoConfigBoolEntry* mShowMenubarOnStartup;
-	BoConfigBoolEntry* mModelTexturesMipmaps;
-	BoConfigIntEntry* mChatScreenMaxItems;
-	BoConfigUIntEntry* mChatScreenRemoveTime;
-	BoConfigUIntEntry* mArrowKeyStep;
-	BoConfigUIntEntry* mCursorEdgeSensity;
-	BoConfigUIntEntry* mUpdateInterval;
-	BoConfigDoubleEntry* mMiniMapScale;
-	BoConfigDoubleEntry* mMiniMapZoom;
-	BoConfigIntListEntry* mUnitSoundsDeactivated;
-	BoConfigIntEntry* mMagnificationFilter;
-	BoConfigIntEntry* mMinificationFilter;
-	BoConfigIntEntry* mMipmapMinificationFilter;
-	BoConfigBoolEntry* mAlignSelectionBoxes;
-	BoConfigBoolEntry* mRMBMovesWithAttack;
-	BoConfigIntEntry* mMouseWheelAction;
-	BoConfigIntEntry* mMouseWheelShiftAction;
-	BoConfigBoolEntry* mDeactivateWeaponSounds;
-	BoConfigBoolEntry* mUseLight;
-	BoConfigBoolEntry* mUseMaterials;
-	BoConfigStringEntry* mCursorDir;
-	BoConfigIntEntry* mCursorMode;
-	BoConfigIntEntry* mToolTipUpdatePeriod;
-	BoConfigIntEntry* mToolTipCreator;
-	BoConfigIntEntry* mGameLogInterval;
-	BoConfigBoolEntry* mUseLOD;
-	BoConfigBoolEntry* mUseVBO;
-
-	// NOT stored to config file!
-	bool mDisableSound;
-	DebugMode mDebugMode;
-	float mAIDelay;
-	bool mWantDirect;
-	bool mWireFrames;
-	bool mDebugOpenGLMatrices;
-	bool mDebugMapCoordinates;
-	bool mDebugPFData;
-	bool mDebugShowCellGrid;
-	bool mDebugItemWorkStatistics;
-	bool mDebugOpenGLCamera;
-	bool mDebugRenderCounts;
-	bool mDebugBoundingBoxes;
-	bool mDebugFPS;
-	bool mShowResources;
-	bool mEnableColormap;
-	unsigned int mDefaultLodCount;
 };
 
 #endif
