@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2003-2005 Andreas Beckermann (b_mann@gmx.de)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -66,6 +66,19 @@ public:
 	bool convertPlayField_From_0_9_To_0_9_1(QMap<QString, QByteArray>& files);
 
 	bool convertPlayField_From_0_9_1_To_0_10(QMap<QString, QByteArray>& files);
+	bool convertPlayField_From_0_10_To_0_10_80(QMap<QString, QByteArray>& files);
+	bool convertPlayField_From_0_10_80_To_0_10_81(QMap<QString, QByteArray>& files);
+	bool convertPlayField_From_0_10_81_To_0_10_82(QMap<QString, QByteArray>& files);
+	bool convertPlayField_From_0_10_82_To_0_10_83(QMap<QString, QByteArray>& files);
+	bool convertPlayField_From_0_10_83_To_0_10_84(QMap<QString, QByteArray>& files);
+	bool convertPlayField_From_0_10_84_To_0_10_85(QMap<QString, QByteArray>& files);
+	bool convertPlayField_From_0_10_85_To_0_11(QMap<QString, QByteArray>& files);
+
+	bool addDummyWaterXML_From_0_10_To_0_10_80(QByteArray& waterXML);
+
+	bool addGamePyScript_From_0_10_82_To_0_10_83(QByteArray& gamePy);
+	bool addLocalPlayerPyScript_From_0_10_82_To_0_10_83(QByteArray& localplayerPy);
+	bool addAIPyScript_From_0_10_82_To_0_10_83(QByteArray& aiPy, int playerid);
 
 protected:
 	bool loadXMLDoc(QDomDocument* doc, const QString& xml) const;
