@@ -22,20 +22,20 @@
 #include "bosonglwidget.h"
 #include "bosonglwidget.moc"
 
+#include "../bomemory/bodummymemory.h"
 #include "bodebug.h"
 
 #include <kapplication.h>
 #include <kmessagebox.h>
 #include <klocale.h>
 
+#include <bogl.h>
+#include <boglx.h>
 #include <qgl.h> // convertToGLFormat()
 #include <qimage.h> // convertToGLFormat()
 #include <qpaintdevicemetrics.h>
 #include <qintdict.h>
 
-#include <GL/gl.h>
-#include <GL/glx.h>
-
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xos.h>
@@ -479,7 +479,13 @@ public:
 	bool mWantDirect;
 };
 
-BosonGLWidget::BosonGLWidget(QWidget* parent, bool direct) : QWidget(parent)
+// WNoAutoErase: Qt 3.2
+BosonGLWidget::BosonGLWidget(QWidget* parent, const char* name, bool direct)
+#if QT_VERSION >= 0x030200
+	: QWidget(parent, name, Qt::WNoAutoErase)
+#else
+	: QWidget(parent, name, Qt::WRepaintNoErase)
+#endif
 {
  boDebug() << k_funcinfo << endl;
  d = new BosonGLWidgetPrivate;
@@ -498,6 +504,7 @@ BosonGLWidget::~BosonGLWidget()
 void BosonGLWidget::init()
 {
  setBackgroundMode(NoBackground);
+ mAutoSwap = true;
  setContext(new BoContext(this));
 }
 
@@ -561,7 +568,9 @@ void BosonGLWidget::slotUpdateGL()
  paintGL();
  if (!context()->deviceIsPixmap()) {
 	// non-pixmap devices are _always_ double buffered for us!
-	swapBuffers();
+	if (autoBufferSwap()) {
+		swapBuffers();
+	}
  } else {
 	// no doublebuffering.
 	glFlush();
@@ -735,63 +744,37 @@ void BosonGLWidget::paintEvent(QPaintEvent*)
 	// tdfx driver) - dunno why.
 	int w = width();
 	int h = height();
+
+	makeCurrent();
 	resizeGL(w - 1, h - 1);
 	resizeGL(w, h);
+	resized = true;
  }
  slotUpdateGL();
 }
 
-
-
-BoMouseMoveDiff::BoMouseMoveDiff()
+// TODO: current QGLWidget has a method grabFrameBuffer() that also handles
+// endianness!
+// -> we should use that one instead of our own here!
+QImage BosonGLWidget::screenShot()
 {
- mButton = QMouseEvent::NoButton;
- mX = 0;
- mY = 0;
- mOldX = 0;
- mOldY = 0;
-}
-
-void BoMouseMoveDiff::moveEvent(QMouseEvent* e)
-{
- moveEvent(e->pos(), e->state());
-}
-
-void BoMouseMoveDiff::moveEvent(const QPoint& pos, int buttonState)
-{
- moveEvent(pos.x(), pos.y(), buttonState);
-}
-
-void BoMouseMoveDiff::moveEvent(int x, int y, int buttonState)
-{
- mOldX = mX;
- mOldY = mY;
- mX = x;
- mY = y;
-
- if (buttonState == -1) {
-	buttonState = QMouseEvent::NoButton;
- }
- if (mButton != QMouseEvent::NoButton) {
-	if (!(buttonState & mButton)) {
-		// button is not pressed anymore
-		stop();
+ glFinish();
+ int w = width();
+ int h = height();
+ unsigned char* buffer = new unsigned char[w * h * 4];
+ glReadPixels(0, 0, w, h, GL_RGBA, GL_UNSIGNED_BYTE, buffer);
+ QImage image(w, h, 32);
+ for (int y = 0; y < h; y++) {
+	QRgb* line = (QRgb*)image.scanLine(y); // AB: use setPixel() instead of scanLine() ! -> endianness must be handled
+	int opengl_y = h - y;
+	for (int x = 0; x < w; x++) {
+		unsigned char* pixel = &buffer[(opengl_y * w + x) * 4];
+		line[x] = qRgb(pixel[0], pixel[1], pixel[2]);
 	}
  }
+ delete[] buffer;
+ return image;
 }
 
-void BoMouseMoveDiff::stop()
-{
- mButton = QMouseEvent::NoButton;
-}
 
-bool BoMouseMoveDiff::isStopped() const
-{
- return button() == QMouseEvent::NoButton;
-}
-
-void BoMouseMoveDiff::start(int button)
-{
- mButton = button;
-}
 
