@@ -1,6 +1,8 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 1999-2000,2001-2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 1999-2000 Thomas Capricelli (capricel@email.enst.fr)
+    Copyright (C) 2001-2005 Andreas Beckermann (b_mann@gmx.de)
+    Copyright (C) 2001-2005 Rivo Laks (rivolaks@hot.ee)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,13 +23,16 @@
 #include "bosonmap.moc"
 
 #include "defines.h"
+#include "../bomemory/bodummymemory.h"
 #include "cell.h"
 #include "bosongroundtheme.h"
 #include "boson.h" // for an ugly boGame->gameMode() hack!
 #include "bodebug.h"
 #include "bosondata.h"
-#include "bosontexturearray.h"
+#include "bowater.h"
+#include "botexture.h"
 
+#include <qdict.h>
 #include <qdatastream.h>
 #include <qimage.h>
 #include <qdom.h>
@@ -37,22 +42,22 @@
 
 float BoHeightMap::pixelToHeight(int p)
 {
- // we have 255 different values available. we use 0.1 steps, i.e. if
- // the pixel is increased by 1 the height is increased by 0.1.
+ // we have 255 different values available. we use 0.125 steps, i.e. if
+ // the pixel is increased by 1 the height is increased by 0.125.
  // values below 105 cause a negative height, values above 105 a positive
  // height.
- // this means we have a height range from -105/10 to 150/10 aka from -10.5 to
- // 15.0
+ // this means we have a height range from -105/8 to 150/8 aka from -13.125 to
+ // 18.75
 // boDebug() << k_funcinfo << p << "->" << ((float)(p-105))/10 << endl;
- return ((float)(p - 105)) / 10;
+ return ((float)(p - 105)) / 8.0f;
 }
 
 int BoHeightMap::heightToPixel(float height)
 {
- return (int)(height * 10 + 105);
+ return ((int)(height * 8.0f)) + 105;
 }
 
-bool BoHeightMap::save(QDataStream& stream)
+bool BoHeightMap::save(QDataStream& stream) const
 {
  if (width() == 0) {
 	boError() << k_funcinfo << "width()==0" << endl;
@@ -94,7 +99,7 @@ bool BoHeightMap::load(QDataStream& stream)
 }
 
 
-bool BoTexMap::save(QDataStream& stream)
+bool BoTexMap::save(QDataStream& stream) const
 {
  boDebug() << k_funcinfo << endl;
  if (!mTexMap) {
@@ -300,12 +305,58 @@ bool BoTexMap::copyTexture(unsigned int dstTexture, const BoTexMap* src, unsigne
 }
 
 
+BoColorMap::BoColorMap(unsigned int width, unsigned int height)
+{
+ mWidth = width;
+ mHeight = height;
+
+ // Load initial black texture
+ int dataSize = mWidth * mHeight * 3;
+ mData = new unsigned char[dataSize];
+ for (int i = 0; i < dataSize; i++) {
+	mData[i] = 0;
+ }
+ mDirtyRect = QRect(0, 0, mWidth, mHeight);
+}
+
+BoColorMap::~BoColorMap()
+{
+ delete[] mData;
+}
+
+void BoColorMap::update(unsigned char* data)
+{
+ updateRect(0, 0, mWidth, mHeight, data);
+}
+
+void BoColorMap::updateRect(int _x, int _y, unsigned int w, unsigned int h, unsigned char* data)
+{
+ QRect r(_x, _y, w, h);
+ int pos = 0;
+ for (int y = r.y(); y < r.y() + r.height(); y++) {
+	for (int x = r.x(); x < r.x() + r.width(); x++) {
+		int index = (y * mWidth + x) * 3;
+		mData[index + 0] = data[pos + 0];
+		mData[index + 1] = data[pos + 1];
+		mData[index + 2] = data[pos + 2];
+		pos += 3;
+	}
+ }
+ mDirtyRect = mDirtyRect.unite(r);
+ mDirtyRect = mDirtyRect.intersect(QRect(0, 0, mWidth, mHeight));
+}
+
+
 class BosonMap::BosonMapPrivate
 {
 public:
 	BosonMapPrivate()
 	{
+		mWaterManager = 0;
 	}
+
+	QDict<BoColorMap> mColorMaps;
+	BoWaterManager* mWaterManager;
 };
 
 BosonMap::BosonMap(QObject* parent) : QObject(parent)
@@ -315,11 +366,13 @@ BosonMap::BosonMap(QObject* parent) : QObject(parent)
 
 BosonMap::~BosonMap()
 {
+ d->mColorMaps.setAutoDelete(true);
+ d->mColorMaps.clear();
  delete[] mCells;
- delete mColorMap;
  delete mNormalMap;
  delete mHeightMap;
  delete mTexMap;
+ delete d->mWaterManager;
  delete d;
 }
 
@@ -327,13 +380,14 @@ void BosonMap::init()
 {
  d = new BosonMapPrivate;
  mCells = 0;
- mColorMap = 0;
+ mActiveColorMap = 0;
  mHeightMap = 0;
  mNormalMap = 0;
  mGroundTheme = 0;
  mTexMap = 0;
  mMapWidth = 0;
  mMapHeight = 0;
+ d->mWaterManager = new BoWaterManager();
  setModified(false);
 }
 
@@ -349,10 +403,13 @@ bool BosonMap::createNewMap(unsigned int width, unsigned int height, BosonGround
  }
  mMapWidth = width;
  mMapHeight = height;
- mGroundTheme = theme;
+ if (!applyGroundTheme(theme->identifier())) {
+	boError() << k_funcinfo << "invalid groundtheme " << theme->identifier() << endl;
+	return false;
+ }
 
  mHeightMap = new BoHeightMap(width + 1, height + 1);
- mTexMap = new BoTexMap(theme->textureCount(), width + 1, height + 1);
+ mTexMap = new BoTexMap(theme->groundTypeCount(), width + 1, height + 1);
  mTexMap->fill(0);
  bool ret = generateCellsFromTexMap();
  if (!ret) {
@@ -383,6 +440,8 @@ bool BosonMap::createNewMap(unsigned int width, unsigned int height, BosonGround
 	return false;
  }
 
+ d->mWaterManager->setMap(this);
+
  QDataStream readStream(buffer, IO_ReadOnly);
  ret = loadCompleteMap(readStream);
  if (!ret) {
@@ -407,7 +466,7 @@ bool BosonMap::createNewMap(unsigned int width, unsigned int height, BosonGround
 	boError() << k_funcinfo << "invalid height loaded" << endl;
 	return false;
  }
- if (mTexMap->textureCount() != mGroundTheme->textureCount()) {
+ if (mTexMap->textureCount() != mGroundTheme->groundTypeCount()) {
 	boError() << k_funcinfo << "texmap has invalid texture count for this groundtheme!" << endl;
 	return false;
  }
@@ -473,13 +532,32 @@ bool BosonMap::loadMapFromFile(const QByteArray& mapXML)
 	boError(270) << k_funcinfo << "Could not load map geo" << endl;
 	return false;
  }
- if (!loadGroundTheme(groundTheme)) {
-	boError(270) << k_funcinfo << "Could not load the ground theme" << endl;
+ if (!applyGroundTheme(groundTheme)) {
+	boError(270) << k_funcinfo << "Could not apply the ground theme " << groundTheme << endl;
 	return false;
  }
  return true;
 }
 
+bool BosonMap::loadWaterFromFile(const QByteArray& waterXML)
+{
+ boDebug(270) << k_funcinfo << endl;
+ QDomDocument doc(QString::fromLatin1("Water"));
+ QString errorMsg;
+ int line, column;
+ if (!doc.setContent(QString(waterXML), &errorMsg, &line, &column)) {
+	boError(270) << k_funcinfo << "unable to load from waterXML (error in line " << line << ", column " << column << ", msg=" << errorMsg << ")" << endl;
+	return false;
+ }
+ QDomElement root = doc.documentElement();
+ if (root.isNull()) {
+	boError(270) << k_funcinfo << "no root element" << endl;
+	return false;
+ }
+ d->mWaterManager->setMap(this);
+ return d->mWaterManager->loadFromXML(root);
+}
+
 
 bool BosonMap::loadCompleteMap(QDataStream& stream)
 {
@@ -504,14 +582,11 @@ bool BosonMap::loadCompleteMap(QDataStream& stream)
 	boError() << k_funcinfo << "Could not load texmap" << endl;
 	return false;
  }
- if (!loadCells(stream)) {
-	boError() << k_funcinfo << "Could not load map cells" << endl;
+ createCells();
+ if (!mCells) {
+	boError() << k_funcinfo << "NULL cells" << endl;
 	return false;
  }
- // should this be done somewhere else?
- boDebug() << k_funcinfo << "creating colormap" << endl;
- createColorMap();
- boDebug() << k_funcinfo << "created colormap" << endl;
  return true;
 }
 
@@ -533,44 +608,24 @@ bool BosonMap::loadMapGeo(unsigned int width, unsigned int height)
  mHeightMap = 0;
  delete mNormalMap;
  mNormalMap = 0;
- boDebug() << k_funcinfo << "deleting colormap" << endl;
- delete mColorMap;
- mColorMap = 0;
  delete mTexMap;
  mTexMap = 0;
 
  return true;
 }
 
-bool BosonMap::loadGroundTheme(const QString& id)
+bool BosonMap::applyGroundTheme(const QString& id)
 {
- mGroundTheme = (BosonGroundTheme*)BosonData::bosonData()->groundTheme(id);
- if (!mGroundTheme) {
+ if (id.isEmpty()) {
+	boError() << k_funcinfo << "empty id string" << endl;
+	return false;
+ }
+ BosonGroundTheme* theme = (BosonGroundTheme*)BosonData::bosonData()->groundTheme(id);
+ if (!theme) {
 	boError() << k_funcinfo << "Cannot find groundTheme with id=" << id << endl;
 	return false;
  }
- emit signalGroundThemeChanged(mGroundTheme);
- return true;
-}
-
-bool BosonMap::loadCells(QDataStream& stream)
-{
- createCells();
- if (!mCells) {
-	boError() << k_funcinfo << "NULL cells" << endl;
-	return false;
- }
-// load all cells:
- for (unsigned int i = 0; i < width(); i++) {
-	for (unsigned int j = 0; j < height(); j++) {
-		unsigned char amountOfLand = 0;
-		unsigned char amountOfWater = 0;
-		if (!loadCell(stream, &amountOfLand, &amountOfWater)) {
-			return false;
-		}
-		slotChangeCell(i, j, amountOfLand, amountOfWater);
-	}
- }
+ mGroundTheme = theme;
  return true;
 }
 
@@ -702,11 +757,11 @@ bool BosonMap::loadTexMap(QDataStream& stream)
 	delete mTexMap;
 	mTexMap = 0;
  }
- mTexMap = new BoTexMap(groundTheme()->textureCount(), width() + 1, height() + 1);
+ mTexMap = new BoTexMap(groundTheme()->groundTypeCount(), width() + 1, height() + 1);
  return mTexMap->load(stream);
 }
 
-bool BosonMap::saveTexMap(QDataStream& stream)
+bool BosonMap::saveTexMap(QDataStream& stream) const
 {
  boDebug() << k_funcinfo << endl;
  if (!mTexMap) {
@@ -717,14 +772,14 @@ bool BosonMap::saveTexMap(QDataStream& stream)
 	BO_NULL_ERROR(groundTheme());
 	return false;
  }
- if (groundTheme()->textureCount() != mTexMap->textureCount()) {
-	boError() << k_funcinfo << "groundTheme()->texturCount() differs from texMap->textureCount() !" << endl;
+ if (groundTheme()->groundTypeCount() != mTexMap->textureCount()) {
+	boError() << k_funcinfo << "groundTheme()->groundTypeCount() differs from texMap->textureCount() !" << endl;
 	return false;
  }
  return mTexMap->save(stream);
 }
 
-QByteArray BosonMap::saveMapToFile()
+QByteArray BosonMap::saveMapToFile() const
 {
  if (!isValidMapGeo(width(), height())) {
 	boError() << k_funcinfo << "Map geo is not valid" << endl;
@@ -756,7 +811,18 @@ QByteArray BosonMap::saveMapToFile()
  return doc.toCString();
 }
 
-bool BosonMap::saveCompleteMap(QDataStream& stream)
+QByteArray BosonMap::saveWaterToFile() const
+{
+ QDomDocument doc(QString::fromLatin1("Water"));
+ QDomElement root = doc.createElement(QString::fromLatin1("Water"));
+ doc.appendChild(root);
+ if (!d->mWaterManager->saveToXML(root)) {
+	return QByteArray();
+ }
+ return doc.toCString();
+}
+
+bool BosonMap::saveCompleteMap(QDataStream& stream) const
 {
  // AB: we may have a problem here - this stream is meant to be sent through
  // network, but it is very big! (sometimes several MB)
@@ -777,33 +843,10 @@ bool BosonMap::saveCompleteMap(QDataStream& stream)
  if (!saveTexMap(stream)) {
 	boError() << k_funcinfo << "Could not save texmap" << endl;
  }
- if (!saveCells(stream)) {
-	boError() << k_funcinfo << "Could not save map cells" << endl;
-	return false;
- }
  return true;
 }
 
-bool BosonMap::saveCells(QDataStream& stream)
-{
- for (unsigned int i = 0; i < width(); i++) {
-	for (unsigned int j = 0; j < height(); j++) {
-		Cell* c = cell(i, j);
-		if (!c) {
-			boError() << k_funcinfo << "NULL Cell" << endl;
-			// do not abort - otherwise all clients receiving this
-			// stream are completely broken as we expect
-			// width()*height() cells
-			saveCell(stream, 0, 0);
-		} else {
-			saveCell(stream, c->amountOfLand(), c->amountOfWater());
-		}
-	}
- }
- return true;
-}
-
-bool BosonMap::saveHeightMap(QDataStream& stream)
+bool BosonMap::saveHeightMap(QDataStream& stream) const
 {
  if (!mHeightMap) {
 	BoHeightMap heightMap(width() + 1, height() + 1);
@@ -812,7 +855,7 @@ bool BosonMap::saveHeightMap(QDataStream& stream)
  return mHeightMap->save(stream);
 }
 
-QByteArray BosonMap::saveHeightMapImage()
+QByteArray BosonMap::saveHeightMapImage() const
 {
  // this function is sloooow !
  if (!width() || !height()) {
@@ -858,7 +901,7 @@ QByteArray BosonMap::saveHeightMapImage()
  return array;
 }
 
-QByteArray BosonMap::saveTexMapImage(unsigned int texture)
+QByteArray BosonMap::saveTexMapImage(unsigned int texture) const
 {
  // this function is sloooow !
  if (!width() || !height()) {
@@ -911,7 +954,7 @@ void BosonMap::createCells()
  }
  mCells = new Cell[width() * height()];
  for (unsigned int x = 0; x < width(); x++) {
-	for (unsigned y = 0; y < height(); y++) {
+	for (unsigned int y = 0; y < height(); y++) {
 		Cell* c = cell(x, y);
 		if (!c) {
 			boError() << k_funcinfo << "Evil internal error!" << endl;
@@ -944,24 +987,6 @@ bool BosonMap::isValidMapGeo(unsigned int width, unsigned int height)
  return true;
 }
 
-bool BosonMap::loadCell(QDataStream& stream, unsigned char* amountOfLand, unsigned char* amountOfWater) const
-{
- Q_UINT8 land;
- Q_UINT8 water;
- stream >> land;
- stream >> water;
-
- (*amountOfLand) = land;
- (*amountOfWater) = water;
- return true;
-}
-
-void BosonMap::saveCell(QDataStream& stream, unsigned char amountOfLand, unsigned char amountOfWater)
-{
- stream << (Q_UINT8)amountOfLand;
- stream << (Q_UINT8)amountOfWater;
-}
-
 Cell* BosonMap::cell(int x, int y) const
 {
  if (!mCells) {
@@ -983,35 +1008,56 @@ float BosonMap::heightAtCorner(int x, int y) const
  return mHeightMap->heightAt(x, y);
 }
 
-void BosonMap::setHeightAtCorner(int x, int y, float h)
+void BosonMap::setHeightsAtCorners(const QValueList< QPair<QPoint, float> >& heights)
 {
  BO_CHECK_NULL_RET(mHeightMap);
  BO_CHECK_NULL_RET(mNormalMap);
  boDebug() << k_funcinfo << endl;
- mHeightMap->setHeightAt(x, y, h);
+
+ int minX = 0;
+ int maxX = 0;
+ int minY = 0;
+ int maxY = 0;
+ QValueList< QPair<QPoint, float> >::const_iterator it;
+ for (it = heights.begin(); it != heights.end(); ++it) {
+	int x = (*it).first.x();
+	int y = (*it).first.y();
+	mHeightMap->setHeightAt(x, y, (*it).second);
+
+	minX = QMIN(minX, x);
+	minY = QMIN(minY, y);
+	maxX = QMAX(maxX, x);
+	maxY = QMAX(maxY, y);
+ }
+
  // Update affected normals
- // Whenever a corner's height changes, normals of four cells, that have this
- //  corner, also changes and every corner that these cells have must be updated
- //  If changed corner is not at the edge of the map, 9 corners have to be
- //  updated.
- recalculateNormalsInRect(QMAX(0, x - 1), QMAX(0, y - 1),
-		QMIN((int)width(), x + 1), QMIN((int)height(), y + 1));
+ minX = QMAX(0, minX - 1);
+ maxX = QMIN((int)width(), maxX + 1);
+ minY = QMAX(0, minY - 1);
+ maxY = QMIN((int)height(), maxY + 1);
+ recalculateNormalsInRect(minX, minY, maxX, maxY);
 }
 
-void BosonMap::slotChangeCell(int x, int y, unsigned char amountOfLand, unsigned char amountOfWater)
+void BosonMap::setHeightAtCorner(int x, int y, float h)
 {
-//boDebug() << x << " -> " << y << endl;
-//boDebug() << width() << " " << height() << endl;
- Cell* c = cell(x, y);
- if (!c) {
-	boError() << k_funcinfo << "Invalid cell x=" << x << ",y=" << y << endl;
-	return;
+ QValueList< QPair<QPoint, float> > heights;
+ heights.append(QPair<QPoint, float>(QPoint(x, y), h));
+ setHeightsAtCorners(heights);
+}
+
+float BosonMap::waterDepthAtCorner(int x, int y) const
+{
+ if (!d->mWaterManager) {
+	boError() << k_funcinfo << "NULL water manager" << endl;
+	return 0.0f;
  }
- if ((int)amountOfLand + (int)amountOfWater > 255) {
-	boWarning() << k_funcinfo << "invalid amounts of land/water: " << amountOfLand << "/" << amountOfWater << endl;
-	// we do not return - we have to keep in sync with network
- }
- c->makeCell(amountOfLand, amountOfWater);
+ return d->mWaterManager->waterDepthAtCorner(x, y);
+}
+
+const QPtrList<BoLake>* BosonMap::lakes() const
+{
+ BO_CHECK_NULL_RET0(d->mWaterManager);
+ return d->mWaterManager->lakes();
 }
 
 void BosonMap::resize(unsigned int width, unsigned int height)
@@ -1030,16 +1076,7 @@ void BosonMap::resize(unsigned int width, unsigned int height)
  loadMapGeo(width, height);
 }
 
-int BosonMap::currentTexture(int texture) const
-{
- BO_CHECK_NULL_RET0(groundTheme());
- BosonTextureArray* t = groundTheme()->textures(texture);
- BO_CHECK_NULL_RET0(t);
- return t->texture((boGame->advanceCallsCount() / groundTheme()->textureAnimationDelay(texture)) % t->count());
-}
-
-
-void BosonMap::fill(unsigned int texture)
+void BosonMap::fill(unsigned int groundtype)
 {
  if (!mCells) {
 	boError() << k_funcinfo << "NULL cells" << endl;
@@ -1047,12 +1084,12 @@ void BosonMap::fill(unsigned int texture)
  }
  BO_CHECK_NULL_RET(mTexMap);
  BO_CHECK_NULL_RET(groundTheme());
- if (texture >= groundTheme()->textureCount()) {
-	boError() << k_funcinfo << "invalid texture " << texture << endl;
+ if (groundtype >= groundTheme()->groundTypeCount()) {
+	boError() << k_funcinfo << "invalid groundtype " << groundtype << endl;
 	return;
  }
 
- mTexMap->fill(texture);
+ mTexMap->fill(groundtype);
 }
 
 float BosonMap::cellAverageHeight(int x, int y) const
@@ -1092,97 +1129,13 @@ bool BosonMap::generateCellsFromTexMap()
 	return false;
  }
  createCells();
- if (groundTheme()->textureCount() == 0) {
-	boError() << k_funcinfo << "0 textures in map" << endl;
+ if (groundTheme()->groundTypeCount() == 0) {
+	boError() << k_funcinfo << "0 ground types in ground theme" << endl;
 	return false;
  }
- for (unsigned int x = 0; x < width(); x++) {
-	for (unsigned int y = 0; y < height(); y++) {
-		recalculateCell(x, y);
-	}
- }
  return true;
 }
 
-QRgb BosonMap::miniMapColor(unsigned int texture) const
-{
- if (!groundTheme()) {
-	boWarning() << k_funcinfo << "NULL groundTheme" << endl;
-	return 0;
- }
- return groundTheme()->miniMapColor(texture);
-}
-
-void BosonMap::recalculateCell(int x, int y)
-{
- Cell* c = cell(x, y);
- BO_CHECK_NULL_RET(c);
- BO_CHECK_NULL_RET(mTexMap);
- BO_CHECK_NULL_RET(groundTheme());
- if (x < 0 || (uint)x >= width()) {
-	boError() << k_funcinfo << "invalid x: " << x << endl;
-	return;
- }
- if (y < 0 || (uint)y >= height()) {
-	boError() << k_funcinfo << "invalid y: " << y << endl;
-	return;
- }
-
- // how much land/water is on the cell. maximum is 255 per corner, i.e. 255*4
- // sum of both must be <= 4*255
- int land = 0;
- int water = 0;
-
- // every (!) cell has exactly 4 corners. every corner has
- // textureCount() alpha values.
- int* alpha = new int[4 * groundTheme()->textureCount()];
- for (unsigned int i = 0; i < groundTheme()->textureCount(); i++) {
-	// top-left corner
-	alpha[4 * i] = (int)texMapAlpha(i, x, y);
-	// top-right corner
-	alpha[4 * i + 1] = (int)texMapAlpha(i, x + 1, y);
-	// bottom-left corner
-	alpha[4 * i + 2] = (int)texMapAlpha(i, x, y + 1);
-	// bottom-right corener
-	alpha[4 * i + 3] = (int)texMapAlpha(i, x + 1, y + 1);
-	for (int j = 0; j < 4; j++) {
-		int a = alpha[4 * i + j];
-		if (a == 0) {
-			// no need to do anything.
-			continue;
-		}
-		int l = (int)groundTheme()->amountOfLand(i);
-		int w = (int)groundTheme()->amountOfWater(i);
-		if (l != 0) {
-			land += (int)(l * a / 255);
-		}
-		if (w != 0) {
-			water += (int)(w * a / 255);
-		}
-	}
- }
- delete[] alpha;
- alpha = 0;
-
- if (land + water == 0) {
-	boWarning() << k_funcinfo << "land + water == 0 for cell at "
-			<< x << "," << y << endl;
-	land = 0;
-	water = 4 * 255;
- }
-
- // in the optimal case sum should be 4 * 255 (as of 4 corners).
- // but we can't be 100% sure (bugs, rounding errors).
- // also we have to scale the numbers down (4*255 to 255)
- int sum = land + water;
- unsigned char amountOfLand = (unsigned char)(land * 255 / sum);
- unsigned char amountOfWater = (unsigned char)(water * 255 / sum);
-
- // amountOfLand + amountOfWater must be 255.
- amountOfLand += (255 - amountOfLand - amountOfWater);
- slotChangeCell(x, y, amountOfLand, amountOfWater);
-}
-
 void BosonMap::slotChangeTexMap(int x, int y, unsigned int texCount, unsigned int* textures, unsigned char* alpha)
 {
  // AB: this is an ugly hack, we shouldn't connect any signals in game mode to
@@ -1204,15 +1157,15 @@ void BosonMap::slotChangeTexMap(int x, int y, unsigned int texCount, unsigned in
 	boError() << k_funcinfo << "invalid y coordinate: " << y << endl;
 	return;
  }
- if (texCount > groundTheme()->textureCount()) {
-	boError() << k_funcinfo << "invalid textureCount " << texCount << " must be <= " << groundTheme()->textureCount() << endl;
+ if (texCount > groundTheme()->groundTypeCount()) {
+	boError() << k_funcinfo << "invalid groundTypeCount " << texCount << " must be <= " << groundTheme()->groundTypeCount() << endl;
 	return;
  }
  BO_CHECK_NULL_RET(alpha);
  BO_CHECK_NULL_RET(textures);
  for (unsigned int i = 0; i < texCount; i++) {
 	unsigned int texture = textures[i];
-	if (texture >= groundTheme()->textureCount()) {
+	if (texture >= groundTheme()->groundTypeCount()) {
 		boError() << k_funcinfo << "invalid texture " << texture << endl;
 		continue;
 	}
@@ -1249,7 +1202,6 @@ void BosonMap::slotChangeTexMap(int x, int y, unsigned int texCount, unsigned in
 		cellsX[count] = x - 1;
 		cellsY[count] = y - 1;
 		count++;
-		recalculateCell(x - 1, y - 1);
 	} else { // somewhere between top and bottom
 		cellsX[count] = x - 1;
 		cellsY[count] = y;
@@ -1294,8 +1246,6 @@ void BosonMap::slotChangeTexMap(int x, int y, unsigned int texCount, unsigned in
 	count = 4;
  }
  for (unsigned int i = 0; i < count; i++) {
-	recalculateCell(cellsX[i], cellsY[i]);
-
 	// update minimap
 	// we may want to group these cells into a single array to save some
 	// speed once the editor is able to modify several cells at once.
@@ -1340,8 +1290,8 @@ void BosonMap::recalculateNormalsInRect(int x1, int y1, int x2, int y2)
 	boWarning() << k_funcinfo << "w*h <= 0 is not valid" << endl;
 	return;
  }
- BoVector3 a, b, c, n;
- BoVector3* normals = new BoVector3[w * h];
+ BoVector3Float a, b, c, n;
+ BoVector3Float* normals = new BoVector3Float[w * h];
 #define NORM(x, y) normals[(y - cy1) * w + (x - cx1)]
  // FIXME: this is not entirely correct: our cells consist of a single quad atm,
  //  however, they can be "folded" so that it seems to consist of two triangles.
@@ -1351,7 +1301,7 @@ void BosonMap::recalculateNormalsInRect(int x1, int y1, int x2, int y2)
 		a.set(x, y, heightAtCorner(x, y));
 		b.set(x + 1, y, heightAtCorner(x + 1, y));
 		c.set(x, y + 1, heightAtCorner(x, y + 1));
-		n = BoVector3::crossProduct(c - b, a - b);
+		n = BoVector3Float::crossProduct(c - b, a - b);
 		n.normalize();
 		NORM(x, y) = n;
 	}
@@ -1405,28 +1355,29 @@ int BosonMap::mapFileFormatVersion()
  return BOSONMAP_VERSION;
 }
 
-void BosonMap::createColorMap()
+void BosonMap::addColorMap(BoColorMap* map, const QString& name)
 {
- boDebug() << k_funcinfo << "" << endl;
- if (mColorMap) {
-	// Old normal map should already be deleted
-	boWarning() << k_funcinfo << "Old color map not deleted!" << endl;
-	delete mColorMap;
-	mColorMap = 0;
+ if (d->mColorMaps[name]) {
+	boWarning() << k_funcinfo << "already have a colormap named " << name << endl;
+	BoColorMap* old = d->mColorMaps[name];
+	delete old;
  }
-
- boDebug() << k_funcinfo << "creating map" << endl;
- mColorMap = new BoColorMap(width(), height());
-
- boDebug() << k_funcinfo << "initing map" << endl;
- // Init colormap to gray
- unsigned char color[] = { 128, 128, 128 };
- for (unsigned int y = 0; y < height(); y++) {
-	for (unsigned int x = 0; x < width(); x++) {
-		mColorMap->setColorAt(x, y, color);
-	}
- }
- boDebug() << k_funcinfo << "done" << endl;
+ d->mColorMaps.insert(name, map);
+ emit signalColorMapsChanged();
 }
 
+QDict<BoColorMap>* BosonMap::colorMaps()
+{
+ return &d->mColorMaps;
+}
+
+void BosonMap::removeColorMap(const QString& name)
+{
+ BoColorMap* map = d->mColorMaps.take(name);
+ if (map == mActiveColorMap) {
+	mActiveColorMap = 0;
+ }
+ delete map;
+ emit signalColorMapsChanged();
+}
 
