@@ -1,6 +1,8 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 1999-2000,2001-2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 1999-2000 Thomas Capricelli (capricel@email.enst.fr)
+    Copyright (C) 2001-2005 Andreas Beckermann (b_mann@gmx.de)
+    Copyright (C) 2001-2005 Rivo Laks (rivolaks@hot.ee)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,6 +22,7 @@
 #define BOSONMAPDOM_H
 
 #include <qobject.h>
+#include <qrect.h>
 
 #include "bo3dtools.h"
 
@@ -29,7 +32,14 @@ class Boson;
 class QStringList;
 class QDataStream;
 class BosonGroundTheme;
-class BosonTextureArray;
+class BoTexture;
+class BosonGroundType;
+class BosonMap;
+class BoLake;
+template<class T> class QDict;
+template<class T> class QValueList;
+template<class T> class QPtrList;
+template<class T1, class T2> class QPair;
 
 /**
  * @short abstract class for data operating of the corners of the cells (like
@@ -51,7 +61,7 @@ public:
 	{
 	}
 
-	virtual bool save(QDataStream& stream) = 0;
+	virtual bool save(QDataStream& stream) const = 0;
 	virtual bool load(QDataStream& stream) = 0;
 
 	/**
@@ -111,7 +121,7 @@ public:
 		delete[] mHeightMap;
 	}
 
-	virtual bool save(QDataStream& stream);
+	virtual bool save(QDataStream& stream) const;
 	virtual bool load(QDataStream& stream);
 
 	inline float* heightMap() const { return mHeightMap; }
@@ -141,8 +151,8 @@ public:
 		}
 		// AB: see pixelToHeight() for explanation on these
 		// restrictions
-		h = QMIN(h, 15.0f);
-		h = QMAX(h, -10.5f);
+		h = QMIN(h, 18.75f);
+		h = QMAX(h, -13.125f);
 		mHeightMap[arrayPos(x, y)] = h;
 	}
 
@@ -190,7 +200,7 @@ public:
 		mTextureCount = textureCount;
 		initialize(0, 255);
 		for (unsigned int i = 1; i < textureCount; i++) {
-			initialize(0, 0);
+			initialize(i, 0);
 		}
 	}
 	~BoTexMap()
@@ -198,7 +208,7 @@ public:
 		delete[] mTexMap;
 	}
 
-	virtual bool save(QDataStream& stream);
+	virtual bool save(QDataStream& stream) const;
 	virtual bool load(QDataStream& stream);
 
 	/**
@@ -348,7 +358,7 @@ public:
 	}
 
 	// Normal maps won't be saved/loaded (they're auto-generated)
-	virtual bool save(QDataStream&) { return true; }
+	virtual bool save(QDataStream&) const { return true; }
 	virtual bool load(QDataStream&) { return true; }
 
 	inline const float* normalMap() const { return mNormalMap; }
@@ -374,7 +384,7 @@ public:
 		return mNormalMap + normalMapArrayPos(x, y);
 	}
 
-	void setNormalAt(int x, int y, const BoVector3& n)
+	void setNormalAt(int x, int y, const BoVector3Float& n)
 	{
 		if (x < 0 || y < 0) {
 			return;
@@ -397,6 +407,72 @@ private:
 };
 
 
+/**
+ * @author Rivo Laks <rivolaks@hot.ee>
+ **/
+class BoColorMap
+{
+public:
+	BoColorMap(unsigned int width, unsigned int height);
+
+	virtual ~BoColorMap();
+
+	// Color maps won't be saved/loaded (they're auto-generated)
+	virtual bool save(QDataStream&) const { return true; }
+	virtual bool load(QDataStream&) { return true; }
+
+	void update(unsigned char* data);
+	void updateRect(int x, int y, unsigned int w, unsigned int h, unsigned char* data);
+
+	const unsigned char* textureData() const
+	{
+		return mData;
+	}
+
+	unsigned int width() const
+	{
+		return mWidth;
+	}
+	unsigned int height() const
+	{
+		return mHeight;
+	}
+
+	/**
+	 * @return Whether the colormap changed since @ref setNotDirty was
+	 * called.
+	 **/
+	bool isDirty() const
+	{
+		return mDirtyRect.isValid();
+	}
+
+	/**
+	 * @return A rect containing the area that changed since @ref
+	 * setNotDirty was called.
+	 **/
+	QRect dirtyRect() const
+	{
+		return mDirtyRect;
+	}
+
+	/**
+	 * Unset any previous dirty settings
+	 **/
+	void setNotDirty()
+	{
+		mDirtyRect = QRect();
+	}
+
+private:
+	unsigned int mWidth;
+	unsigned int mHeight;
+	unsigned char* mData;
+
+	QRect mDirtyRect;
+};
+
+
 /**
  * This class represents a boson map. It is part of a @ref BosonPlayField (a
  * .bpf file) and gets stored on disk as binary.
@@ -543,11 +619,16 @@ public:
 		return BoMapCornerArray::arrayPos(x, y, width() + 1);
 	}
 
-	int currentTexture(int texture) const;
 	inline const float* heightMap() const { return mHeightMap->heightMap(); }
 	inline const float* normalMap() const { return mNormalMap->normalMap(); }
+	void setActiveColorMap(BoColorMap* map) { mActiveColorMap = map; }
+	inline BoColorMap* activeColorMap() const { return mActiveColorMap; }
 	BosonGroundTheme* groundTheme() const { return mGroundTheme; }
 
+	void addColorMap(BoColorMap* map, const QString& name);
+	QDict<BoColorMap>* colorMaps();
+	void removeColorMap(const QString& name);
+
 	/**
 	 * @return The internal texmap array, which defines how much percent
 	 * of every texture (see also @ref textureCount) are used in the corners
@@ -599,35 +680,34 @@ public:
 		return false;
 	}
 
-	/**
-	 * @return @ref BosonGroundTheme::miniMapColor
-	 **/
-	QRgb miniMapColor(unsigned int texture) const;
-
 	/**
 	 * Note that you can specify (width() + 1) * (height() + 1) corners here!
 	 * See also @ref cornerArrayPos for detailed description of possible
 	 * values.
 	 * @return The height of the upper left corner of the cell at @p x, @p
 	 * y or 1.0 if invalid coordinates were specified.
+	 *
+	 * WARNING this returns the height of the ground at this corner. It does
+	 * NOT consider the @ref waterDepth !
 	 **/
 	float heightAtCorner(int x, int y) const;
 	void setHeightAtCorner(int x, int y, float height);
-	float cellAverageHeight(int x, int y);
-
-
-
-
-	QByteArray saveMapToFile();
+	float cellAverageHeight(int x, int y) const;
+	void setHeightsAtCorners(const QValueList< QPair<QPoint, float> >& heights);
 
 	/**
-	 * Save the complete map into the stream, even the parts that are
-	 * usually stored in different files, such as the height map.
-	 *
-	 * Use this to send the map over network, but remember that there will
-	 * be a lot of data! (probably more a few MB for 500x500 maps!)
+	 * @return The water depth at the specified corner. This is the distance
+	 * of the @ref heightAtCorner to the surface of the water.
 	 **/
-	bool saveCompleteMap(QDataStream& stream);
+	float waterDepthAtCorner(int x, int y) const;
+
+	const QPtrList<BoLake>* lakes() const;
+
+
+
+	QByteArray saveMapToFile() const;
+	QByteArray saveWaterToFile() const;
+
 
 	/**
 	 * Load the "main" map, i.e. the map geo (it's size) and it's cells from
@@ -636,6 +716,8 @@ public:
 	 **/
 	bool loadMapFromFile(const QByteArray& map);
 
+	bool loadWaterFromFile(const QByteArray& water);
+
 	/**
 	 * Load the complete map, even those data that are stored in different
 	 * files in the .bpf file. This is e.g. the height map.
@@ -657,7 +739,7 @@ public:
 
 	bool importHeightMapImage(const QImage& image);
 
-	QByteArray saveHeightMapImage();
+	QByteArray saveHeightMapImage() const;
 
 	/**
 	 * Load the @ref texMap from @p stream.
@@ -667,9 +749,9 @@ public:
 	 * already loaded cells. The default number of textures is used then.
 	 **/
 	bool loadTexMap(QDataStream& stream);
-	bool saveTexMap(QDataStream& stream);
+	bool saveTexMap(QDataStream& stream) const;
 
-	QByteArray saveTexMapImage(unsigned int texture);
+	QByteArray saveTexMapImage(unsigned int texture) const;
 
 	void setModified(bool m) { mModified = m; }
 	bool modified() const { return mModified; }
@@ -688,8 +770,6 @@ public:
 
 
 public slots:
-	void slotChangeCell(int x, int y, unsigned char amountOfLand, unsigned char amountOfWater);
-
 	/**
 	 * Change the alpha value of the texmap at @ x, @p y for @ texture.
 	 *
@@ -710,8 +790,6 @@ public slots:
 	void slotChangeTexMap(int x, int y, unsigned int textureCount, unsigned int* textures, unsigned char* alpha);
 
 signals:
-	void signalGroundThemeChanged(BosonGroundTheme*);
-
 	/**
 	 * The @ref cell at @p x, @p y has been changed. This refers to the
 	 * texture values / amounts of land and water only, <em>not</em> to the
@@ -722,13 +800,10 @@ signals:
 	 **/
 	void signalCellChanged(int x, int y);
 
+	void signalColorMapsChanged();
+
 protected:
-	bool loadCell(QDataStream& stream, unsigned char* amountOfLand, unsigned char* amountOfLand) const;
-
-	void saveCell(QDataStream& stream, unsigned char amountOfLand, unsigned char amountOfWater);
-
-	bool saveCells(QDataStream& stream);
-	bool saveHeightMap(QDataStream& stream);
+	bool saveHeightMap(QDataStream& stream) const;
 
 	/**
 	 * Save @p heightMap into @p stream.
@@ -739,37 +814,24 @@ protected:
 
 	bool loadMapGeo(unsigned int width, unsigned int height);
 
-	bool loadGroundTheme(const QString& id);
-
-	/**
-	 * Load the cells from the stream.
-	 * @param stream The stream to read from
-	 **/
-	bool loadCells(QDataStream& stream);
+	bool applyGroundTheme(const QString& id);
 
 	bool loadHeightMap(QDataStream& stream);
 
-	/**
-	 * Recalculate the amountOfLand/Water values for the cell at @ x,@p y.
-	 *
-	 * These values are based on the alpha values in the textures at all 4
-	 * corners of the cell.
-	 *
-	 * This should be called on map construction only (see @ref 
-	 * generateCellsFromTexMap), except in editor mode.
-	 *
-	 * Also note that the calculated values for this cell are <em>not</em>
-	 * fully network safe! The ADMIN will calculate these values and send
-	 * them out through network (see @ref loadCompleteMap), so that all
-	 * clients will use the same values.
-	 **/
-	void recalculateCell(int x, int y);
-
 	/**
 	 * Create the map array accordint to @ref width and @ref height.
 	 **/
 	void createCells();
 
+	/**
+	 * Save the complete map into the stream, even the parts that are
+	 * usually stored in different files, such as the height map.
+	 *
+	 * Use this to send the map over network, but remember that there will
+	 * be a lot of data! (probably more a few MB for 500x500 maps!)
+	 **/
+	bool saveCompleteMap(QDataStream& stream) const;
+
 private:
 	void init();
 
@@ -779,6 +841,7 @@ private:
 	Cell* mCells;
 	BoHeightMap* mHeightMap;
 	BoNormalMap* mNormalMap;
+	BoColorMap* mActiveColorMap;
 	BoTexMap* mTexMap;
 	bool mModified;
 
