@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2002-2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2002-2005 Andreas Beckermann (b_mann@gmx.de)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -66,6 +66,7 @@ public:
 	}
 
 	bool loadInformation(BPFFile* file);
+	bool loadInformation(const QMap<QString, QByteArray>& files);
 
 protected:
 	bool loadPlayersInformation(const QByteArray& xml);
@@ -107,7 +108,7 @@ public:
 	 * Use @ref loadPlayField to load <em>all</em> data. preLoadPlayField is
 	 * much faster than @ref loadPlayField.
 	 **/
-	bool preLoadPlayField(const QString& identifier);
+	bool preLoadPlayField(const QString& file);
 
 	/**
 	 * Load the playfield from @p files to this object.
@@ -130,8 +131,6 @@ public:
 	 **/
 	QByteArray exportTexMap(unsigned int texture) const;
 
-	bool savePlayField(const QString& fileName);
-
 	/**
 	 * Make @p map the new map of this playfield. Called by the editor
 	 * only.
@@ -174,17 +173,9 @@ public:
 	QString playFieldName() const;
 	QString playFieldComment() const;
 
-	/**
-	 * @return Whether the the playfield has been completely loaded. This is
-	 * the case when @ref loadPlayField has been called (<em>not</em> @ref
-	 * preLoadPlayField)
-	 **/
-	bool isLoaded() const { return mLoaded; }
-
 	/**
 	 * @return Whether the the playfield has been pre-loaded. This is
-	 * the case when @ref preLoadPlayField has been called. Note that this
-	 * happens in @ref loadPlayField as well.
+	 * the case when @ref preLoadPlayField has been called.
 	 **/
 	bool isPreLoaded() const { return mPreLoaded; }
 
@@ -214,18 +205,13 @@ public:
 	 **/
 	static QString defaultPlayField();
 
-signals:
-	/**
-	 * Emitted when the map changes. Note that this can even be 0!
-	 **/
-	void signalNewMap(BosonMap*);
-
 protected:
 	bool loadDescriptionFromFile(const QByteArray& xml);
-	bool loadMapFromFile(const QByteArray& xml, const QByteArray& heightMapImage, const QByteArray& texMap);
+	bool loadMapFromFile(const QByteArray& xml, const QByteArray& heightMapImage, const QByteArray& texMap, const QByteArray& waterXML);
 
 	QString saveDescriptionToFile();
 	QByteArray saveMapToFile();
+	QByteArray saveWaterToFile();
 	QByteArray saveTexMapToFile();
 
 	/**
@@ -242,7 +228,7 @@ protected:
 	 * conversion.
 	 *
 	 * Here you are meant to place the actual conversion code.
-	 * @param set to TRUE when the @p version has been recognized and (!)
+	 * @param handled set to TRUE when the @p version has been recognized and (!)
 	 * used to convert the files to a new format (i.e. at least changed
 	 * the version number). Otherwise set to FALSE (i.e. if the files have
 	 * not been touched at all).
@@ -265,7 +251,6 @@ private:
 	BosonPlayFieldInformation* mPlayFieldInformation;
 	BPFDescription* mDescription;
 	QString mIdentifier; // AB: this is not yet fully implemented - e.g. it isn't changed when saving or changing the map. should be the filename (see BPFFile::identifier())
-	bool mLoaded;
 	bool mPreLoaded;
 	BPFFile* mFile;
 	QString mFileName;
