@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2003-2004 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2003-2005 Andreas Beckermann (b_mann@gmx.de)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 #include "bosonsaveload.h"
 #include "bosonsaveload.moc"
 
+#include "../bomemory/bodummymemory.h"
 #include "boson.h"
 #include "boversion.h"
 #include "bosonprofiling.h"
@@ -30,9 +31,7 @@
 #include "player.h"
 #include "bodebug.h"
 #include "bosonfileconverter.h"
-
-// FIXME do not include
-#include <startupwidgets/bosonloadingwidget.h>
+#include "boeventmanager.h"
 
 #include <kgame/kgamemessage.h>
 #include <kgame/kgamepropertyhandler.h>
@@ -42,6 +41,7 @@
 #include <qdatastream.h>
 #include <qptrqueue.h>
 #include <qvaluelist.h>
+#include <qregexp.h>
 
 SaveLoadError::SaveLoadError(ErrorType type, const QString& text, const QString& caption)
 {
@@ -100,14 +100,12 @@ public:
 	BosonPlayField* mPlayField;
 	BosonCanvas* mCanvas;
 
-	BosonSaveLoad::LoadingStatus mLoadingStatus;
 	QPtrQueue<SaveLoadError> mErrorQueue;
 };
 
 BosonSaveLoad::BosonSaveLoad(Boson* boson) : QObject(boson, "bosonsaveload")
 {
  d = new BosonSaveLoadPrivate;
- d->mLoadingStatus = NotLoaded;
  d->mBoson = boson;
  d->mErrorQueue.setAutoDelete(true);
 
@@ -127,23 +125,12 @@ void BosonSaveLoad::initBoson()
 	return;
  }
 
- connect(this, SIGNAL(signalLoadExternalStuff(QDataStream&)),
-		d->mBoson, SIGNAL(signalLoadExternalStuff(QDataStream&)));
- connect(this, SIGNAL(signalSaveExternalStuff(QDataStream&)),
-		d->mBoson, SIGNAL(signalSaveExternalStuff(QDataStream&)));
  connect(this, SIGNAL(signalLoadExternalStuffFromXML(const QDomElement&)),
 		d->mBoson, SIGNAL(signalLoadExternalStuffFromXML(const QDomElement&)));
  connect(this, SIGNAL(signalSaveExternalStuffAsXML(QDomElement&)),
 		d->mBoson, SIGNAL(signalSaveExternalStuffAsXML(QDomElement&)));
 }
 
-void BosonSaveLoad::systemAddPlayer(KPlayer* p)
-{
- if (p && d->mBoson) {
-	d->mBoson->systemAddPlayer_(this, p);
- }
-}
-
 void BosonSaveLoad::setCanvas(BosonCanvas* canvas)
 {
  d->mCanvas = canvas;
@@ -169,11 +156,6 @@ void BosonSaveLoad::addSaveError(SaveLoadError::ErrorType type, const QString& t
  addError(new SaveError(type, text, caption));
 }
 
-BosonSaveLoad::LoadingStatus BosonSaveLoad::loadingStatus() const
-{
- return d->mLoadingStatus;
-}
-
 unsigned long int BosonSaveLoad::latestSavegameVersion()
 {
  return BOSON_SAVEGAME_FORMAT_VERSION;
@@ -221,7 +203,7 @@ bool BosonSaveLoad::saveToFiles(QMap<QString, QByteArray>& files)
 	boError() << k_funcinfo << "NULL boson object" << endl;
 	return false;
  }
- BosonProfiler profiler(BosonProfiling::SaveGameToXML);
+ BosonProfiler profiler("SaveKGameToXML");
  if (!d->mPlayField) {
 	BO_NULL_ERROR(d->mPlayField);
 	return false;
@@ -230,45 +212,47 @@ bool BosonSaveLoad::saveToFiles(QMap<QString, QByteArray>& files)
 	boError() << k_funcinfo << "saving the playfield failed" << endl;
 	return false;
  }
- boProfiling->start(BosonProfiling::SaveKGameToXML);
  QByteArray kgameXML = saveKGameAsXML();
- boProfiling->stop(BosonProfiling::SaveKGameToXML);
  if (kgameXML.isNull()) {
 	return false;
  }
 
- boProfiling->start(BosonProfiling::SavePlayersToXML);
  QByteArray playersXML = savePlayersAsXML();
- boProfiling->stop(BosonProfiling::SavePlayersToXML);
  if (playersXML.isNull()) {
 	return false;
  }
 
- boProfiling->start(BosonProfiling::SaveCanvasToXML);
  QByteArray canvasXML = saveCanvasAsXML();
- boProfiling->stop(BosonProfiling::SaveCanvasToXML);
  if (canvasXML.isNull()) {
 	return false;
  }
 
- boProfiling->start(BosonProfiling::SaveExternalToXML);
  QByteArray externalXML = saveExternalAsXML();
- boProfiling->stop(BosonProfiling::SaveExternalToXML);
  if (externalXML.isNull()) {
 	return false;
  }
 
+ if (!saveEventListenerScripts(&files)) {
+	boError() << k_funcinfo << "could not save event listener scripts" << endl;
+	return false;
+ }
+
  files.insert("kgame.xml", kgameXML);
  files.insert("players.xml", playersXML);
  files.insert("canvas.xml", canvasXML);
  files.insert("external.xml", externalXML);
 
+ if (!convertPlayerIdsToIndices(files)) {
+	return false;
+ }
+
  return true;
 }
 
 bool BosonSaveLoad::savePlayFieldToFiles(QMap<QString, QByteArray>& files)
 {
  // first we save a complete game.
+ BosonProfiler prof("SavePlayFieldToFiles()");
  bool ret = saveToFiles(files);
  if (!ret) {
 	boError() << k_funcinfo << "saving failed" << endl;
@@ -278,25 +262,16 @@ bool BosonSaveLoad::savePlayFieldToFiles(QMap<QString, QByteArray>& files)
  return ret;
 }
 
-bool BosonSaveLoad::saveToFile(const QString& file)
-{
- QMap<QString, QByteArray> files;
- if (!saveToFiles(files)) {
-	boError() << k_funcinfo << "saving failed" << endl;
-	return false;
- }
- return saveToFile(files, file);
-}
-
 bool BosonSaveLoad::saveToFile(const QMap<QString, QByteArray>& files, const QString& file)
 {
  boDebug() << k_funcinfo << file << endl;
- BosonProfiler writeProfiler(BosonProfiling::SaveGameToXMLWriteFile);
+ PROFILE_METHOD
  QByteArray kgameXML = files["kgame.xml"];
  QByteArray playersXML = files["players.xml"];
  QByteArray canvasXML = files["canvas.xml"];
  QByteArray externalXML = files["external.xml"];
  QByteArray mapXML = files["map/map.xml"];
+ QByteArray waterXML = files["map/water.xml"];
  QByteArray heightMap = files["map/heightmap.png"];
  QByteArray texMap = files["map/texmap"];
  QByteArray descriptionXML = files["C/description.xml"];
@@ -323,6 +298,10 @@ bool BosonSaveLoad::saveToFile(const QMap<QString, QByteArray>& files, const QSt
 	boError() << k_funcinfo << "no mapXML found" << endl;
 	return false;
  }
+ if (waterXML.size() == 0) {
+	boError() << k_funcinfo << "no waterXML found" << endl;
+	return false;
+ }
  if (heightMap.size() == 0) {
 	boError() << k_funcinfo << "no heightMap found" << endl;
 	return false;
@@ -356,6 +335,10 @@ bool BosonSaveLoad::saveToFile(const QMap<QString, QByteArray>& files, const QSt
 	boError() << k_funcinfo << "Could not write map to " << file << endl;
 	return false;
  }
+ if (!f.writeFile(QString::fromLatin1("water.xml"), QString(waterXML), QString::fromLatin1("map"))) {
+	boError() << k_funcinfo << "Could not write water to " << file << endl;
+	return false;
+ }
  if (!f.writeFile(QString::fromLatin1("heightmap.png"), heightMap, QString::fromLatin1("map"))) {
 	boError() << k_funcinfo << "Could not write map to " << file << endl;
 	return false;
@@ -368,11 +351,24 @@ bool BosonSaveLoad::saveToFile(const QMap<QString, QByteArray>& files, const QSt
 	boError() << k_funcinfo << "Could not write map to " << file << endl;
 	return false;
  }
+
+ QStringList scripts = QStringList(files.keys()).grep("scripts");
+ for (QStringList::iterator it = scripts.begin(); it != scripts.end(); ++it) {
+	QString path = *it;
+	int lastSlash = path.findRev('/');
+	QString dir = path.left(lastSlash);
+	QString baseName = path.right(path.length() - (lastSlash + 1));
+	if (!f.writeFile(baseName, files[path], dir)) {
+		boError() << k_funcinfo << "Could not write " << baseName << " to " << file << endl;
+		return false;
+	}
+ }
  return true;
 }
 
 QCString BosonSaveLoad::saveKGameAsXML()
 {
+ PROFILE_METHOD
  QDomDocument doc(QString::fromLatin1("Boson"));
  QDomElement root = doc.createElement(QString::fromLatin1("Boson")); // XML file for the Boson object
  doc.appendChild(root);
@@ -390,14 +386,20 @@ QCString BosonSaveLoad::saveKGameAsXML()
  propertyXML.removeProperty(handler, KGamePropertyBase::IdGameStatus);
  root.appendChild(handler);
 
- // here we could add additional data for the Boson object.
- // but I believe all data should get into the datahandler as far as possible
+
+ QDomElement eventManager = doc.createElement(QString::fromLatin1("EventManager"));
+ root.appendChild(eventManager);
+ if (!d->mBoson->eventManager()->saveAsXML(eventManager)) {
+	boError() << k_funcinfo << "unable to save the event manager" << endl;
+	return false;
+ }
 
  return doc.toCString();
 }
 
 QCString BosonSaveLoad::savePlayersAsXML()
 {
+ PROFILE_METHOD
  QDomDocument doc(QString::fromLatin1("Players"));
  QDomElement root = doc.createElement(QString::fromLatin1("Players"));
  doc.appendChild(root);
@@ -426,6 +428,7 @@ QCString BosonSaveLoad::savePlayersAsXML()
 
 QCString BosonSaveLoad::saveCanvasAsXML()
 {
+ PROFILE_METHOD
  QDomDocument doc(QString::fromLatin1("Canvas"));
  QDomElement root = doc.createElement(QString::fromLatin1("Canvas")); // XML file for canvas
  doc.appendChild(root);
@@ -443,6 +446,7 @@ QCString BosonSaveLoad::saveCanvasAsXML()
 
 QCString BosonSaveLoad::saveExternalAsXML()
 {
+ PROFILE_METHOD
  QDomDocument doc(QString::fromLatin1("External"));
  QDomElement root = doc.createElement(QString::fromLatin1("External")); // XML file for external data
  doc.appendChild(root);
@@ -453,25 +457,34 @@ QCString BosonSaveLoad::saveExternalAsXML()
 }
 
 
-bool BosonSaveLoad::loadNewGame(const QByteArray& playersXML, const QByteArray& canvasXML)
+bool BosonSaveLoad::startFromFiles(const QMap<QString, QByteArray>& files)
 {
- // AB: nearly all code copied from loadFromFile().
  boDebug(270) << k_funcinfo << endl;
- d->mLoadingStatus = LoadingInProgress;
 
+ QByteArray playersXML = files["players.xml"];
+ QByteArray canvasXML = files["canvas.xml"];
+ QByteArray kgameXML = files["kgame.xml"];
+ QByteArray externalXML = files["external.xml"];
+
+  if (kgameXML.isEmpty()) {
+	boError(270) << k_funcinfo << "Empty kgameXML" << endl;
+	addLoadError(SaveLoadError::LoadBSGFileError, i18n("empty file: kgame.xml"));
+	return false;
+  }
   if (playersXML.isEmpty()) {
 	boError(270) << k_funcinfo << "Empty playersXML" << endl;
 	addLoadError(SaveLoadError::LoadBSGFileError, i18n("empty file: players.xml"));
-	d->mLoadingStatus = BSGFileError;
 	return false;
  }
  if (canvasXML.isEmpty()) {
 	boError(270) << k_funcinfo << "Empty canvasXML" << endl;
 	addLoadError(SaveLoadError::LoadBSGFileError, i18n("empty file: canvas.xml"));
-	d->mLoadingStatus = BSGFileError;
 	return false;
  }
 
+ if (!loadKGameFromXML(kgameXML)) {
+	return false;
+ }
  if (!loadPlayersFromXML(playersXML)) {
 	return false;
  }
@@ -484,7 +497,49 @@ bool BosonSaveLoad::loadNewGame(const QByteArray& playersXML, const QByteArray&
 	addLoadError(SaveLoadError::General, i18n("error while loading canvas"));
 	return false;
  }
+ if (!loadExternalFromXML(externalXML)) {
+	addLoadError(SaveLoadError::General, i18n("error while loading external data"));
+	return false;
+ }
 
+ if (!loadEventListenerScripts(files)) {
+	boError(270) << k_funcinfo << "could not load eventlistener scripts" << endl;
+	addLoadError(SaveLoadError::General, i18n("error while loading event listener scripts"));
+	return false;
+ }
+
+ return true;
+}
+
+bool BosonSaveLoad::loadKGameFromXML(const QString& kgameXML)
+{
+ QDomDocument doc(QString::fromLatin1("Boson"));
+ if (!loadXMLDoc(&doc, kgameXML)) {
+	addLoadError(SaveLoadError::LoadInvalidXML, i18n("Parsing error in kgame.xml"));
+	return false;
+ }
+ QDomElement root = doc.documentElement();
+
+ BosonCustomPropertyXML propertyXML;
+ QDomElement handler = root.namedItem(QString::fromLatin1("DataHandler")).toElement();
+ if (!handler.isNull()) {
+	// note: a missing DataHandler is valid.
+	 BosonPropertyXML::removeProperty(handler, KGamePropertyBase::IdGameStatus);
+	if (!propertyXML.loadFromXML(handler, d->mBoson->dataHandler())) {
+		boError(270) << k_funcinfo << "unable to load KGame DataHandler" << endl;
+		return false;
+	}
+ }
+
+ QDomElement eventManager = root.namedItem(QString::fromLatin1("EventManager")).toElement();
+ if (eventManager.isNull()) {
+	boError(270) << k_funcinfo << "no EventManager tag" << endl;
+	return false;
+ }
+ if (!d->mBoson->eventManager()->loadFromXML(eventManager)) {
+	boError(270) << k_funcinfo << "unable to load EventManager" << endl;
+	return false;
+ }
  return true;
 }
 
@@ -494,7 +549,6 @@ bool BosonSaveLoad::loadPlayersFromXML(const QString& playersXML)
  QDomDocument doc(QString::fromLatin1("Players"));
  if (!loadXMLDoc(&doc, playersXML)) {
 	addLoadError(SaveLoadError::LoadInvalidXML, i18n("Parsing error in players.xml"));
-	d->mLoadingStatus = InvalidXML;
 	return false;
  }
  QDomElement root = doc.documentElement();
@@ -502,7 +556,6 @@ bool BosonSaveLoad::loadPlayersFromXML(const QString& playersXML)
  if (list.count() < 1) {
 	boError(270) << k_funcinfo << "no Player tags in file" << endl;
 	addLoadError(SaveLoadError::LoadInvalidXML, i18n("No Player Tag in players.xml"));
-	d->mLoadingStatus = InvalidXML;
 	return false;
  }
  for (unsigned int i = 0; i < d->mBoson->playerCount(); i++) {
@@ -511,9 +564,9 @@ bool BosonSaveLoad::loadPlayersFromXML(const QString& playersXML)
 	for (unsigned int j = 0; j < list.count() && player.isNull(); j++) {
 		QDomElement e = list.item(j).toElement();
 		bool ok = false;
-		unsigned int id = e.attribute(QString::fromLatin1("Id")).toUInt(&ok);
+		unsigned int id = e.attribute(QString::fromLatin1("PlayerId")).toUInt(&ok);
 		if (!ok) {
-			boError(270) << k_funcinfo << "missing or invalid Id attribute for Player tag " << j << endl;
+			boError(270) << k_funcinfo << "missing or invalid PlayerId attribute for Player tag " << j << endl;
 			continue;
 		}
 		if (p->id() != id) {
@@ -540,7 +593,10 @@ bool BosonSaveLoad::loadPlayersFromXML(const QString& playersXML)
 			return false;
 		}
 	}
-	p->loadFromXML(player);
+	if (!p->loadFromXML(player)) {
+		boError(270) << k_funcinfo << "failed loading player " << i << endl;
+		return false;
+	}
  }
  return true;
 }
@@ -551,14 +607,12 @@ bool BosonSaveLoad::loadCanvasFromXML(const QString& xml)
  QDomDocument doc(QString::fromLatin1("Canvas"));
  if (!loadXMLDoc(&doc, xml)) {
 	addLoadError(SaveLoadError::LoadInvalidXML, i18n("Parsing error in canvas.xml"));
-	d->mLoadingStatus = InvalidXML;
 	return false;
  }
  QDomElement root = doc.documentElement();
 
  if (!d->mCanvas->loadFromXML(root)) {
 	addLoadError(SaveLoadError::LoadInvalidXML, i18n("error while loading canvas.xml"));
-	d->mLoadingStatus = InvalidXML;
 	return false;
  }
 
@@ -567,13 +621,34 @@ bool BosonSaveLoad::loadCanvasFromXML(const QString& xml)
 
 bool BosonSaveLoad::loadExternalFromXML(const QString& xml)
 {
- boDebug(260) << k_funcinfo << endl;
- // Load external stuff (camera)
+ boDebug() << k_funcinfo << endl;
+
+ // external.xml is optional only, it's valid that it's missing
+ // if it is, we use a dummy document. this is meant to make sure that all
+ // pointers are set correctly, even if we don't use them (e.g. canvas pointers
+ // in all widgets that require it)
  QDomDocument doc(QString::fromLatin1("External"));
- if (!loadXMLDoc(&doc, xml)) {
-	addLoadError(SaveLoadError::LoadInvalidXML, i18n("Parsing error in external.xml"));
-	d->mLoadingStatus = InvalidXML;
-	return false;
+ if (xml.length() != 0) {
+	boDebug(260) << k_funcinfo << endl;
+	if (!loadXMLDoc(&doc, xml)) {
+		addLoadError(SaveLoadError::LoadInvalidXML, i18n("Parsing error in external.xml"));
+		return false;
+	}
+ } else {
+	QDomElement root = doc.createElement(QString::fromLatin1("External"));
+	doc.appendChild(root);
+
+	QDomElement unitGroups = doc.createElement("UnitGroups");
+	root.appendChild(unitGroups);
+
+	QDomElement effects = doc.createElement("Effects");
+	root.appendChild(effects);
+
+	QDomElement displays = doc.createElement("Displays");
+	root.appendChild(displays);
+	QDomElement display = doc.createElement("Display");
+	displays.appendChild(display);
+
  }
  QDomElement root = doc.documentElement();
 
@@ -581,8 +656,10 @@ bool BosonSaveLoad::loadExternalFromXML(const QString& xml)
 
  return true;
 }
+
 bool BosonSaveLoad::convertSaveGameToPlayField(QMap<QString, QByteArray>& files)
 {
+ BosonProfiler prof("convertSaveGameToPlayField()");
  // now we remove / change what does not belong there
  QDomDocument kgameDoc("Boson");
  if (!loadXMLDoc(&kgameDoc, QString(files["kgame.xml"]))) {
@@ -607,22 +684,24 @@ bool BosonSaveLoad::convertSaveGameToPlayField(QMap<QString, QByteArray>& files)
 	QDomElement p = playerList.item(i).toElement();
 	p.removeAttribute("NetworkPriority");
 	p.removeAttribute("UnitPropId");
-	p.removeAttribute("TeamColor");
-	p.removeAttribute("SpeciesTheme");
 	p.removeChild(p.namedItem("Statistics"));
 	p.removeChild(p.namedItem("Fogged"));
+
+	QDomElement speciesThemeTag = p.namedItem("SpeciesTheme").toElement();
+	speciesThemeTag.removeAttribute("Identifier");
+	speciesThemeTag.removeAttribute("TeamColor");
 	bool ok = false;
-	unsigned int id = p.attribute("Id").toUInt(&ok);
+	unsigned int id = p.attribute("PlayerId").toUInt(&ok);
 	if (!ok) {
-		boError() << k_funcinfo << "invalid value for Id attribute of Player tag" << endl;
+		boError() << k_funcinfo << "invalid value for PlayerId attribute of Player tag" << endl;
 		return false;
 	}
 	if (id >= playerList.count()) {
-		boError() << k_funcinfo << "invalid Id for Player tag!" << endl;
+		boError() << k_funcinfo << "invalid PlayerId for Player tag!" << endl;
 		return false;
 	}
 	if (id != i) {
-		boError() << k_funcinfo << "unexpected Id " << id << " for Player tag - expected " << i << endl;
+		boError() << k_funcinfo << "unexpected PlayerId " << id << " for Player tag - expected " << i << endl;
 	}
 	QDomElement handler = p.namedItem("DataHandler").toElement();
 	if (handler.isNull()) {
@@ -641,34 +720,8 @@ bool BosonSaveLoad::convertSaveGameToPlayField(QMap<QString, QByteArray>& files)
 	return false;
  }
  QDomElement canvasRoot = canvasDoc.documentElement();
- QDomNodeList itemsList = canvasRoot.elementsByTagName("Items");
- for (unsigned int i = 0; i < itemsList.count(); i++) {
-	QDomElement items = itemsList.item(i).toElement();
-	bool ok = false;
-	unsigned int id = items.attribute("Id").toUInt(&ok);
-	if (!ok) {
-		boError() << k_funcinfo << "invalid value for Id attribute of Items tag" << endl;
-		return false;
-	}
-	if (id >= itemsList.count()) {
-		boError() << k_funcinfo << "invalid Id for Items tag: " << id << endl;
-		return false;
-	}
-	if (id != i) {
-		boError() << k_funcinfo << "unexpected Id " << id << " for Items tag - expected " << i << endl;
-		return false;
-	}
-	QDomNodeList itemList = items.elementsByTagName("Item");
-	for (unsigned int j = 0; j < itemList.count(); j++) {
-		QDomElement item = itemList.item(j).toElement();
-		item.setAttribute("Id", 0);
-	}
-	QDomElement handler = items.namedItem("DataHandler").toElement();
-	if (handler.isNull()) {
-		continue;
-	}
-	BosonPropertyXML::removeProperty(handler, BosonCanvas::IdNextItemId);
- }
+ canvasRoot.removeChild(canvasRoot.namedItem("Pathfinder"));
+ canvasRoot.appendChild(canvasDoc.createElement("Effects"));
 
  QByteArray kgameXML = kgameDoc.toCString();
  QByteArray playersXML = playersDoc.toCString();
@@ -677,7 +730,73 @@ bool BosonSaveLoad::convertSaveGameToPlayField(QMap<QString, QByteArray>& files)
  files.insert("kgame.xml", kgameXML);
  files.insert("players.xml", playersXML);
  files.insert("canvas.xml", canvasXML);
+
+ QStringList list = files.keys();
+ list = list.grep(QRegExp("^scripts\\/.*\\/data\\/"));
+ for (QStringList::Iterator it = list.begin(); it != list.end(); ++it) {
+	files.remove(*it);
+ }
  return true;
 }
 
 
+bool BosonSaveLoad::loadEventListenerScripts(const QMap<QString, QByteArray>& files)
+{
+ if (!boGame) {
+	return false;
+ }
+ if (!boGame->eventManager()) {
+	return false;
+ }
+ return boGame->eventManager()->loadListenerScripts(files);
+}
+
+bool BosonSaveLoad::saveEventListenerScripts(QMap<QString, QByteArray>* files)
+{
+ if (!boGame) {
+	return false;
+ }
+ if (!boGame->eventManager()) {
+	return false;
+ }
+ return boGame->eventManager()->saveListenerScripts(files);
+}
+
+bool BosonSaveLoad::convertPlayerIdsToIndices(QMap<QString, QByteArray>& files) const
+{
+ QStringList removeFiles;
+ QMap<QString, QByteArray> addFiles;
+ QRegExp hasPlayerId("-player_([0-9]+)");
+ for (QMap<QString, QByteArray>::iterator it = files.begin(); it != files.end(); ++it) {
+	int pos = hasPlayerId.search(it.key());
+	if (pos < 0) {
+		continue;
+	}
+	QString number = hasPlayerId.cap(1);
+	bool ok;
+	unsigned int id = number.toUInt(&ok);
+	if (!ok) {
+		boError() << k_funcinfo << it.key() << " does not contain a valid number" << endl;
+		return false;
+	}
+	Player* p = (Player*)boGame->findPlayer(id);
+	if (!p) {
+		boError() << k_funcinfo << "no player with id " << id << " in game" << endl;
+		return false;
+	}
+	QString file = it.key();
+	QByteArray b = it.data();
+	int index = boGame->playerList()->findRef(p);
+	file.replace(hasPlayerId, QString("-player_%1").arg(index));
+	removeFiles.append(it.key());
+	addFiles.insert(file, b);
+ }
+ for (QStringList::iterator it = removeFiles.begin(); it != removeFiles.end(); ++it) {
+	files.remove(*it);
+ }
+ for (QMap<QString, QByteArray>::iterator it = addFiles.begin(); it != addFiles.end(); ++it) {
+	files.insert(it.key(), it.data());
+ }
+ return true;
+}
+
