@@ -19,12 +19,12 @@
 #include <kdebug.h>
 #include <kconfig.h>
 #include <kapplication.h>
-#include <kfiledialog.h>
 
 #include "unitproperties.h"
 #include "bosonweapon.h"
 #include "pluginproperties.h"
 #include "bo3dtools.h"
+#include "bofiledialog.h"
 
 QString listToString(QValueList<unsigned long int> list)
 {
@@ -143,7 +143,7 @@ void BoUnitEditor::slotSaveUnit()
     QString dir = mUnitPath->text();
     QDir d;
     if(dir.isEmpty()) {
-	dir = KFileDialog::getExistingDirectory(QString::null, 0,
+	dir = BoFileDialog::getExistingDirectory(QString::null, 0,
 						i18n("Please select directory for your new unit"));
 	if(dir.isEmpty()) { // No directory was chosen
 	    return;
@@ -239,7 +239,6 @@ void BoUnitEditor::init()
     mUnitLoaded = false; // Bad hack
     mUnit = new UnitProperties(false);
     mSearchPaths = new BosonSearchPathsWidget;
-    mWeapons.setAutoDelete(true);
     mCurrentWeapon = -1;
     connect(mSearchPaths->mOkButton, SIGNAL(clicked()), this, SLOT(slotHideSearchPaths()));
     mSearchPaths->hide();
@@ -255,20 +254,18 @@ void BoUnitEditor::init()
     QRegExpValidator* v = new QRegExpValidator(QRegExp("[0-9,]*"), this);
     mUnitProducerList->setValidator(v);
     mUnitRequirements->setValidator(v);
-    mUnitDestroyedParticles->setValidator(v);
-    mUnitConstructedParticles->setValidator(v);
-    mWeaponShootParticles->setValidator(v);
-    mWeaponFlyParticles->setValidator(v);
-    mWeaponHitParticles->setValidator(v);
+    mUnitDestroyedEffects->setValidator(v);
+    mUnitConstructedEffects->setValidator(v);
+    mWeaponShootEffects->setValidator(v);
+    mWeaponFlyEffects->setValidator(v);
+    mWeaponHitEffects->setValidator(v);
     
     // Clear everything to default values
     if(mUnits.isEmpty()) {
 	mUnit->reset();
     } else {
 	mUnitsList->setSelected(0, true);
-	//slotLoadUnit(mUnits[0]);
     }
-    slotUpdateWidgets();
     mConfigChanged = false;
     updateConfigWidgets();
 }
@@ -342,7 +339,7 @@ void BoUnitEditor::slotOpenUnit()
     }
     
     // Open new unit
-    QString dir = KFileDialog::getExistingDirectory();
+    QString dir = BoFileDialog::getExistingDirectory();
     if((dir == QString::null) || (!QFile::exists(dir + "/index.unit"))) {
 	KMessageBox::error(this, i18n("No unit configuration file was found in this directory!"), i18n("Invalid directory!"));
 	return;
@@ -369,31 +366,31 @@ void BoUnitEditor::slotCanProduceChanged()
 
 void BoUnitEditor::slotUpdateUnitProperties()
 {
-    mUnit->clearPlugins(false);
+    mUnit->clearPlugins(true);
     // General page
     mUnit->setName(mUnitName->text());
-    mUnit->setId(mUnitId->value());
-    mUnit->setUnitWidth((unsigned int)mUnitWidth->value() * BO_TILE_SIZE);
-    mUnit->setUnitHeight((unsigned int)mUnitHeight->value() * BO_TILE_SIZE);
-    mUnit->setUnitDepth((unsigned int)mUnitDepth->value() * BO_TILE_SIZE);
+    mUnit->setTypeId(mUnitId->value());
+    mUnit->setUnitWidth((mUnitWidth->value()));
+    mUnit->setUnitHeight((mUnitHeight->value()));
+    mUnit->setUnitDepth(mUnitDepth->value());
     
     // Mobile/facility properties
     if(mUnitTypeFacility->isChecked()) {
-	mUnit->createFacilityProperties();
+	mUnit->setIsFacility(true);
 	mUnit->setConstructionSteps(mUnitConstructionSteps->value());
     } else {
-	mUnit->createMobileProperties();
-	mUnit->setSpeed(mUnitSpeed->value());
+	mUnit->setIsFacility(false);
+	mUnit->insertBoFixedBaseValue(bofixed(mUnitSpeed->value()), "Speed");
 	mUnit->setCanGoOnLand(mUnitCanGoOnLand->isChecked());
 	mUnit->setCanGoOnWater(mUnitCanGoOnWater->isChecked());
     }
     // Properties page
-    mUnit->setHealth(mUnitHealth->value());
-    mUnit->setArmor(mUnitArmor->value());
-    mUnit->setShields(mUnitShields->value());
-    mUnit->setMineralCost(mUnitMineralCost->value());
-    mUnit->setOilCost(mUnitOilCost->value());
-    mUnit->setSightRange(mUnitSight->value());
+    mUnit->insertULongBaseValue(mUnitHealth->value(), "Health");
+    mUnit->insertULongBaseValue(mUnitArmor->value(), "Armor");
+    mUnit->insertULongBaseValue(mUnitShields->value(), "Shields");
+    mUnit->insertULongBaseValue(mUnitMineralCost->value(), "MineralCost");
+    mUnit->insertULongBaseValue(mUnitOilCost->value(), "OilCost");
+    mUnit->insertULongBaseValue(mUnitSight->value(), "SightRange");
     mUnit->setTerrainType((UnitProperties::TerrainType)(mUnitTerrain->currentItem()));
     mUnit->setSupportMiniMap(mUnitSupportMiniMap->isChecked());
     // Weapons page
@@ -430,14 +427,14 @@ void BoUnitEditor::slotUpdateUnitProperties()
 	mUnit->addPlugin(p);
     }
     // Producing page
-    mUnit->setProductionTime(mUnitProductionTime->value());
+    mUnit->insertULongBaseValue(mUnitProductionTime->value(), "ProductionTime");
     mUnit->setProducer(mUnitProducer->value());
     mUnit->setRequirements(stringToList(mUnitRequirements->text()));
     // Mapping page
     if(mUnitTexturesList->childCount() > 0) {
 	QListViewItemIterator it(mUnitTexturesList);
 	QStringList textures;
-	for (; it.current(); ++it) {	    
+	for (; it.current(); ++it) {
 	    mUnit->addTextureMapping(it.current()->text(0), it.current()->text(1));
 	}
     }
@@ -448,16 +445,14 @@ void BoUnitEditor::slotUpdateUnitProperties()
     mUnit->addSound(SoundReportDestroyed, mUnitSoundReportDestroyed->text());
     mUnit->addSound(SoundReportUnderAttack, mUnitSoundReportUnderAttack->text());
     // Other page
-    mUnit->setDestroyedParticleSystemIds(stringToList(mUnitDestroyedParticles->text()));
-    mUnit->setConstructedParticleSystemIds(stringToList(mUnitConstructedParticles->text()));
+    mUnit->setDestroyedEffectIds(stringToList(mUnitDestroyedEffects->text()));
+    mUnit->setConstructedEffectIds(stringToList(mUnitConstructedEffects->text()));
     mUnit->setExplodingDamageRange(mUnitExplodingDamageRange->value());
     mUnit->setExplodingDamage(mUnitExplodingDamage->value());
-    BoVector3 hitpoint(mUnitHitPointX->value(), mUnitHitPointY->value(), mUnitHitPointZ->value());
-    hitpoint.cellToCanvas();
+    BoVector3Fixed hitpoint(mUnitHitPointX->value(), mUnitHitPointY->value(), mUnitHitPointZ->value());
     mUnit->setHitPoint(hitpoint);
 }
 
-
 void BoUnitEditor::slotUpdateWidgets()
 {
     // General page
@@ -468,23 +463,23 @@ void BoUnitEditor::slotUpdateWidgets()
     mUnitTypeFacility->setChecked(isFac);
     mUnitTypeMobile->setChecked(!isFac);
     slotTypeChanged();
-    mUnitWidth->setValue(mUnit->unitWidth() / (double)BO_TILE_SIZE);
-    mUnitHeight->setValue(mUnit->unitHeight() / (double)BO_TILE_SIZE);
-    mUnitDepth->setValue(mUnit->unitDepth() / (double)BO_TILE_SIZE);
+    mUnitWidth->setValue(mUnit->unitWidth());
+    mUnitHeight->setValue(mUnit->unitHeight());
+    mUnitDepth->setValue(mUnit->unitDepth());
     // Properties page
-    mUnitHealth->setValue(mUnit->health());
-    mUnitArmor->setValue(mUnit->armor());
-    mUnitShields->setValue(mUnit->shields());
-    mUnitMineralCost->setValue(mUnit->mineralCost());
-    mUnitOilCost->setValue(mUnit->oilCost());
-    mUnitSight->setValue(mUnit->sightRange());
+    mUnitHealth->setValue(mUnit->ulongBaseValue("Health"));
+    mUnitArmor->setValue(mUnit->ulongBaseValue("Armor"));
+    mUnitShields->setValue(mUnit->ulongBaseValue("Shields"));
+    mUnitMineralCost->setValue(mUnit->ulongBaseValue("MineralCost"));
+    mUnitOilCost->setValue(mUnit->ulongBaseValue("OilCost"));
+    mUnitSight->setValue(mUnit->ulongBaseValue("SightRange"));
     int terrain = (int)(mUnit->terrainType());
     mUnitTerrain->setCurrentItem(terrain);
     mUnitSupportMiniMap->setChecked(mUnit->supportMiniMap());
     // FIXME: UnitProperties only saves mobile *or* facility properties, but
     //  I'd like to have them both saved
     // TODO: This MUST be double, but Designer knows nothing about KDoubleNumInput
-    mUnitSpeed->setValue(mUnit->speed());
+    mUnitSpeed->setValue(mUnit->bofixedBaseValue("Speed"));
     mUnitCanGoOnLand->setChecked(mUnit->canGoOnLand());
     mUnitCanGoOnWater->setChecked(mUnit->canGoOnWater());
     mUnitConstructionSteps->setValue(mUnit->constructionSteps());
@@ -551,12 +546,11 @@ void BoUnitEditor::slotUpdateWidgets()
     mUnitSoundReportDestroyed->setText(mUnit->sound(SoundReportDestroyed));
     mUnitSoundReportUnderAttack->setText(mUnit->sound(SoundReportUnderAttack));
     // Other page
-    mUnitDestroyedParticles->setText(listToString(mUnit->destroyedParticleSystemIds()));
-    mUnitConstructedParticles->setText(listToString(mUnit->constructedParticleSystemIds()));
+    mUnitDestroyedEffects->setText(listToString(mUnit->destroyedEffectIds()));
+    mUnitConstructedEffects->setText(listToString(mUnit->constructedEffectIds()));
     mUnitExplodingDamageRange->setValue(mUnit->explodingDamageRange());
     mUnitExplodingDamage->setValue(mUnit->explodingDamage());
-    BoVector3 hitpoint = mUnit->hitPoint();
-    hitpoint.canvasToCell();
+    BoVector3Fixed hitpoint = mUnit->hitPoint();
     mUnitHitPointX->setValue(hitpoint.x());
     mUnitHitPointY->setValue(hitpoint.y());
     mUnitHitPointZ->setValue(hitpoint.z());
@@ -569,22 +563,20 @@ void BoUnitEditor::slotUpdateWeaponProps()
     }
     BosonWeaponProperties* w = mWeapons.at(mCurrentWeapon);
     w->setWeaponName(mWeaponName->text());
-    w->setDamage(mWeaponDamage->value());
-    w->setDamageRange(mWeaponDamageRange->value());
-    w->setFullDamageRange(mWeaponFullDamageRange->value());
-    w->setReloadingTime(mWeaponReload->value());
-    w->setRange(mWeaponRange->value());
+    w->insertLongWeaponBaseValue(mWeaponDamage->value(), "Damage");
+    w->insertBoFixedWeaponBaseValue(mWeaponDamageRange->value(), "DamageRange");
+    w->insertBoFixedWeaponBaseValue(mWeaponFullDamageRange->value(), "FullDamageRange");
+    w->insertULongWeaponBaseValue(mWeaponReload->value(), "Reload");
+    w->insertULongWeaponBaseValue(mWeaponRange->value(), "Range");
+    w->insertBoFixedWeaponBaseValue(mWeaponSpeed->value(), "Speed");
     w->setCanShootAtAirUnits(mWeaponCanShootAtAirUnits->isChecked());
     w->setCanShootAtLandUnits(mWeaponCanShootAtLandUnits->isChecked());
-    w->setSpeed((long unsigned int)mWeaponSpeed->value());
     w->setModelFileName(mWeaponModel->text());
     w->setHeight(mWeaponHeight->value());
-    BoVector3 offset(mWeaponOffsetX->value(), mWeaponOffsetY->value(), mWeaponOffsetZ->value());
-    offset.cellToCanvas();
-    w->setOffset(offset);
-    w->setShootParticleSystemIds(stringToList(mWeaponShootParticles->text()));
-    w->setFlyParticleSystemIds(stringToList(mWeaponFlyParticles->text()));
-    w->setHitParticleSystemIds(stringToList(mWeaponHitParticles->text()));
+    BoVector3Fixed offset(mWeaponOffsetX->value(), mWeaponOffsetY->value(), mWeaponOffsetZ->value());
+    w->setShootEffectIds(stringToList(mWeaponShootEffects->text()));
+    w->setFlyEffectIds(stringToList(mWeaponFlyEffects->text()));
+    w->setHitEffectIds(stringToList(mWeaponHitEffects->text()));
     w->setSound(SoundWeaponShoot, mWeaponSoundShoot->text());
 }
 
@@ -655,14 +647,13 @@ void BoUnitEditor::slotUpdateWeaponWidgets()
     mWeaponSpeed->setValue(w->speed());
     mWeaponModel->setText(w->modelFileName());
     mWeaponHeight->setValue(w->height());
-    BoVector3 o = w->offset();
-    o.canvasToCell();
+    BoVector3Fixed o = w->offset();
     mWeaponOffsetX->setValue(o.x());
     mWeaponOffsetY->setValue(o.y());
     mWeaponOffsetZ->setValue(o.z());
-    mWeaponShootParticles->setText(listToString(w->shootParticleSystemIds()));
-    mWeaponFlyParticles->setText(listToString(w->flyParticleSystemIds()));
-    mWeaponHitParticles->setText(listToString(w->hitParticleSystemIds()));
+    mWeaponShootEffects->setText(listToString(w->shootEffectIds()));
+    mWeaponFlyEffects->setText(listToString(w->flyEffectIds()));
+    mWeaponHitEffects->setText(listToString(w->hitEffectIds()));
     mWeaponSoundShoot->setText(w->sound(SoundWeaponShoot));
 }
 
