@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2003 Andreas Beckermann <b_mann@gmx.de>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,6 +21,8 @@
 
 #include <qstring.h>
 
+#include "boinfoglcache.h"
+
 class QStringList;
 class QWidget;
 class QVariant;
@@ -28,6 +30,9 @@ template<class T1, class T2> class QMap;
 class QDomElement;
 
 class BoInfoPrivate;
+
+#define MAKE_VERSION(a,b,c) ( ((a) << 16) | ((b) << 8) | (c) )
+
 /**
  * There has always been one problem in boson: speed. But not because boson is
  * so slow (yes it is - but less than 1 FPS is <em>not</em> boson's fault) but
@@ -78,6 +83,11 @@ public:
 	BoInfo();
 	~BoInfo();
 
+	BoInfoGLCache* gl() const
+	{
+		return mGLCache;
+	}
+
 	enum InfoEntries {
 		// boson specific
 		BosonVersion = 0,
@@ -98,6 +108,7 @@ public:
 		OpenGLRendererString = 22,
 		OpenGLExtensionsString = 23,
 		OpenGLValuesString = 24,
+		OpenGLVersion = 25,
 		GLUVersionString = 30,
 		GLUExtensionsString = 31,
 		GLXClientVersionString = 40,
@@ -277,38 +288,6 @@ public:
 		return getBool(HaveOpenGLData);
 	}
 
-	/**
-	 * Note that we don't have the OpenGL version available boson was
-	 * compiled with (at least not yet)!
-	 * @return A string containing the OpenGL version (with both, vendor
-	 * version such as mesa 3.2 and OpenGL version such as 1.1).
-	 **/
-	QString openGLVersionString() const
-	{
-		return getString(OpenGLVersionString);
-	}
-
-	/**
-	 * @return A string containing the vendor of the installed OpenGL
-	 * version (runtime). Usually mesa or nvidia.
-	 **/
-	QString openGLVendorString() const
-	{
-		return getString(OpenGLVendorString);
-	}
-
-	QString openGLRendererString() const
-	{
-		return getString(OpenGLRendererString);
-	}
-
-	/**
-	 * @return A list of available OpenGL extensions. This <em>could</em> be
-	 * used to do certain tasks only if the extension is available. But if
-	 * we actually want to do that we should cache the list somewhere.
-	 **/
-	QStringList openGLExtensions() const;
-
 	/**
 	 * @return A list of all implementation dependant OpenGL values (those
 	 * that can be retrieved using glGetIntegerv() and friends). An example
@@ -316,44 +295,6 @@ public:
 	 **/
 	QStringList openGLValues() const;
 
-	/**
-	 * @return A string describing the GLU version.
-	 **/
-	QString gluVersionString() const
-	{
-		return getString(GLUVersionString);
-	}
-	QStringList gluExtensions() const;
-
-	QString glXClientVersionString() const
-	{
-		return getString(GLXClientVersionString);
-	}
-	QString glXClientVendorString() const
-	{
-		return getString(GLXClientVendorString);
-	}
-	QStringList glXClientExtensions() const;
-	void glXVersion(int* major, int* minor) const;
-	QString glXServerVersionString() const
-	{
-		return getString(GLXServerVersionString);
-	}
-	QString glXServerVendorString() const
-	{
-		return getString(GLXServerVendorString);
-	}
-	QStringList glXServerExtensions() const;
-
-	/**
-	 * @return TRUE if the rendering context is direct (should be so, as
-	 * non-direct is slow)
-	 **/
-	bool isDirect() const
-	{
-		return getBool(IsDirect);
-	}
-
 	/**
 	 * @return TRUE if this object has information about X, otherwise FALSE
 	 * (e.g. when you call @ref update with a NULL widget)
@@ -378,7 +319,7 @@ public:
 	{
 		return getInt(BoInfo::XProtocolVersion);
 	}
-	
+
 	/**
 	 * @return The revision of the X protocol. Probably we don't need this.
 	 **/
@@ -493,7 +434,7 @@ public:
 	{
 		return getString(BoInfo::KernelModuleTDFXString);
 	}
-	
+
 	/**
 	 * @return If the OS was recognized AND the kernel module is loaded this
 	 * returns the string that made us believe the kernel module was loaded.
@@ -521,30 +462,8 @@ public:
 	 **/
 	bool haveMtrr() const;
 
-	/**
-	 * Note that this is <em>always<em> current data!!
-	 *
-	 * This entry is <em>not</em> updates by @ref updateOSInfo or friends
-	 * and it isn't stored into the boinfo log either! This is for debugging
-	 * purposes (think about it: memory usage changes over lifetime of the
-	 * programe, so logging it only once is useless)
-	 *
-	 * Also note that this value is <em>not</em> accurate! E.g. on linux it
-	 * gives the amount of memory pages that are allocated for the
-	 * application, but it doesn't say anything about how much of this
-	 * memory is actually in use! It may be possible that there are 30MB
-	 * allocated, but 20MB have been freed already (but still appear in this
-	 * number)! Keep this in mind when using this number
-	 *
-	 * @return The amount of memory in the data segment that is allocated by
-	 * the current application (boson/borender/bounit/...). On linux this is the
-	 * number after "VmData" in /proc/PID/status. The returned string should
-	 * be the number only, without the "VmData: ". In kb.
-	 **/
-	QString dataMemory() const;
-
-
 	QMap<int, QVariant> completeData() const;
+
 	/**
 	 * This goes through the BoInfo data and tries to ensure that everything
 	 * is fine for a user of the proprietary nvidia driver. They have a
@@ -590,7 +509,7 @@ public:
 	//   that.
 	//   // AB: we need the LoadModule() stuff *only*. if GLX isn't loaded,
 	//   then it is not listed in the extensions list.
-	// 
+	//
 protected:
 	void insert(int key, int value);
 	void insert(int key, unsigned int value);
@@ -618,6 +537,7 @@ protected:
 	void updateOSInfo();
 	void updateLibraryInfo();
 	void updateDevicesInfo();
+	void updateCachedValues();
 
 	/**
 	 * @return 0 On success, otherwise from @ref CharacterDevice, OR'ed
@@ -632,7 +552,9 @@ private:
 
 private:
 	BoInfoPrivate* d;
+	BoInfoGLCache* mGLCache;
 };
 
+
 #endif
 
