@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2003 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2003 Andreas Beckermann <b_mann@gmx.de>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,31 +20,30 @@
 #define QT_CLEAN_NAMESPACE
 
 #include "boinfo.h"
+
+#include "../../bomemory/bodummymemory.h"
 #include "bodebug.h"
 #include "boglquerystates.h"
+#include <bogl.h>
+#include <boglx.h>
 
 #include <qwidget.h>
 #include <qregexp.h>
+#include <qstringlist.h>
 
-#include <GL/gl.h>
-#include <GL/glu.h>
-#include <GL/glx.h>
 #include <X11/Xlib.h>
 
 void BoInfo::updateOpenGLInfo(QWidget* widget)
 {
  // OpenGL (warning: we don't have compile-time versions here!)
  QString extensions;
- insert(BoInfo::OpenGLVersionString, (const char*)glGetString(GL_VERSION));
- insert(BoInfo::OpenGLVendorString, (const char*)glGetString(GL_VENDOR));
- insert(BoInfo::OpenGLRendererString, (const char*)glGetString(GL_RENDERER));
- extensions = (const char*)glGetString(GL_EXTENSIONS);
- extensions.replace(QRegExp(" "), "\n");
- insert(BoInfo::OpenGLExtensionsString, extensions);
- insert(BoInfo::GLUVersionString, (const char*)gluGetString(GLU_VERSION));
- extensions = (const char*)gluGetString(GLU_EXTENSIONS);
- extensions.replace(QRegExp(" "), "\n");
- insert(BoInfo::GLUExtensionsString, extensions);
+ insert(BoInfo::OpenGLVersionString, boglGetOpenGLVersionString());
+ insert(BoInfo::OpenGLVendorString, boglGetOpenGLVendorString());
+ insert(BoInfo::OpenGLRendererString, boglGetOpenGLRendererString());
+ insert(BoInfo::OpenGLExtensionsString, boglGetOpenGLExtensions().join("\n"));
+ insert(BoInfo::OpenGLVersion, boglGetOpenGLVersion());
+ insert(BoInfo::GLUVersionString, boglGetGLUVersionString());
+ insert(BoInfo::GLUExtensionsString, boglGetGLUExtensions().join("\n"));
 
  GLXContext context = glXGetCurrentContext();
  if (!context) {
