@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2001 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2001 Andreas Beckermann (b_mann@gmx.de)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,7 +17,9 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 #include "kgameunitdebug.h"
+#include "kgameunitdebug.moc"
 
+#include "../bomemory/bodummymemory.h"
 #include "boson.h"
 #include "cell.h"
 #include "unit.h"
@@ -26,6 +28,8 @@
 #include "boitemlist.h"
 #include "bosoncanvas.h"
 #include "bodebug.h"
+#include "bo3dtools.h"
+#include "bosonpropertyxml.h"
 
 #include <klistview.h>
 #include <klistbox.h>
@@ -41,8 +45,48 @@
 #include <qpointarray.h>
 #include <qptrdict.h>
 #include <qptrvector.h>
+#include <qsplitter.h>
+#include <qpopupmenu.h>
+#include <qcursor.h>
+
+
+class QListViewItemNumber : public QListViewItem
+{
+public:
+	QListViewItemNumber(QListView* p) : QListViewItem(p)
+	{
+	}
+	QListViewItemNumber(QListViewItem* p) : QListViewItem(p)
+	{
+	}
+	virtual int compare(QListViewItem* i, int col, bool ascending) const
+	{
+		bool ok = true;
+		bool ok2 = true;
+		double n = key(col, ascending).toDouble(&ok);
+		double n2 = i->key(col, ascending).toDouble(&ok2);
+		// numbers first - then letters
+		if (ok && ok2) {
+			if (n == n2) {
+				return 0;
+			} else if (n > n2) {
+				return 1;
+			} else {
+				return -1;
+			}
+		} else if (ok) {
+			// this is a number, i is not. this comes first.
+			return -1;
+		} else if (ok2) {
+			// this is not a number, i is. i comes first.
+			return 1;
+		} else {
+			return QListViewItem::compare(i, col, ascending);
+		}
+	}
+};
+
 
-#include "kgameunitdebug.moc"
 
 class KGameUnitDebug::KGameUnitDebugPrivate
 {
@@ -51,6 +95,8 @@ public:
 	{
 		mBoson = 0;
 		mUnitList = 0;
+		mProperties = 0;
+		mWeaponProperties = 0;
 		mWaypoints = 0;
 		mProduction = 0;
 		mUnitsInRange = 0;
@@ -61,6 +107,8 @@ public:
 	Boson* mBoson;
 
 	KListView* mUnitList;
+	KListView* mProperties;
+	KListView* mWeaponProperties;
 	KListBox* mWaypoints;
 	KListView* mProduction;
 	KListView* mUnitsInRange;
@@ -90,9 +138,11 @@ KGameUnitDebug::KGameUnitDebug(QWidget* parent) : QWidget(parent)
 {
  d = new KGameUnitDebugPrivate;
  QVBoxLayout* topLayout = new QVBoxLayout(this);
- QHBoxLayout* layout = new QHBoxLayout(topLayout);
- 
- d->mUnitList = new KListView(this);
+ QSplitter* splitter = new QSplitter(this);
+ topLayout->addWidget(splitter);
+
+ d->mUnitList = new KListView(splitter);
+ d->mUnitList->setAllColumnsShowFocus(true);
  d->mId = d->mUnitList->addColumn(i18n("Id"));
  d->mOwner = d->mUnitList->addColumn(i18n("Owner"));
  d->mRTTI = d->mUnitList->addColumn(i18n("RTTI"));
@@ -104,58 +154,81 @@ KGameUnitDebug::KGameUnitDebug(QWidget* parent) : QWidget(parent)
  d->mHealth = d->mUnitList->addColumn(i18n("Health"));
 // d->mUnitList->addColumn(i18n("Costs"));
  d->mSpeed = d->mUnitList->addColumn(i18n("Speed"));
-// connect(d->mUnitList, SIGNAL(executed(QListBoxItem*)), 
+// connect(d->mUnitList, SIGNAL(executed(QListBoxItem*)),
 //		this, SLOT(slotSelectUnit(QListBoxItem*)));
  d->mWidth = d->mUnitList->addColumn(i18n("Width"));
  d->mHeight = d->mUnitList->addColumn(i18n("Height"));
  d->mBoundingRect = d->mUnitList->addColumn(i18n("BoundingRect"));
- layout->addWidget(d->mUnitList);
+ connect(d->mUnitList, SIGNAL(selectionChanged(QListViewItem*)),
+		this, SLOT(slotUnitSelected(QListViewItem*)));
+ connect(d->mUnitList, SIGNAL(contextMenuRequested(QListViewItem*, const QPoint&, int)),
+		this, SLOT(slotUnitListMenu(QListViewItem*, const QPoint&, int)));
 
- QVBoxLayout* l = new QVBoxLayout(layout);
- QVBoxLayout* waypointLayout = new QVBoxLayout(l);
- QLabel* waypointTitle = new QLabel(i18n("Waypoints"), this);
+ QSplitter* propertiesSplitter = new QSplitter(Vertical, splitter);
+ QWidget* propertiesBox = new QWidget(propertiesSplitter);
+ QVBoxLayout* propertiesLayout = new QVBoxLayout(propertiesBox);
+ QLabel* propertiesTitle = new QLabel(i18n("Properties"), propertiesBox);
+ propertiesLayout->addWidget(propertiesTitle, 0);
+ d->mProperties = new KListView(propertiesBox);
+ d->mProperties->setAllColumnsShowFocus(true);
+ d->mProperties->addColumn(i18n("Property"));
+ d->mProperties->addColumn(i18n("Id"));
+ d->mProperties->addColumn(i18n("Value"));
+ propertiesLayout->addWidget(d->mProperties, 1);
+
+ QWidget* weaponPropertiesBox = new QWidget(propertiesSplitter);
+ QVBoxLayout* weaponPropertiesLayout = new QVBoxLayout(weaponPropertiesBox);
+ QLabel* weaponPropertiesTitle = new QLabel(i18n("Weapon Properties"), weaponPropertiesBox);
+ weaponPropertiesLayout->addWidget(weaponPropertiesTitle, 0);
+ d->mWeaponProperties = new KListView(weaponPropertiesBox);
+ d->mWeaponProperties->setAllColumnsShowFocus(true);
+ d->mWeaponProperties->addColumn(i18n("Property"));
+ d->mWeaponProperties->addColumn(i18n("Id"));
+ d->mWeaponProperties->addColumn(i18n("Value"));
+ weaponPropertiesLayout->addWidget(d->mWeaponProperties, 1);
+
+ QVBox* vbox = new QVBox(splitter);
+ QWidget* waypointBox = new QWidget(vbox);
+ QVBoxLayout* waypointLayout = new QVBoxLayout(waypointBox);
+ QLabel* waypointTitle = new QLabel(i18n("Waypoints"), waypointBox);
  waypointLayout->addWidget(waypointTitle, 0);
- d->mWaypoints = new KListBox(this);
- connect(d->mUnitList, SIGNAL(selectionChanged(QListViewItem*)), this, SLOT(updateWaypoints(QListViewItem*)));
+ d->mWaypoints = new KListBox(waypointBox);
  waypointLayout->addWidget(d->mWaypoints, 1);
 
- QVBoxLayout* productionLayout = new QVBoxLayout(l);
- QLabel* productionTitle = new QLabel(i18n("Productions"), this);
+ QWidget* productionBox = new QWidget(vbox);
+ QVBoxLayout* productionLayout = new QVBoxLayout(productionBox);
+ QLabel* productionTitle = new QLabel(i18n("Productions"), productionBox);
  productionLayout->addWidget(productionTitle, 0);
- d->mProduction = new KListView(this);
+ d->mProduction = new KListView(productionBox);
+ d->mProduction->setAllColumnsShowFocus(true);
  d->mProduction->addColumn(i18n("Number"));
  d->mProduction->addColumn(i18n("TypeId"));
  d->mProduction->addColumn(i18n("ETA"));
- connect(d->mUnitList, SIGNAL(selectionChanged(QListViewItem*)), this, SLOT(updateProduction(QListViewItem*)));
  productionLayout->addWidget(d->mProduction, 1);
 
- QVGroupBox* collisionsBox = new QVGroupBox(i18n("Collisions"), this);
- layout->addWidget(collisionsBox);
+ QVGroupBox* collisionsBox = new QVGroupBox(i18n("Collisions"), splitter);
  QVBox* inRangeWidget = new QVBox(collisionsBox);
  (void)new QLabel(i18n("InRange:"), inRangeWidget);
  d->mUnitsInRange = new KListView(inRangeWidget);
+ d->mUnitsInRange->setAllColumnsShowFocus(true);
  d->mUnitsInRange->addColumn(i18n("ID"));
  d->mUnitsInRange->addColumn(i18n("Enemy"));
- connect(d->mUnitList, SIGNAL(selectionChanged(QListViewItem*)), this, SLOT(updateUnitsInRange(QListViewItem*)));
  QVBox* unitCollisionsWidget = new QVBox(collisionsBox);
  (void)new QLabel(i18n("UnitCollisions:"), unitCollisionsWidget);
  d->mUnitCollisions = new KListView(unitCollisionsWidget);
+ d->mUnitCollisions->setAllColumnsShowFocus(true);
  d->mUnitCollisions->addColumn(i18n("ID"));
  d->mUnitCollisions->addColumn(i18n("Exact"));
- connect(d->mUnitList, SIGNAL(selectionChanged(QListViewItem*)), this, SLOT(updateUnitCollisions(QListViewItem*)));
 
- d->mCells = new KListView(this);
- layout->addWidget(d->mCells);
+ QWidget* cellsBox = new QWidget(splitter);
+ QVBoxLayout* cellsLayout = new QVBoxLayout(cellsBox);
+ QLabel* cellsTitle = new QLabel(i18n("Cells"), cellsBox);
+ cellsLayout->addWidget(cellsTitle, 0);
+ d->mCells = new KListView(cellsBox);
+ d->mCells->setAllColumnsShowFocus(true);
  d->mCells->addColumn(i18n("X"));
  d->mCells->addColumn(i18n("Y"));
- connect(d->mUnitList, SIGNAL(selectionChanged(QListViewItem*)), this, SLOT(updateCells(QListViewItem*)));
- 
-
-/*
- d->mUnitView = new KListView(this);
- d->mUnitView->addColumn(i18n("Id"));
- d->mUnitView->addColumn(i18n("Name"));
- layout->addWidget(d->mUnitView);*/
+ cellsLayout->addWidget(d->mCells, 1);
 
  QPushButton* update = new QPushButton(i18n("Update"), this);
  connect(update, SIGNAL(pressed()), this, SLOT(slotUpdate()));
@@ -166,6 +239,8 @@ KGameUnitDebug::~KGameUnitDebug()
 {
  d->mUnits.clear();
  d->mItems.clear();
+ d->mProperties->clear();
+ d->mWeaponProperties->clear();
  d->mWaypoints->clear();
  d->mProduction->clear();
  d->mUnitsInRange->clear();
@@ -183,9 +258,10 @@ void KGameUnitDebug::setBoson(Boson* b)
 void KGameUnitDebug::slotUpdate()
 {
  d->mUnitList->clear();
-// d->mUnitView->clear();
  d->mUnits.clear();
  d->mItems.clear();
+ d->mProperties->clear();
+ d->mWeaponProperties->clear();
  d->mWaypoints->clear();
  d->mProduction->clear();
  d->mUnitsInRange->clear();
@@ -221,7 +297,7 @@ void KGameUnitDebug::slotUpdate()
 
 void KGameUnitDebug::addUnit(Unit* unit)
 {
- QListViewItem* item = new QListViewItem(d->mUnitList);
+ QListViewItem* item = new QListViewItemNumber(d->mUnitList);
  d->mItems.insert(unit, item);
  connect(unit->dataHandler(), SIGNAL(signalPropertyChanged(KGamePropertyBase*)),
 		this, SLOT(slotUnitPropertyChanged(KGamePropertyBase*)));
@@ -243,8 +319,12 @@ void KGameUnitDebug::update(QListViewItem* item, Unit* unit)
  item->setText(d->mWidth, QString::number(unit->width()));
  item->setText(d->mHeight, QString::number(unit->height()));
 
- QRect r = unit->boundingRect();
- item->setText(d->mBoundingRect, QString("%1,%2,%3,%4").arg(r.x()).arg(r.y()).arg(r.width()).arg(r.height()));
+ BoRectFixed r = unit->boundingRect();
+ item->setText(d->mBoundingRect, QString("%1,%2,%3,%4").
+		arg(r.x()).
+		arg(r.y()).
+		arg(r.width()).
+		arg(r.height()));
 }
 
 void KGameUnitDebug::slotUnitPropertyChanged(KGamePropertyBase* prop)
@@ -259,7 +339,7 @@ void KGameUnitDebug::slotUnitPropertyChanged(KGamePropertyBase* prop)
 		if (unit->dataHandler() == (KGamePropertyHandler*)sender()) {
 			break;
 		}
-		
+
 	}
  }
  if (!unit) {
@@ -274,34 +354,125 @@ void KGameUnitDebug::slotUnitPropertyChanged(KGamePropertyBase* prop)
  }
 }
 
-void KGameUnitDebug::updateWaypoints(QListViewItem* item)
+void KGameUnitDebug::slotUnitListMenu(QListViewItem*, const QPoint&, int)
+{
+ QPopupMenu menu(this);
+ QPopupMenu columns(this);
+ columns.setCheckable(true);
+
+ for (int i = 0; i < d->mUnitList->columns(); i++) {
+	QString text = d->mUnitList->columnText(i);
+	bool hidden = (d->mUnitList->columnWidth(i) == 0);
+	int id = columns.insertItem(text, this, SLOT(slotUnitListToggleShowColumn(int)));
+	columns.setItemParameter(id, i);
+	columns.setItemChecked(id, !hidden);
+ }
+ menu.insertItem(i18n("Show Column"), &columns);
+ menu.exec(QCursor::pos());
+}
+
+void KGameUnitDebug::slotUnitListToggleShowColumn(int index)
+{
+ if (index < 0) {
+	boError() << k_funcinfo << "index < 0: index=" << index << endl;
+	return;
+ }
+ if (index >= d->mUnitList->columns()) {
+	boError() << k_funcinfo << "index >= number of columns: index=" << index << endl;
+	return;
+ }
+ bool hidden = (d->mUnitList->columnWidth(index) == 0);
+ if (hidden) {
+	d->mUnitList->setColumnWidthMode(index, QListView::Maximum);
+	d->mUnitList->adjustColumn(index);
+ } else {
+	d->mUnitList->setColumnWidthMode(index, QListView::Manual);
+	d->mUnitList->hideColumn(index);
+ }
+}
+
+void KGameUnitDebug::slotUnitSelected(QListViewItem* item)
+{
+ Unit* unit = 0;
+ if (item) {
+	int id = item->text(0).toInt();
+	unit = d->mUnits[id];
+	if (!unit) {
+		boWarning() << k_funcinfo << "id " << id << " not found" << endl;
+	}
+ }
+ updateWaypoints(unit);
+ updateProduction(unit);
+ updateUnitsInRange(unit);
+ updateUnitCollisions(unit);
+ updateCells(unit);
+ updateProperties(unit);
+}
+
+void KGameUnitDebug::updateProperties(Unit* unit)
+{
+ d->mProperties->clear();
+ d->mWeaponProperties->clear();
+ if (!unit) {
+	return;
+ }
+
+ BosonCustomPropertyXML propertyXML;
+ {
+	KGamePropertyHandler* dataHandler = unit->dataHandler();
+	QIntDict<KGamePropertyBase>& dict = dataHandler->dict();
+	QIntDictIterator<KGamePropertyBase> it(dict);
+	while (it.current()) {
+		QString name = dataHandler->propertyName(it.current()->id());
+		QString id = QString::number(it.current()->id());
+		QString value = propertyXML.propertyValue(it.current());
+		QListViewItemNumber* item = new QListViewItemNumber(d->mProperties);
+		if (name.isEmpty()) {
+			name = i18n("Unknown");
+		}
+		item->setText(0, name);
+		item->setText(1, id);
+		item->setText(2, value);
+		++it;
+	}
+ }
+
+ {
+	KGamePropertyHandler* weaponDataHandler = unit->weaponDataHandler();
+	QIntDict<KGamePropertyBase>& dict = weaponDataHandler->dict();
+	QIntDictIterator<KGamePropertyBase> it(dict);
+	while (it.current()) {
+		QString name = weaponDataHandler->propertyName(it.current()->id());
+		QString id = QString::number(it.current()->id());
+		QString value = propertyXML.propertyValue(it.current());
+		QListViewItemNumber* item = new QListViewItemNumber(d->mWeaponProperties);
+		if (name.isEmpty()) {
+			name = i18n("Unknown");
+		}
+		item->setText(0, name);
+		item->setText(1, id);
+		item->setText(2, value);
+		++it;
+	}
+ }
+}
+
+void KGameUnitDebug::updateWaypoints(Unit* unit)
 {
  d->mWaypoints->clear();
- if (!item) {
-	return;
- }
- int id = item->text(0).toInt();
- Unit* unit = d->mUnits[id];
  if (!unit) {
-	boWarning() << k_lineinfo << "id " << id << " not found" << endl;
 	return;
  }
- QValueList<QPoint> points = unit->waypointList();
+ QValueList<BoVector2Fixed> points = unit->waypointList();
  for (unsigned int i = 0; i < points.count(); i++) {
 	(void)new QListBoxText(d->mWaypoints, i18n("x=%1 y=%2").arg(points[i].x()).arg(points[i].y()));
  }
 }
 
-void KGameUnitDebug::updateProduction(QListViewItem* item)
+void KGameUnitDebug::updateProduction(Unit* unit)
 {
  d->mProduction->clear();
- if (!item) {
-	return;
- }
- int id = item->text(0).toInt();
- Unit* unit = d->mUnits[id];
  if (!unit) {
-	boWarning() << k_lineinfo << "id " << id << " not found" << endl;
 	return;
  }
  ProductionPlugin* production = (ProductionPlugin*)unit->plugin(UnitPlugin::Production);
@@ -314,20 +485,14 @@ void KGameUnitDebug::updateProduction(QListViewItem* item)
 		item->setText(2, i18n("Ready")); // currently always ready
 		// TODO: also show ProductionType
 	}
- 
+
  }
 }
 
-void KGameUnitDebug::updateUnitsInRange(QListViewItem* item)
+void KGameUnitDebug::updateUnitsInRange(Unit* unit)
 {
  d->mUnitsInRange->clear();
- if (!item) {
-	return;
- }
- int id = item->text(0).toInt();
- Unit* unit = d->mUnits[id];
  if (!unit) {
-	boWarning() << k_lineinfo << "id " << id << " not found" << endl;
 	return;
  }
  /*BoItemList inRange = unit->unitsInRange();
@@ -355,16 +520,10 @@ void KGameUnitDebug::updateUnitsInRange(QListViewItem* item)
  }*/
 }
 
-void KGameUnitDebug::updateUnitCollisions(QListViewItem* item)
+void KGameUnitDebug::updateUnitCollisions(Unit* unit)
 {
  d->mUnitCollisions->clear();
- if (!item) {
-	return;
- }
- int id = item->text(0).toInt();
- Unit* unit = d->mUnits[id];
  if (!unit) {
-	boWarning() << k_lineinfo << "id " << id << " not found" << endl;
 	return;
  }
  QValueList<Unit*> collisionsFalse = unit->unitCollisions(false);
@@ -387,16 +546,10 @@ void KGameUnitDebug::updateUnitCollisions(QListViewItem* item)
  }
 }
 
-void KGameUnitDebug::updateCells(QListViewItem* item)
+void KGameUnitDebug::updateCells(Unit* unit)
 {
  d->mCells->clear();
- if (!item) {
-	return;
- }
- int id = item->text(0).toInt();
- Unit* unit = d->mUnits[id];
  if (!unit) {
-	boWarning() << k_lineinfo << "id " << id << " not found" << endl;
 	return;
  }
  const QPtrVector<Cell>* cells = unit->cells();
