@@ -1,6 +1,7 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2001 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2001-2005 Andreas Beckermann (b_mann@gmx.de)
+    Copyright (C) 2001-2005 Rivo Laks (rivolaks@hot.ee)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,11 +18,12 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 #include "optionswidgets.h"
-#include "bosonconfig.h"
+#include "optionswidgets.moc"
 
+#include "../bomemory/bodummymemory.h"
+#include "bosonconfig.h"
 #include "bosoncursor.h"
 #include "bosonmodeltextures.h"
-#include "bosontexturearray.h"
 #include "boson.h"
 #include "defines.h"
 #include "bodebug.h"
@@ -30,9 +32,10 @@
 #include "bo3dtools.h"
 #include "bofullscreen.h"
 #include "bomeshrenderermanager.h"
-#include "bosonfont/bosonglfont.h"
-#include "bosonfont/bosonglfontchooser.h"
+#include "bowaterrenderer.h"
 #include "info/boinfo.h"
+#include "botexture.h"
+#include "bosongroundthemedata.h"
 
 #include <kapplication.h>
 #include <klocale.h>
@@ -49,7 +52,15 @@
 #include <qpushbutton.h>
 #include <qlineedit.h>
 
-#include "optionswidgets.moc"
+
+// we use libufo fonts now.
+// TODO: make them configurable
+// TODO: port bosonfont to libufo
+#define BOSONFONT 0
+#if BOSONFONT
+#include "bosonfont/bosonglfont.h"
+#include "bosonfont/bosonglfontchooser.h"
+#endif
 
 OptionsWidget::OptionsWidget()
 {
@@ -78,16 +89,8 @@ GeneralOptions::GeneralOptions(QWidget* parent) : QVBox(parent), OptionsWidget()
  mMiniMapScale->setLabel(i18n("Mini Map scale factor"));
 
 
- QHBox* hbox = new QHBox(this);
- (void)new QLabel(i18n("Command frame background pixmap"), hbox);
- mCmdBackground = new QComboBox(hbox);
- mCmdBackgrounds = KGlobal::dirs()->findAllResources("data", "boson/themes/ui/*/cmdpanel*.png");
- mCmdBackground->insertItem(i18n("None"));
- //TODO: display filename only... - not the complete path
- mCmdBackground->insertStringList(mCmdBackgrounds);
-
  mRMBMovesWithAttack = new QCheckBox("Units attack enemies in sight while moving", this);
- mRMBMovesWithAttack->setChecked(boConfig->RMBMovesWithAttack());
+ mRMBMovesWithAttack->setChecked(boConfig->boolValue("RMBMovesWithAttack"));
 }
 
 GeneralOptions::~GeneralOptions()
@@ -105,13 +108,10 @@ void GeneralOptions::apply()
  if (mGameSpeed->value() != game()->gameSpeed()) {
 	game()->slotSetGameSpeed(mGameSpeed->value());
  }
- boConfig->setMiniMapScale(mMiniMapScale->value());
+ boConfig->setDoubleValue("MiniMapScale", mMiniMapScale->value());
  QString file;
- if (mCmdBackground->currentItem() > 0) {
-	file = mCmdBackgrounds[mCmdBackground->currentItem() - 1];
- }
- emit signalCmdBackgroundChanged(file);
- boConfig->setRMBMovesWithAttack(mRMBMovesWithAttack->isChecked());
+
+ boConfig->setBoolValue("RMBMovesWithAttack", mRMBMovesWithAttack->isChecked());
  boDebug(210) << k_funcinfo << "done" << endl;
 }
 
@@ -122,16 +122,14 @@ void GeneralOptions::load()
 	return;
  }
  setGameSpeed(game()->gameSpeed());
- setMiniMapScale(boConfig->miniMapScale());
- setRMBMovesWithAttack(boConfig->RMBMovesWithAttack());
- // TODO: cmdbackground
+ setMiniMapScale(boConfig->doubleValue("MiniMapScale"));
+ setRMBMovesWithAttack(boConfig->boolValue("RMBMovesWithAttack"));
 }
 
 void GeneralOptions::setDefaults()
 {
  setGameSpeed(DEFAULT_GAME_SPEED);
  setMiniMapScale(DEFAULT_MINIMAP_SCALE);
- setCmdBackground(QString::null);
  setRMBMovesWithAttack(DEFAULT_RMB_MOVES_WITH_ATTACK);
 }
 
@@ -145,16 +143,6 @@ void GeneralOptions::setMiniMapScale(double scale)
  mMiniMapScale->setValue(scale);
 }
 
-void GeneralOptions::setCmdBackground(const QString& file)
-{
- if (file.isEmpty() || mCmdBackgrounds.findIndex(file) < 0) {
-	mCmdBackground->setCurrentItem(0);
- } else {
-	int index = mCmdBackgrounds.findIndex(file);
-	mCmdBackground->setCurrentItem(index + 1);
- }
-}
-
 void GeneralOptions::setRMBMovesWithAttack(bool attack)
 {
  mRMBMovesWithAttack->setChecked(attack);
@@ -221,19 +209,18 @@ void CursorOptions::setDefaults()
 
 void CursorOptions::load()
 {
- setCursor((CursorMode)boConfig->cursorMode());
+ setCursor((CursorMode)boConfig->intValue("CursorMode"));
  int dirIndex = -1;
- if (boConfig->cursorDir().isNull()) {
+ if (boConfig->stringValue("CursorDir").isNull()) {
 	dirIndex = 0;
  } else {
-	dirIndex = BosonCursor::availableThemes().findIndex(boConfig->cursorDir());
+	dirIndex = BosonCursor::availableThemes().findIndex(boConfig->stringValue("CursorDir"));
  }
  if (dirIndex < 0) {
-	boWarning() << k_funcinfo << "could not find cusor theme " << boConfig->cursorDir() << endl;
+	boWarning() << k_funcinfo << "could not find cusor theme " << boConfig->stringValue("CursorDir") << endl;
 	dirIndex = 0;
  }
  mCursorTheme->setCurrentItem(dirIndex);
- QString dir = boConfig->cursorDir();
 }
 
 void CursorOptions::setCursor(CursorMode mode)
@@ -296,18 +283,18 @@ ScrollingOptions::~ScrollingOptions()
 void ScrollingOptions::apply()
 {
  boDebug(210) << k_funcinfo << endl;
- boConfig->setRMBMove(mRMBScrolling->isChecked());
- boConfig->setMMBMove(mMMBScrolling->isChecked());
+ boConfig->setBoolValue("RMBMove", mRMBScrolling->isChecked());
+ boConfig->setBoolValue("MMBMove", mMMBScrolling->isChecked());
  if (mCursorEdgeSensity->value() < 0) {
 	mCursorEdgeSensity->setValue(0);
  }
- boConfig->setCursorEdgeSensity(mCursorEdgeSensity->value());
+ boConfig->setUIntValue("CursorEdgeSensity", mCursorEdgeSensity->value());
  if (mArrowSpeed->value() < 0) {
 	mArrowSpeed->setValue(0);
  }
- boConfig->setArrowKeyStep(mArrowSpeed->value());
- boConfig->setMouseWheelAction((CameraAction)(mMouseWheelAction->currentItem()));
- boConfig->setMouseWheelShiftAction((CameraAction)(mMouseWheelShiftAction->currentItem()));
+ boConfig->setUIntValue("ArrowKeyStep", mArrowSpeed->value());
+ boConfig->setIntValue("MouseWheelAction", (CameraAction)(mMouseWheelAction->currentItem()));
+ boConfig->setIntValue("MouseWheelShiftAction", (CameraAction)(mMouseWheelShiftAction->currentItem()));
  boDebug(210) << k_funcinfo << "done" << endl;
 }
 
@@ -323,12 +310,12 @@ void ScrollingOptions::setDefaults()
 
 void ScrollingOptions::load()
 {
- setArrowScrollSpeed(boConfig->arrowKeyStep());
- setCursorEdgeSensity(boConfig->cursorEdgeSensity());
- setRMBScrolling(boConfig->rmbMove());
- setMMBScrolling(boConfig->mmbMove());
- setMouseWheelAction(boConfig->mouseWheelAction());
- setMouseWheelShiftAction(boConfig->mouseWheelShiftAction());
+ setArrowScrollSpeed(boConfig->uintValue("ArrowKeyStep"));
+ setCursorEdgeSensity(boConfig->uintValue("CursorEdgeSensity"));
+ setRMBScrolling(boConfig->boolValue("RMBMove"));
+ setMMBScrolling(boConfig->boolValue("MMBMove"));
+ setMouseWheelAction((CameraAction)boConfig->intValue("MouseWheelAction"));
+ setMouseWheelShiftAction((CameraAction)boConfig->intValue("MouseWheelShiftAction"));
 }
 
 void ScrollingOptions::setCursorEdgeSensity(int s)
@@ -397,7 +384,7 @@ void SoundOptions::apply()
  for (; it != mCheckBox2UnitSoundEvent.end(); ++it) {
 	boConfig->setUnitSoundActivated(it.data(), !it.key()->isChecked());
  }
- boConfig->setDeactivateWeaponSounds(weaponsounds->isChecked());
+ boConfig->setBoolValue("DeactivateWeaponSounds", weaponsounds->isChecked());
  boDebug(210) << k_funcinfo << "done" << endl;
 }
 
@@ -413,7 +400,7 @@ void SoundOptions::setDefaults()
 void SoundOptions::load()
 {
  setUnitSoundsDeactivated(boConfig);
- weaponsounds->setChecked(boConfig->deactivateWeaponSounds());
+ weaponsounds->setChecked(boConfig->boolValue("DeactivateWeaponSounds"));
 }
 
 void SoundOptions::setUnitSoundsDeactivated(BosonConfig* conf)
@@ -463,12 +450,14 @@ OpenGLOptions::OpenGLOptions(QWidget* parent) : QVBox(parent), OptionsWidget()
  mEnableATIDepthWorkaround->setChecked(false);
  slotEnableATIDepthWorkaround(false);
 
+#if BOSONFONT
  QHBox* fontBox = new QHBox(this);
  (void)new QLabel(i18n("Font: "), fontBox);
  mFont = new QPushButton(fontBox);
  mFontChanged = false;
  mFontInfo = new BoFontInfo();
  connect(mFont, SIGNAL(clicked()), this, SLOT(slotChangeFont()));
+#endif
 
  QHBox* resolutionBox = new QHBox(this);
  (void)new QLabel(i18n("Resolution:"), resolutionBox);
@@ -494,34 +483,38 @@ OpenGLOptions::OpenGLOptions(QWidget* parent) : QVBox(parent), OptionsWidget()
  mUpdateInterval->setRange(1, 100);
  mUpdateInterval->setLabel(i18n("Update interval (low values hurt performance)"));
  QToolTip::add(mUpdateInterval, i18n("The update interval specifies after how many milli seconds the scene gets re-rendered and therefore directly influence the frames per seconds. But low values prevent boson from doing other important tasks and therefore you might end up in a game that takes several seconds until your units react to your commands! 20ms are usually a good value."));
- mModelTexturesMipmaps = new QCheckBox(mAdvanced);
- mModelTexturesMipmaps->setText(i18n("Use mipmaps for model textures"));
- QToolTip::add(mModelTexturesMipmaps, i18n("With mipmapping disabled units often look ugly - but it consumes less memory and therefore might improve rendering speed."));
 
  hbox = new QHBox(mAdvanced);
- (void)new QLabel(i18n("Texture Magnification Filter"), hbox);
- mMagnificationFilter = new QComboBox(hbox);
- mMagnificationFilter->insertItem(i18n("Use GL_NEAREST (fastest)"));
- mMagnificationFilter->insertItem(i18n("Use GL_LINEAR (best quality)"));
- QToolTip::add(mMagnificationFilter, i18n("This selects which filter method is used when the textures need to be displayed at a bigger size than they are stored at. This applies to all textures.")); // AB: also note how big the impact on speed is!
+ (void)new QLabel(i18n("Texture Filter"), hbox);
+ mTextureFilter = new QComboBox(hbox);
+ //mTextureFilter->insertItem(i18n("Use GL_NEAREST (fastest)"));
+ mTextureFilter->insertItem(i18n("Use GL_LINEAR"));
+ //mTextureFilter->insertItem(i18n("Use GL_NEAREST_MIPMAP_NEAREST"));
+ //mTextureFilter->insertItem(i18n("Use GL_NEAREST_MIPMAP_LINEAR"));
+ mTextureFilter->insertItem(i18n("Use GL_LINEAR_MIPMAP_NEAREST (bilinear)"));
+ mTextureFilter->insertItem(i18n("Use GL_LINEAR_MIPMAP_LINEAR (trilinear)"));
+ if (boTextureManager->maxTextureAnisotropy() >= 2) {
+	mTextureFilter->insertItem(i18n("Low anisotropic"));
+ }
+ if (boTextureManager->maxTextureAnisotropy() >= 4) {
+	mTextureFilter->insertItem(i18n("Medium anisotropic"));
+ }
+ if (boTextureManager->maxTextureAnisotropy() >= 8) {
+	mTextureFilter->insertItem(i18n("High anisotropic"));
+ }
+ if (boTextureManager->maxTextureAnisotropy() >= 16) {
+	mTextureFilter->insertItem(i18n("Very high anisotropic"));
+ }
 
- hbox = new QHBox(mAdvanced);
- (void)new QLabel(i18n("Texture Minification Filter"), hbox);
- mMinificationFilter = new QComboBox(hbox);
- mMinificationFilter->insertItem(i18n("Use GL_NEAREST (fastest)"));
- mMinificationFilter->insertItem(i18n("Use GL_LINEAR (best quality)"));
- QToolTip::add(mMinificationFilter, i18n("This selects which filter method is used when the textures need to be displayed at a smaller size than they are stored at. This applies to not-mipmapped textures only.")); // AB: also note how big the impact on speed is!
+ QToolTip::add(mTextureFilter, i18n("Selects texture filtering method\nGL_LINEAR_MIPMAP_LINEAR looks very good.\nAnisotropic modes look even better but are slower."));
 
- hbox = new QHBox(mAdvanced);
- (void)new QLabel(i18n("Texture Mipmap Minification Filter"), hbox);
- mMipmapMinificationFilter = new QComboBox(hbox);
- mMipmapMinificationFilter->insertItem(i18n("Use GL_NEAREST (fastest)"));
- mMipmapMinificationFilter->insertItem(i18n("Use GL_LINEAR"));
- mMipmapMinificationFilter->insertItem(i18n("Use GL_NEAREST_MIPMAP_NEAREST"));
- mMipmapMinificationFilter->insertItem(i18n("Use GL_NEAREST_MIPMAP_LINEAR"));
- mMipmapMinificationFilter->insertItem(i18n("Use GL_LINEAR_MIPMAP_NEAREST"));
- mMipmapMinificationFilter->insertItem(i18n("Use GL_LINEAR_MIPMAP_LINEAR (best quality)"));
- QToolTip::add(mMipmapMinificationFilter, i18n("This selects which filter method is used when the textures need to be displayed at a smaller size than they are stored at. This applies to mipmapped textures (i.e. model textures) only.\nNote: The speed GL_*_MIPMAP_* will probably be noticebly slower, but it's quality is way better!"));
+ mUseCompressedTextures = new QCheckBox(mAdvanced);
+ mUseCompressedTextures->setText(i18n("Use compressed textures"));
+ QToolTip::add(mUseCompressedTextures, i18n("Compressing textures reduces amount of memory used\nat the expense of very slight quality loss."));
+
+ mUseColoredMipmaps = new QCheckBox(mAdvanced);
+ mUseColoredMipmaps->setText(i18n("Use colored mipmaps"));
+ QToolTip::add(mUseColoredMipmaps, i18n("Colors different mipmap levels so you can make difference between them.\nDo not enable this unless you know what you're doing."));
 
  mUseLight = new QCheckBox(mAdvanced);
  mUseLight->setText(i18n("Enable light"));
@@ -533,6 +526,8 @@ OpenGLOptions::OpenGLOptions(QWidget* parent) : QVBox(parent), OptionsWidget()
  hbox = new QHBox(mAdvanced);
  (void)new QLabel(i18n("Ground render:"), hbox);
  mGroundRenderer = new QComboBox(hbox);
+ mUseGroundShaders = new QCheckBox(i18n("Use shaders for ground rendering"), mAdvanced);
+ mUseGroundShaders->setEnabled(BosonGroundThemeData::shadersSupported());
 
  mUseLOD = new QCheckBox(i18n("Use level of detail"), mAdvanced);
  hbox = new QHBox(mAdvanced);
@@ -601,13 +596,13 @@ void OpenGLOptions::setRenderingSpeed(int speed)
 		boWarning(210) << k_funcinfo << "invalid value: " << speed << endl;
 	case Defaults:
 		setUpdateInterval(DEFAULT_UPDATE_INTERVAL);
-		mModelTexturesMipmaps->setChecked(DEFAULT_USE_MIPMAPS_FOR_MODELS);
-		setMagnificationFilter(DEFAULT_MAGNIFICATION_FILTER);
-		setMinificationFilter(DEFAULT_MINIFICATION_FILTER);
-		setMipmapMinificationFilter(DEFAULT_MIPMAP_MINIFICATION_FILTER);
+		setTextureFilter(DEFAULT_TEXTURE_FILTER);
+		mUseCompressedTextures->setChecked(DEFAULT_TEXTURE_COMPRESSION);
+		mUseColoredMipmaps->setChecked(false);
 		mUseLight->setChecked(DEFAULT_USE_LIGHT);
 		mUseMaterials->setChecked(DEFAULT_USE_MATERIALS);
 		setCurrentGroundRenderer(DEFAULT_GROUND_RENDERER);
+		mUseGroundShaders->setChecked(false);
 		setCurrentMeshRenderer(DEFAULT_MESH_RENDERER);
 		setUseLOD(true);
 		setDefaultLOD(0);
@@ -616,13 +611,13 @@ void OpenGLOptions::setRenderingSpeed(int speed)
 	case BestQuality:
 		// we mostly use defaults here.
 		setUpdateInterval(DEFAULT_UPDATE_INTERVAL);
-		mModelTexturesMipmaps->setChecked(true);
-		setMagnificationFilter(DEFAULT_MAGNIFICATION_FILTER);
-		setMinificationFilter(DEFAULT_MINIFICATION_FILTER);
-		setMipmapMinificationFilter(DEFAULT_MIPMAP_MINIFICATION_FILTER);
+		setTextureFilter(DEFAULT_TEXTURE_FILTER);
+		mUseCompressedTextures->setChecked(DEFAULT_TEXTURE_COMPRESSION);
+		mUseColoredMipmaps->setChecked(false);
 		mUseLight->setChecked(DEFAULT_USE_LIGHT);
 		mUseMaterials->setChecked(DEFAULT_USE_MATERIALS);
 		setCurrentGroundRenderer(DEFAULT_GROUND_RENDERER);
+		mUseGroundShaders->setChecked(true);
 		setCurrentMeshRenderer(DEFAULT_MESH_RENDERER);
 		setUseLOD(true);
 		setDefaultLOD(0);
@@ -630,19 +625,19 @@ void OpenGLOptions::setRenderingSpeed(int speed)
 		break;
 	case Fastest:
 		setUpdateInterval(DEFAULT_UPDATE_INTERVAL);
-		mModelTexturesMipmaps->setChecked(false);
-		setMagnificationFilter(GL_NEAREST);
-		setMinificationFilter(GL_NEAREST);
-		setMipmapMinificationFilter(GL_NEAREST);
+		setTextureFilter(GL_LINEAR);  // Use bilinear maybe?
+		mUseCompressedTextures->setChecked(true);
+		mUseColoredMipmaps->setChecked(false);
 		mUseLight->setChecked(false);
 		mUseMaterials->setChecked(false);
 		setCurrentGroundRenderer("BoFastGroundRenderer");
+		mUseGroundShaders->setChecked(false);
 		setCurrentMeshRenderer(DEFAULT_MESH_RENDERER);
 		setUseLOD(true);
 		setDefaultLOD(5000);
 		mSmoothShading->setChecked(false);
 		break;
-	
+
  }
  mRenderingSpeed->setCurrentItem(renderingSpeedToIndex(speed));
 }
@@ -658,6 +653,7 @@ void OpenGLOptions::slotShowDetails(bool show)
 
 void OpenGLOptions::slotChangeFont()
 {
+#if BOSONFONT
  BoFontInfo f = *mFontInfo;
  int result = BosonGLFontChooser::getFont(f, this);
  if (result == QDialog::Accepted) {
@@ -665,6 +661,7 @@ void OpenGLOptions::slotChangeFont()
 	mFontChanged = true;
 	mFont->setText(mFontInfo->guiName());
  }
+#endif
 }
 
 void OpenGLOptions::apply()
@@ -672,58 +669,58 @@ void OpenGLOptions::apply()
  boDebug(210) << k_funcinfo << endl;
  boConfig->setIntValue("RenderingSpeed", renderingSpeed());
 
- bool reloadModelTextures = false;
+ QString changesThatNeedRestart;
  bool resetTexParameter = false;
- boConfig->setUpdateInterval((unsigned int)mUpdateInterval->value());
- if (boConfig->modelTexturesMipmaps() != mModelTexturesMipmaps->isChecked()) {
-	boConfig->setModelTexturesMipmaps(mModelTexturesMipmaps->isChecked());
-	reloadModelTextures = true;
- }
- if (boConfig->magnificationFilter() != magnificationFilter()) {
-	boConfig->setMagnificationFilter(magnificationFilter());
+ bool reloadTextures = false;
+ boConfig->setUIntValue("GLUpdateInterval", (unsigned int)mUpdateInterval->value());
+ if (boConfig->intValue("TextureFilter") != textureFilter()) {
+	boConfig->setIntValue("TextureFilter", textureFilter());
 	resetTexParameter = true;
+	// TODO: reload only if mipmapping was turned on/off
+	reloadTextures = true;
  }
- if (boConfig->minificationFilter() != minificationFilter()) {
-	boConfig->setMinificationFilter(minificationFilter());
+ if (boConfig->intValue("TextureAnisotropy") != textureAnisotropy()) {
+	boConfig->setIntValue("TextureAnisotropy", textureAnisotropy());
 	resetTexParameter = true;
  }
- if (boConfig->mipmapMinificationFilter() != mipmapMinificationFilter()) {
-	boConfig->setMipmapMinificationFilter(mipmapMinificationFilter());
-	// only models use mipmaps (if at all)
-	if (boConfig->modelTexturesMipmaps()) {
-		resetTexParameter = true;
-	}
- }
 
- if (reloadModelTextures) {
-	int r = KMessageBox::questionYesNo(this, i18n("You need to reload the model textures to see your changes. Do you want to reload now (takes some time)?"));
-	reloadModelTextures = (r == KMessageBox::Yes);
-	if (reloadModelTextures) {
-		boDebug(210) << k_funcinfo << "reloading all textures" << endl;
-		BosonModelTextures::modelTextures()->reloadTextures();
-	}
+ if(boConfig->boolValue("TextureCompression") != mUseCompressedTextures->isChecked()) {
+	boConfig->setBoolValue("TextureCompression", mUseCompressedTextures->isChecked());
+	reloadTextures = true;
+ }
+ if(boConfig->boolValue("TextureColorMipmaps") != mUseColoredMipmaps->isChecked()) {
+	boConfig->setBoolValue("TextureColorMipmaps", mUseColoredMipmaps->isChecked());
+	reloadTextures = true;
+ }
+
+ if(!changesThatNeedRestart.isEmpty()) {
+	KMessageBox::information(this, i18n("Following changes take effect after restart:\n%1").arg(changesThatNeedRestart));
  }
  if (resetTexParameter) {
 	// maybe display a message box now, asking for permission to reset the
 	// parameters. currently we just reset them - leave it at this as long
 	// as no problems appear
-	if (resetTexParameter) {
-		boDebug(210) << k_funcinfo << "resetting all textures parameters" << endl;
-		BosonTextureArray::resetAllTexParameter();
-	}
+	boDebug(210) << k_funcinfo << "resetting all textures parameters" << endl;
+	boTextureManager->textureFilterChanged();
+ }
+ if (reloadTextures) {
+	boDebug(210) << k_funcinfo << "reloading all textures" << endl;
+	boTextureManager->reloadTextures();
  }
 
- boConfig->setAlignSelectionBoxes(mAlignSelectBoxes->isChecked());
- boConfig->setUseLight(mUseLight->isChecked());
- boConfig->setUseMaterials(mUseMaterials->isChecked());
- boConfig->setUseLOD(useLOD());
+ boConfig->setBoolValue("AlignSelectionBoxes", mAlignSelectBoxes->isChecked());
+ boConfig->setBoolValue("UseLight", mUseLight->isChecked());
+ boConfig->setBoolValue("UseMaterials", mUseMaterials->isChecked());
+ boConfig->setBoolValue("UseLOD", useLOD());
  boConfig->setUIntValue("DefaultLOD", defaultLOD());
 
+#if BOSONFONT
  if (mFontChanged) {
 	boConfig->setStringValue("GLFont", mFontInfo->toString());
 	mFontChanged = false;
 	emit signalFontChanged(*mFontInfo);
  }
+#endif
 
 
  boConfig->setBoolValue("EnableATIDepthWorkaround", mEnableATIDepthWorkaround->isChecked());
@@ -764,6 +761,8 @@ void OpenGLOptions::apply()
 		kapp->exit(1);
 	}
  }
+ boConfig->setBoolValue("UseGroundShaders", mUseGroundShaders->isChecked());
+ BosonGroundThemeData::setUseGroundShaders(mUseGroundShaders->isChecked());
 
 
  emit signalOpenGLSettingsUpdated();
@@ -777,14 +776,14 @@ void OpenGLOptions::setDefaults()
  slotRenderingSpeedChanged(0);
 
  setUpdateInterval(DEFAULT_UPDATE_INTERVAL);
- mModelTexturesMipmaps->setChecked(DEFAULT_USE_MIPMAPS_FOR_MODELS);
- setMagnificationFilter(DEFAULT_MAGNIFICATION_FILTER);
- setMinificationFilter(DEFAULT_MINIFICATION_FILTER);
- setMipmapMinificationFilter(DEFAULT_MIPMAP_MINIFICATION_FILTER);
+ setTextureFilter(DEFAULT_TEXTURE_FILTER);
+ mUseCompressedTextures->setChecked(DEFAULT_TEXTURE_COMPRESSION);
+ mUseColoredMipmaps->setChecked(false);
  setAlignSelectionBoxes(DEFAULT_ALIGN_SELECTION_BOXES);
  mUseLight->setChecked(DEFAULT_USE_LIGHT);
  mUseMaterials->setChecked(DEFAULT_USE_MATERIALS);
  setCurrentGroundRenderer(DEFAULT_GROUND_RENDERER);
+ mUseGroundShaders->setChecked(false);
  setCurrentMeshRenderer(DEFAULT_MESH_RENDERER);
  setUseLOD(DEFAULT_USE_LOD);
  setDefaultLOD(0);
@@ -795,29 +794,32 @@ void OpenGLOptions::setDefaults()
 void OpenGLOptions::load()
 {
  setRenderingSpeed(boConfig->intValue("RenderingSpeed", Defaults));
- setUpdateInterval(boConfig->updateInterval());
- mModelTexturesMipmaps->setChecked(boConfig->modelTexturesMipmaps());
- setMagnificationFilter(boConfig->magnificationFilter());
- setMinificationFilter(boConfig->minificationFilter());
- setMipmapMinificationFilter(boConfig->mipmapMinificationFilter());
- setAlignSelectionBoxes(boConfig->alignSelectionBoxes());
- mUseLight->setChecked(boConfig->useLight());
- mUseMaterials->setChecked(boConfig->useMaterials());
- setUseLOD(boConfig->useLOD());
+ setUpdateInterval(boConfig->uintValue("GLUpdateInterval"));
+ setTextureFilter(boConfig->intValue("TextureFilter"));
+ setTextureAnisotropy(boConfig->intValue("TextureAnisotropy"));
+ mUseCompressedTextures->setChecked(boConfig->boolValue("TextureCompression"));
+ mUseColoredMipmaps->setChecked(boConfig->boolValue("TextureColorMipmaps"));
+ setAlignSelectionBoxes(boConfig->boolValue("AlignSelectionBoxes"));
+ mUseLight->setChecked(boConfig->boolValue("UseLight"));
+ mUseMaterials->setChecked(boConfig->boolValue("UseMaterials"));
+ setUseLOD(boConfig->boolValue("UseLOD"));
  setDefaultLOD(boConfig->uintValue("DefaultLOD", 0));
  mEnableATIDepthWorkaround->setChecked(boConfig->boolValue("EnableATIDepthWorkaround"));
  mATIDepthWorkaroundValue->setText(QString::number(boConfig->doubleValue("ATIDepthWorkaroundValue")));
+#if BOSONFONT
  if (!mFontInfo->fromString(boConfig->stringValue("GLFont", QString::null))) {
 	boError() << k_funcinfo << "Could not load font " << boConfig->stringValue("GLFont", QString::null) << endl;
 	*mFontInfo = BoFontInfo();
  }
  mFont->setText(mFontInfo->guiName());
- mSmoothShading->setChecked(boConfig->boolValue("SmoothShading", true));
  mFontChanged = false;
+#endif
+ mSmoothShading->setChecked(boConfig->boolValue("SmoothShading", true));
  mResolution->setCurrentItem(0);
 
  setCurrentMeshRenderer(BoMeshRendererManager::manager()->currentRendererName());
  setCurrentGroundRenderer(BoGroundRendererManager::manager()->currentRendererName());
+ mUseGroundShaders->setChecked(boConfig->boolValue("UseGroundShaders"));
 }
 
 void OpenGLOptions::setUpdateInterval(int ms)
@@ -826,98 +828,80 @@ void OpenGLOptions::setUpdateInterval(int ms)
  mUpdateInterval->setValue(ms);
 }
 
-void OpenGLOptions::setMagnificationFilter(int f)
-{
- switch(f) {
-	case GL_NEAREST:
-	default:
-		mMagnificationFilter->setCurrentItem(0);
-		break;
-	case GL_LINEAR:
-		mMagnificationFilter->setCurrentItem(1);
-		break;
- }
-}
-
-int OpenGLOptions::magnificationFilter() const
-{
- if (mMagnificationFilter->currentItem() == 1) {
-	return GL_LINEAR;
- }
- // index is either 0 or -1
- return GL_NEAREST;
-}
-
-int OpenGLOptions::minificationFilter() const
-{
- if (mMinificationFilter->currentItem() == 1) {
-	return GL_LINEAR;
- }
- // index is either 0 or -1
- return GL_NEAREST;
-}
-
-void OpenGLOptions::setMinificationFilter(int f)
-{
- switch(f) {
-	case GL_NEAREST:
-	default:
-		mMinificationFilter->setCurrentItem(0);
-		break;
-	case GL_LINEAR:
-		mMinificationFilter->setCurrentItem(1);
-		break;
- }
-}
-
-int OpenGLOptions::mipmapMinificationFilter() const
+int OpenGLOptions::textureFilter() const
 {
  GLenum e;
- switch (mMipmapMinificationFilter->currentItem()) {
+ switch (mTextureFilter->currentItem()) {
 	case 0:
-		e = GL_NEAREST;
+		e = GL_LINEAR;
 		break;
 	case 1:
-		e = GL_LINEAR;
+		e = GL_LINEAR_MIPMAP_NEAREST;
 		break;
 	default:
 	case 2:
-		e = GL_NEAREST_MIPMAP_NEAREST;
-		break;
-	case 3:
-		e = GL_NEAREST_MIPMAP_LINEAR;
-		break;
-	case 4:
-		e = GL_LINEAR_MIPMAP_NEAREST;
-		break;
-	case 5:
 		e = GL_LINEAR_MIPMAP_LINEAR;
 		break;
  }
  return (int)e;
 }
 
-void OpenGLOptions::setMipmapMinificationFilter(int f)
+void OpenGLOptions::setTextureFilter(int f)
 {
  switch(f) {
-	case GL_NEAREST:
-		mMipmapMinificationFilter->setCurrentItem(0);
-		break;
 	case GL_LINEAR:
-		mMipmapMinificationFilter->setCurrentItem(1);
-		break;
-	default:
-	case GL_NEAREST_MIPMAP_NEAREST:
-		mMipmapMinificationFilter->setCurrentItem(2);
-		break;
-	case GL_NEAREST_MIPMAP_LINEAR:
-		mMipmapMinificationFilter->setCurrentItem(3);
+		mTextureFilter->setCurrentItem(0);
 		break;
 	case GL_LINEAR_MIPMAP_NEAREST:
-		mMipmapMinificationFilter->setCurrentItem(4);
+		mTextureFilter->setCurrentItem(1);
 		break;
+	default:
 	case GL_LINEAR_MIPMAP_LINEAR:
-		mMipmapMinificationFilter->setCurrentItem(5);
+		mTextureFilter->setCurrentItem(2);
+		break;
+ }
+}
+
+int OpenGLOptions::textureAnisotropy() const
+{
+ switch (mTextureFilter->currentItem()) {
+	case 3:
+		return 2;
+		break;
+	case 4:
+		return 4;
+		break;
+	case 5:
+		return 8;
+		break;
+	case 6:
+		return 16;
+		break;
+	default:
+		return 1;
+		break;
+ }
+}
+
+void OpenGLOptions::setTextureAnisotropy(int a)
+{
+ switch(a) {
+	case 2:
+		mTextureFilter->setCurrentItem(3);
+		break;
+	case 4:
+		mTextureFilter->setCurrentItem(4);
+		break;
+	case 8:
+		mTextureFilter->setCurrentItem(5);
+		break;
+	case 16:
+		mTextureFilter->setCurrentItem(6);
+		break;
+	default:
+		if (mTextureFilter->currentItem() >= 3) {
+			mTextureFilter->setCurrentItem(2);
+		}
 		break;
  }
 }
@@ -1000,6 +984,71 @@ void OpenGLOptions::setCurrentGroundRenderer(const QString& renderer)
  }
 }
 
+//////////////////////////////////////////////////////////////////////
+// Water Options
+//////////////////////////////////////////////////////////////////////
+
+WaterOptions::WaterOptions(QWidget* parent) : QVBox(parent), OptionsWidget()
+{
+// QHBox* hbox = new QHBox(this);
+ mShaders = new QCheckBox(i18n("Enable shaders"), this);
+ mShaders->setEnabled(boWaterRenderer->supportsShaders());
+ mReflections = new QCheckBox(i18n("Enable reflections"), this);
+ mReflections->setEnabled(boWaterRenderer->supportsReflections());
+ mTranslucency = new QCheckBox(i18n("Enable translucent water"), this);
+ mTranslucency->setEnabled(boWaterRenderer->supportsTranslucency());
+ mBumpmapping = new QCheckBox(i18n("Enable bumpmapped water"), this);
+ mBumpmapping->setEnabled(boWaterRenderer->supportsBumpmapping());
+ mAnimatedBumpmaps = new QCheckBox(i18n("Enable animated bumpmaps"), this);
+ mAnimatedBumpmaps->setEnabled(boWaterRenderer->supportsBumpmapping());
+
+ connect(mShaders, SIGNAL(toggled(bool)), this, SLOT(slotEnableShaders(bool)));
+}
+
+WaterOptions::~WaterOptions()
+{
+ boDebug(210) << k_funcinfo << endl;
+}
+
+void WaterOptions::apply()
+{
+ boDebug(210) << k_funcinfo << endl;
+ boConfig->setBoolValue("WaterShaders", mShaders->isChecked());
+ boConfig->setBoolValue("WaterReflections", mReflections->isChecked());
+ boConfig->setBoolValue("WaterTranslucency", mTranslucency->isChecked());
+ boConfig->setBoolValue("WaterBumpmapping", mBumpmapping->isChecked());
+ boConfig->setBoolValue("WaterAnimatedBumpmaps", mAnimatedBumpmaps->isChecked());
+ boWaterRenderer->reloadConfiguration();
+ boDebug(210) << k_funcinfo << "done" << endl;
+}
+
+void WaterOptions::setDefaults()
+{
+ mShaders->setChecked(DEFAULT_WATER_SHADERS);
+ mReflections->setChecked(DEFAULT_WATER_REFLECTIONS);
+ mTranslucency->setChecked(DEFAULT_WATER_TRANSLUCENCY);
+ mBumpmapping->setChecked(DEFAULT_WATER_BUMPMAPPING);
+ mAnimatedBumpmaps->setChecked(DEFAULT_WATER_ANIMATED_BUMPMAPS);
+ slotEnableShaders(mShaders->isChecked());
+}
+
+void WaterOptions::load()
+{
+ mShaders->setChecked(boConfig->boolValue("WaterShaders"));
+ mReflections->setChecked(boConfig->boolValue("WaterReflections"));
+ mTranslucency->setChecked(boConfig->boolValue("WaterTranslucency"));
+ mBumpmapping->setChecked(boConfig->boolValue("WaterBumpmapping"));
+ mAnimatedBumpmaps->setChecked(boConfig->boolValue("WaterAnimatedBumpmaps"));
+ slotEnableShaders(mShaders->isChecked());
+}
+
+void WaterOptions::slotEnableShaders(bool)
+{
+ mReflections->setEnabled(!mShaders->isChecked());
+ mTranslucency->setEnabled(!mShaders->isChecked());
+ mBumpmapping->setEnabled(!mShaders->isChecked());
+}
+
 //////////////////////////////////////////////////////////////////////
 // Chat Options
 //////////////////////////////////////////////////////////////////////
@@ -1025,8 +1074,8 @@ ChatOptions::~ChatOptions()
 void ChatOptions::apply()
 {
  boDebug(210) << k_funcinfo << endl;
- boConfig->setChatScreenRemoveTime(mScreenRemoveTime->value());
- boConfig->setChatScreenMaxItems(mScreenMaxItems->value());
+ boConfig->setUIntValue("ChatScreenRemoveTime", mScreenRemoveTime->value());
+ boConfig->setIntValue("ChatScreenMaxItems", mScreenMaxItems->value());
  boDebug(210) << k_funcinfo << "done" << endl;
 }
 
@@ -1038,8 +1087,8 @@ void ChatOptions::setDefaults()
 
 void ChatOptions::load()
 {
- setScreenRemoveTime(boConfig->chatScreenRemoveTime());
- setScreenMaxItems(boConfig->chatScreenMaxItems());
+ setScreenRemoveTime(boConfig->uintValue("ChatScreenRemoveTime"));
+ setScreenMaxItems(boConfig->intValue("ChatScreenMaxItems"));
 }
 
 void ChatOptions::setScreenRemoveTime(unsigned int s)
@@ -1088,11 +1137,11 @@ void ToolTipOptions::apply()
  QValueList<int> tips = factory.availableTipCreators();
  int index = mToolTipCreator->currentItem();
  if (index >= 0 && index < (int)tips.count()) {
-	boConfig->setToolTipCreator(tips[index]);
+	boConfig->setIntValue("ToolTipCreator", tips[index]);
  } else {
 	boWarning() << k_funcinfo << "invalid tooltip creator index=" << index << endl;
  }
- boConfig->setToolTipUpdatePeriod(mUpdatePeriod->value());
+ boConfig->setIntValue("ToolTipUpdatePeriod", mUpdatePeriod->value());
 }
 
 void ToolTipOptions::setDefaults()
@@ -1111,12 +1160,12 @@ void ToolTipOptions::setDefaults()
 
 void ToolTipOptions::load()
 {
- mUpdatePeriod->setValue(boConfig->toolTipUpdatePeriod());
+ mUpdatePeriod->setValue(boConfig->intValue("ToolTipUpdatePeriod"));
  BoToolTipCreatorFactory factory;
  QValueList<int> tips = factory.availableTipCreators();
  int index = -1;
  for (unsigned int i = 0; i < tips.count(); i++) {
-	if (tips[i] == boConfig->toolTipCreator()) {
+	if (tips[i] == boConfig->intValue("ToolTipCreator")) {
 		index = i;
 	}
  }
