@@ -1,6 +1,7 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 1999-2000,2001 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 1999-2000 Thomas Capricelli (capricel@email.enst.fr)
+    Copyright (C) 2001-2005 Andreas Beckermann (b_mann@gmx.de)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,11 +20,13 @@
 
 #include "player.h"
 
+#include "../bomemory/bodummymemory.h"
 #include "playerio.h"
 #include "speciestheme.h"
 #include "unit.h"
 #include "unitproperties.h"
-#include "bosonmessage.h"
+#include "unitplugins.h"
+#include "bosonmessageids.h"
 #include "bosonmap.h"
 #include "bosonstatistics.h"
 #include "boson.h"
@@ -33,6 +36,7 @@
 #include "bosonprofiling.h"
 #include "bodebug.h"
 #include "bobincoder.h"
+#include "boevent.h"
 
 #include <kgame/kgame.h>
 #include <kgame/kgamemessage.h>
@@ -43,7 +47,8 @@
 
 #include "player.moc"
 
-class Player::PlayerPrivate
+
+class PlayerPrivate
 {
 public:
 	PlayerPrivate()
@@ -62,6 +67,7 @@ public:
 	int mUnitPropID; // used for KGamePropertyHandler
 
 	QBitArray mFogged; // TODO: use KGameProperty
+	unsigned int mFoggedCount; // helper variable, doesn't need to be saved
 	KGameProperty<unsigned long int> mMinerals;
 	KGameProperty<unsigned long int> mOil;
 	KGamePropertyBool mIsNeutralPlayer;
@@ -72,6 +78,10 @@ public:
 	int mFacilitiesCount;
 
 	PlayerIO* mPlayerIO;
+
+	QValueList<unsigned long int> mResearchedUpgrades;
+
+	BoUpgradesCollection mUpgradesCollection;
 };
 
 Player::Player(bool isNeutralPlayer) : KPlayer()
@@ -79,6 +89,7 @@ Player::Player(bool isNeutralPlayer) : KPlayer()
  boDebug() << k_funcinfo << endl;
  mSpecies = 0;
  d = new PlayerPrivate;
+ d->mFoggedCount = 0;
  setAsyncInput(true);
  connect(this, SIGNAL(signalNetworkData(int, const QByteArray&, Q_UINT32, KPlayer*)),
 		this, SLOT(slotNetworkData(int, const QByteArray&, Q_UINT32, KPlayer*)));
@@ -90,13 +101,22 @@ Player::Player(bool isNeutralPlayer) : KPlayer()
 	boError() << k_funcinfo << "can't find name property" << endl;
  }
 // TODO d->mFogged.registerData() or something like this
+ mOutOfGame.registerData(IdOutOfGame, dataHandler(),
+		KGamePropertyBase::PolicyLocal, "OutOfGame");
+ mHasLost.registerData(IdHasLost, dataHandler(),
+		KGamePropertyBase::PolicyLocal, "HasLost");
+ mHasWon.registerData(IdHasWon, dataHandler(),
+		KGamePropertyBase::PolicyLocal, "HasWon");
  d->mMinerals.registerData(IdMinerals, dataHandler(),
-		KGamePropertyBase::PolicyLocal, "MineralCost");
+		KGamePropertyBase::PolicyLocal, "Minerals");
  d->mOil.registerData(IdOil, dataHandler(),
-		KGamePropertyBase::PolicyLocal, "OilCost");
+		KGamePropertyBase::PolicyLocal, "Oil");
  d->mIsNeutralPlayer.registerData(IdIsNeutralPlayer, dataHandler(),
 		KGamePropertyBase::PolicyLocal, "IsNeutralPlayer");
  d->mIsNeutralPlayer = isNeutralPlayer;
+ mOutOfGame = false;
+ mHasLost = false;
+ mHasWon = false;
  d->mPlayerIO = new PlayerIO(this);
 
  quitGame(); // this will reset some variables
@@ -133,8 +153,12 @@ void Player::quitGame(bool destruct)
  boDebug() << k_funcinfo << endl;
  d->mMobilesCount = 0;
  d->mFacilitiesCount = 0;
+ d->mFoggedCount = 0;
  mOutOfGame = false;
+ mHasWon = false;
+ mHasLost = false;
  d->mMap = 0;
+ d->mResearchedUpgrades.clear();
 
  boDebug() << k_funcinfo << "clearing units" << endl;
  d->mUnits.clear();
@@ -179,7 +203,7 @@ void Player::slotNetworkData(int msgid, const QByteArray& buffer, Q_UINT32 sende
 			KGameMessage::extractPropertyCommand(stream2, propertyId, cmd);
 			switch (propertyId) {
 				case Unit::IdWaypoints:
-					// waypoints have PolicyClean, so they 
+					// waypoints have PolicyClean, so they
 					// send a message which is handled here.
 					break;
 				default:
@@ -198,7 +222,7 @@ void Player::slotNetworkData(int msgid, const QByteArray& buffer, Q_UINT32 sende
 
  // this wasn't a unit property but a normal message
  switch (msgid) {
-	case BosonMessage::IdChat:
+	case BosonMessageIds::IdChat:
 		break;
 	default:
 		boWarning() << k_funcinfo << "Unknown message " << msgid << endl;
@@ -223,7 +247,7 @@ void Player::addUnit(Unit* unit, int dataHandlerId)
 {
  d->mUnits.append(unit);
  if (dataHandlerId == -1) {
-	dataHandlerId = BosonMessage::UnitPropertyHandler + d->mUnitPropID;
+	dataHandlerId = BosonMessageIds::UnitPropertyHandler + d->mUnitPropID;
 	d->mUnitPropID++;// used for ID of KGamePropertyHandler
  }
  unit->dataHandler()->registerHandler(dataHandlerId, this,
@@ -253,8 +277,10 @@ void Player::unitDestroyed(Unit* unit)
  }
  if (unit->unitProperties()->supportMiniMap()) {
 	if (!hasMiniMap()) {
-		speciesTheme()->playSound(SoundReportMinimapDeactivated);
-		emit signalShowMiniMap(false);
+		BoEvent* event = new BoEvent("LostMinimap");
+		event->setPlayerId(id());
+		event->setLocation(BoVector3Fixed(unit->x(), unit->y(), unit->z()));
+		boGame->queueEvent(event);
 	}
  }
 }
@@ -268,10 +294,10 @@ void Player::slotUnitPropertyChanged(KGamePropertyBase* prop)
 
  bool emitSignalUnitChanged = false;
  switch (prop->id()) {
-	case UnitBase::IdHealth:
-	case UnitBase::IdArmor:
-	case UnitBase::IdShields:
-	case UnitBase::IdSightRange:
+	case UnitBase::IdHealthFactor:
+	case UnitBase::IdArmorFactor:
+	case UnitBase::IdShieldsFactor:
+	case UnitBase::IdSightRangeFactor:
 		// update BosonUnitView if the unit is selected.
 		// not all of these IDs are displayed there. But perhaps they
 		// will one day.
@@ -430,6 +456,11 @@ void Player::initMap(BosonMap* map, bool fogged)
 	return;
  }
  d->mFogged.fill(fogged, d->mMap->width() * d->mMap->height());
+ if (fogged) {
+	d->mFoggedCount = d->mFogged.size();
+ } else {
+	d->mFoggedCount = 0;
+ }
 }
 
 void Player::fog(int x, int y)
@@ -437,13 +468,22 @@ void Player::fog(int x, int y)
  if (!d->mMap) {
 	return;
  }
- if (x + d->mMap->width() * y >= d->mFogged.size()) {
+ unsigned int index = x + d->mMap->width() * y;
+ if (index >= d->mFogged.size()) {
 	boError() << k_funcinfo << "x=" << x << ",y=" << y << " out of range ("
 			<< d->mFogged.size() << ")" << endl;
 	return;
  }
+ bool isFogged = d->mFogged.at(index);
 //boDebug() << k_funcinfo << x << "," << y << endl;
- d->mFogged.setBit(x + d->mMap->width() * y);
+ d->mFogged.setBit(index);
+ if (!isFogged) {
+	if (d->mFoggedCount >= d->mFogged.size()) {
+		boError() << k_funcinfo << "invalid fogged count value " << d->mFoggedCount << endl;
+	} else {
+		d->mFoggedCount++;
+	}
+ }
  // emit signal (actual fog on map + minimap)
  // TODO: any way to emit only for the local player?
  emit signalFog(x, y);
@@ -456,12 +496,21 @@ void Player::unfog(int x, int y)
 	return;
  }
 //boDebug() << k_funcinfo << x << "," << y << endl;
- if (x + d->mMap->width() * y >= d->mFogged.size()) {
+ unsigned int index = x + d->mMap->width() * y;
+ if (index >= d->mFogged.size()) {
 	boError() << k_funcinfo << "x=" << x << ",y=" << y << " out of range ("
 			<< d->mFogged.size() << ")" << endl;
 	return;
  }
- d->mFogged.clearBit(x + d->mMap->width() * y);
+ bool isFogged = d->mFogged.at(index);
+ d->mFogged.clearBit(index);
+ if (isFogged) {
+	if (d->mFoggedCount == 0) {
+		boError() << k_funcinfo << "invalid fogged count value " << d->mFoggedCount << endl;
+	} else {
+		d->mFoggedCount--;
+	}
+ }
  // emit signal (actual fog on map + minimap)
  // TODO: any way to emit only for the local player?
  emit signalUnfog(x, y);
@@ -532,24 +581,31 @@ void Player::facilityCompleted(Facility* fac)
 	boError() << k_funcinfo << "NULL facility" << endl;
 	return;
  }
+
+ BoVector3Fixed location(fac->x(), fac->y(), fac->z());
+ BoEvent* constructedEvent = new BoEvent("FacilityWithTypeConstructed", QString::number(fac->type()));
+ constructedEvent->setPlayerId(id());
+ constructedEvent->setUnitId(fac->id());
+ constructedEvent->setLocation(location);
+ boGame->queueEvent(constructedEvent);
+
  if (fac->unitProperties()->supportMiniMap()) {
-	speciesTheme()->playSound(SoundReportMinimapActivated);
-	emit signalShowMiniMap(true);
+	BoEvent* miniMapEvent = new BoEvent("GainedMinimap");
+	miniMapEvent->setPlayerId(id());
+	miniMapEvent->setLocation(location);
+	boGame->queueEvent(miniMapEvent);
  }
 }
 
-bool Player::checkOutOfGame()
+void Player::setOutOfGame()
 {
- // TODO: make more clever.
- // e.g. currently every small unit, like harvesters, needs to be destroyed...
  if ((Player*)boGame->playerList()->getLast() == this) {
 	// the last player is per definition the neutral player (which is never
 	// out of the game).
 	mOutOfGame = false;
-	return isOutOfGame();
+	return;
  }
- mOutOfGame = (d->mUnits.count() == 0);
- return isOutOfGame();
+ mOutOfGame = true;
 }
 
 BosonStatistics* Player::statistics() const
@@ -660,39 +716,41 @@ bool Player::hasUnitWithType(unsigned long int type) const
 
 bool Player::hasTechnology(unsigned long int id) const
 {
- UpgradeProperties* tech = speciesTheme()->technologyList().find(id);
- if (!tech) {
-	return false;
+ if (d->mResearchedUpgrades.contains(id)) {
+	return true;
  }
- if (!tech->isResearched()) {
-	return false;
- }
- return true;
+ return false;
 }
 
-void Player::technologyResearched(ProductionPlugin*, unsigned long int id)
+const UpgradeProperties* Player::technologyProperties(unsigned long int type) const
 {
- boDebug() << k_funcinfo << "id: " << id << endl;
- // Check if it isn't researched already
- QIntDictIterator<UpgradeProperties> it(speciesTheme()->technologyList());
- while (it.current()) {
-	if (((unsigned long int)(it.currentKey()) == id) && (it.current()->isResearched())) {
-		boError() << k_funcinfo << "upgrade " << it.current() << " already researched!" << endl;
-		return;
-	}
-	++it;
- }
+ return speciesTheme()->technology(type);
+}
 
- UpgradeProperties* prop = speciesTheme()->technology(id);
- if (!prop) {
-	boError() << k_funcinfo << "NULL technology " << id << endl;
+void Player::technologyResearched(ProductionPlugin* plugin, unsigned long int type)
+{
+ BO_CHECK_NULL_RET(plugin);
+ BO_CHECK_NULL_RET(plugin->unit());
+ boDebug() << k_funcinfo << "type: " << type << endl;
+ // Check if it isn't researched already
+ if (d->mResearchedUpgrades.contains(type)) {
+	boError() << k_funcinfo << "upgrade " << type << " already researched!" << endl;
 	return;
  }
- prop->setResearched(true);
- prop->apply(this);
 
- ((Boson*)game())->slotUpdateProductionOptions();
- // TODO: also update unit view
+ const UpgradeProperties* prop = speciesTheme()->technology(type);
+ if (!prop) {
+	boError() << k_funcinfo << "NULL technology " << type << endl;
+	return;
+ }
+ d->mResearchedUpgrades.append(type);
+
+ addUpgrade(prop);
+
+ BoEvent* event = new BoEvent("TechnologyWithTypeResearched", QString::number(type), QString::number(plugin->unit()->id()));
+ event->setPlayerId(id());
+ event->setLocation(BoVector3Fixed(plugin->unit()->x(), plugin->unit()->y(), plugin->unit()->z()));
+ ((Boson*)game())->queueEvent(event);
 }
 
 bool Player::advanceFlag() const
@@ -702,14 +760,18 @@ bool Player::advanceFlag() const
 
 bool Player::saveAsXML(QDomElement& root)
 {
- BosonProfiler profiler(BosonProfiling::SavePlayerToXML);
+ PROFILE_METHOD
  if (!game() || !game()->playerList()) {
 	boError() << k_funcinfo << "NULL game/playerlist" << endl;
 	return false;
  }
+ if (!speciesTheme()) {
+	BO_NULL_ERROR(speciesTheme());
+	return false;
+ }
 
  // note: we need to save the index in the list, not the actual id()
- root.setAttribute(QString::fromLatin1("Id"), game()->playerList()->findRef(this));
+ root.setAttribute(QString::fromLatin1("PlayerId"), game()->playerList()->findRef(this));
 
  if (game()->playerList()->findRef(this) == (int)game()->playerList()->count() - 1) {
 	root.setAttribute(QString::fromLatin1("IsNeutral"), 1);
@@ -728,30 +790,17 @@ bool Player::saveAsXML(QDomElement& root)
  }
  root.appendChild(handler);
 
- // Save speciestheme
- if (speciesTheme()) {
-	root.setAttribute(QString::fromLatin1("SpeciesTheme"), speciesTheme()->identifier());
-	root.setAttribute(QString::fromLatin1("TeamColor"), (unsigned int)speciesTheme()->teamColor().rgb());
- } else {
-	// the attributes won't be there
-	boWarning() << k_funcinfo << "NULL speciestheme while saving into XML" << endl;
- }
-
  QDomElement upgrades = doc.createElement("Upgrades");
- if (speciesTheme()) {
-	QIntDict<UpgradeProperties> list = speciesTheme()->technologyList();
-	QIntDictIterator<UpgradeProperties> it(speciesTheme()->technologyList());
-	for (; it.current(); ++it) {
-		if (!it.current()->isResearched()) {
-			continue;
-		}
-		QDomElement e = doc.createElement("Researched");
-		e.setAttribute("Id", QString::number(it.current()->id()));
-		upgrades.appendChild(e);
-	}
- }
+ d->mUpgradesCollection.saveAsXML(upgrades);
  root.appendChild(upgrades);
 
+ QDomElement speciesThemeTag = doc.createElement("SpeciesTheme");
+ if (!speciesTheme()->saveGameDataAsXML(speciesThemeTag)) {
+	boError() << k_funcinfo << "Unable to save SpeciesTheme" << endl;
+	return false;
+ }
+ root.appendChild(speciesThemeTag);
+
  // Save unitpropID
  root.setAttribute(QString::fromLatin1("UnitPropId"), (unsigned int)d->mUnitPropID);
 
@@ -795,29 +844,25 @@ bool Player::loadFromXML(const QDomElement& root)
 	return false;
  }
 
+ clearUpgrades();
  QDomElement upgrades = root.namedItem("Upgrades").toElement();
- if (!upgrades.isNull() && speciesTheme()) {
-	QDomNodeList researched = upgrades.elementsByTagName("Researched");
-	for (unsigned int i = 0; i < researched.count(); i++) {
-		QDomElement e = researched.item(i).toElement();
-		bool ok = false;
-		unsigned int id = e.attribute("Id").toUInt(&ok);
-		if (!ok) {
-			boError() << k_funcinfo << "Id attribute of Researched tag is not a valid number" << endl;
-			continue;
-		}
-		UpgradeProperties* u = speciesTheme()->technology(id);
-		if (!u) {
-			boWarning() << k_funcinfo << "technology " << id << " not found" << endl;
-			continue;
-		}
-		if (u->isResearched()) {
-			boWarning() << k_funcinfo << "technology " << id << " already researched" << endl;
-			continue;
-		}
-		u->setResearched(true);
-		u->apply(this);
-	}
+ if (upgrades.isNull()) {
+	boError(260) << k_funcinfo << "no Upgrades tag found" << endl;
+	return false;
+ }
+ if (!d->mUpgradesCollection.loadFromXML(speciesTheme(), upgrades)) {
+	boError(260) << k_funcinfo << "errror loading upgrades" << endl;
+	return false;
+ }
+
+ QDomElement speciesThemeTag = root.namedItem("SpeciesTheme").toElement();
+ if (speciesThemeTag.isNull()) {
+	boError(260) << k_funcinfo << "NULL SpeciesTheme tag" << endl;
+	return false;
+ }
+ if (!speciesTheme()->loadGameDataFromXML(speciesThemeTag)) {
+	boError(260) << k_funcinfo << "Unable to load SpeciesTheme" << endl;
+	return false;
  }
 
  if (root.hasAttribute(QString::fromLatin1("UnitPropId"))) {
@@ -868,16 +913,24 @@ bool Player::loadFogOfWar(const QDomElement& root)
  if (element.isNull()) {
 	boError() << k_funcinfo << "No Fogged tag found" << endl;
 	d->mFogged.fill(true);
+	d->mFoggedCount = d->mFogged.size();
 	return false;
  }
  QString text = element.text();
  if (text.isEmpty()) {
 	boError() << k_funcinfo << "no content for Fogged tag found" << endl;
 	d->mFogged.fill(true);
+	d->mFoggedCount = d->mFogged.size();
 	return false;
  }
  boDebug() << k_funcinfo << "decoding" << endl;
  d->mFogged = BoBinCoder::toBinary(text);
+ d->mFoggedCount = 0;
+ for (unsigned int i = 0; i < d->mFogged.size(); i++) {
+	if (d->mFogged.at(i)) {
+		d->mFoggedCount++;
+	}
+ }
  boDebug() << k_funcinfo << "decoded: " << d->mFogged.size() << endl;
  if (d->mMap) {
 	if (d->mMap->width() * d->mMap->height() != d->mFogged.size()) {
@@ -905,3 +958,57 @@ void Player::writeGameLog(QTextStream& log)
  log << endl;
 }
 
+unsigned int Player::foggedCells() const
+{
+ return d->mFoggedCount;
+}
+
+void Player::clearUpgrades()
+{
+ d->mUpgradesCollection.clearUpgrades();
+}
+
+void Player::addUpgrade(const UpgradeProperties* upgrade)
+{
+ BO_CHECK_NULL_RET(upgrade);
+ d->mUpgradesCollection.addUpgrade(upgrade);
+
+ for (QPtrListIterator<Unit> it(d->mUnits); it.current(); ++it) {
+	if (upgrade->appliesTo(it.current())) {
+		it.current()->addUpgrade(upgrade);
+	}
+ }
+
+ const QIntDict<UnitProperties>* units = speciesTheme()->allUnitsNonConst();
+ for (QIntDictIterator<UnitProperties> it(*units); it.current(); ++it) {
+	if (upgrade->appliesTo(it.current())) {
+		it.current()->addUpgrade(upgrade);
+	}
+ }
+}
+
+void Player::removeUpgrade(unsigned long int id)
+{
+ removeUpgrade(d->mUpgradesCollection.findUpgrade(id));
+}
+
+void Player::removeUpgrade(const UpgradeProperties* upgrade)
+{
+ d->mUpgradesCollection.removeUpgrade(upgrade);
+
+ for (QPtrListIterator<Unit> it(d->mUnits); it.current(); ++it) {
+	it.current()->removeUpgrade(upgrade);
+ }
+
+ const QIntDict<UnitProperties>* units = speciesTheme()->allUnitsNonConst();
+ for (QIntDictIterator<UnitProperties> it(*units); it.current(); ++it) {
+	it.current()->removeUpgrade(upgrade);
+ }
+}
+
+const QValueList<const UpgradeProperties*>* Player::upgrades() const
+{
+ return d->mUpgradesCollection.upgrades();
+}
+
+
