@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2002 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2002 Andreas Beckermann (b_mann@gmx.de)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 
 #include "pluginproperties.h"
 
+#include "../bomemory/bodummymemory.h"
 #include "unitproperties.h"
 #include "speciestheme.h"
 #include "bodebug.h"
@@ -28,6 +29,7 @@
 #include <ksimpleconfig.h>
 
 PluginProperties::PluginProperties(const UnitProperties* parent)
+	: BoBaseValueCollection()
 {
  mUnitProperties = parent;
 }
@@ -41,6 +43,11 @@ SpeciesTheme* PluginProperties::speciesTheme() const
  return unitProperties()->theme();
 }
 
+const BoUpgradesCollection& PluginProperties::upgradesCollection() const
+{
+ return unitProperties()->upgradesCollection();
+}
+
 
 RepairProperties::RepairProperties(const UnitProperties* parent)
 		: PluginProperties(parent)
@@ -149,8 +156,9 @@ void HarvesterProperties::loadPlugin(KSimpleConfig* config)
 	mCanMineOil = false;
  }
  mMaxResources = config->readUnsignedNumEntry("MaxResources", 100);
- mMiningSpeed = config->readUnsignedNumEntry("MiningSpeed", 10);
- mUnloadingSpeed = config->readUnsignedNumEntry("UnloadingSpeed", 10);
+ // Convert speeds from amount/second to amount/adv.call
+ mMiningSpeed = (int)(config->readDoubleNumEntry("MiningSpeed", 100) / 20.0f);
+ mUnloadingSpeed = (int)(config->readDoubleNumEntry("UnloadingSpeed", 200) / 20.0f);
 }
 
 void HarvesterProperties::savePlugin(KSimpleConfig* config)
@@ -159,8 +167,9 @@ void HarvesterProperties::savePlugin(KSimpleConfig* config)
  config->writeEntry("CanMineMinerals", mCanMineMinerals);
  config->writeEntry("CanMineOil", mCanMineOil);
  config->writeEntry("MaxResources", mMaxResources);
- config->writeEntry("MiningSpeed", mMiningSpeed);
- config->writeEntry("UnloadingSpeed", mUnloadingSpeed);
+ // Convert speeds from amount/adv.call to amount/second
+ config->writeEntry("MiningSpeed", mMiningSpeed * 20.0f);
+ config->writeEntry("UnloadingSpeed", mUnloadingSpeed * 20.0f);
 }
 
 
