@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2003-2004 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2003-2005 Rivo Laks (rivolaks@hot.ee)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,13 +19,12 @@
 
 #include "bosonscript.h"
 
+#include <bomemory/bodummymemory.h>
 #include "../bo3dtools.h"
-#include "../bocamera.h"
-#include "../boautocamera.h"
-#include "../bosonbigdisplaybase.h"
 #include "../player.h"
 #include "../boson.h"
 #include "../bosonmessage.h"
+#include "../bosonmessageids.h"
 #include "../bosoncanvas.h"
 #include "../bosoncollisions.h"
 #include "../rtti.h"
@@ -36,33 +35,36 @@
 #include "../unitplugins.h"
 #include "../bosonprofiling.h"
 #include "../bosonpath.h"
-#include "../bolight.h"
 #include "../speciestheme.h"
-#include "../bosoneffectproperties.h"
 #include "../playerio.h"
 #include "../pluginproperties.h"
 #include "../unitbase.h"
+#include "../bosonplayfield.h"
+#include "../bosonmap.h"
+#include "bosonscriptinterface.h"
 #include "bodebug.h"
 
 #include "pythonscript.h"
 
 #include <kgame/kgamemessage.h>
+#include <kglobal.h>
+#include <kstandarddirs.h>
+
 
 #include <qdatastream.h>
-#include <qpoint.h>
 
 
-BosonBigDisplayBase* BosonScript::mDisplay = 0;
+BosonScript* BosonScript::mCurrentScript = 0;
 BosonCanvas* BosonScript::mCanvas = 0;
 Boson* BosonScript::mGame = 0;
 
-BosonScript* BosonScript::newScriptParser(Language lang, Player* p)
+BosonScript* BosonScript::newScriptParser(Language lang, int playerId)
 {
   boDebug() << k_funcinfo << endl;
   BosonScript* s = 0;
   if(lang == Python)
   {
-    s = new PythonScript(p);
+    s = new PythonScript(playerId);
   }
   else
   {
@@ -72,15 +74,17 @@ BosonScript* BosonScript::newScriptParser(Language lang, Player* p)
   return s;
 }
 
-BosonScript::BosonScript(Player* p)
+BosonScript::BosonScript(int playerId)
 {
   boDebug() << k_funcinfo << endl;
-  mPlayer = p;
+  mInterface = new BosonScriptInterface(0);
+  mPlayerId = playerId;
 }
 
 BosonScript::~BosonScript()
 {
   boDebug() << k_funcinfo << endl;
+  delete mInterface;
 }
 
 void BosonScript::sendInput(int playerId, QDataStream& stream)
@@ -104,52 +108,43 @@ void BosonScript::sendInput(int playerId, QDataStream& stream)
   p->forwardInput(stream);
 }
 
-BoGameCamera* BosonScript::camera()
+void BosonScript::makeScriptCurrent(BosonScript* s)
 {
-  if(!display())
-  {
-    boError() << k_funcinfo << "NULL display" << endl;
-    return 0;
-  }
-
-  return display()->camera();
-}
-
-BoAutoGameCamera* BosonScript::autoCamera()
-{
-  if(!display())
-  {
-    boError() << k_funcinfo << "NULL display" << endl;
-    return 0;
-  }
-
-  return display()->autoCamera();
+  mCurrentScript = s;
 }
 
 int BosonScript::playerId() const
 {
-  if(!player())
-  {
-    boError() << k_funcinfo << "NULL player" << endl;
-    return -1;
-  }
-
-  return player()->id();
+  return mPlayerId;
 }
 
-BoLight* BosonScript::light(int id)
+QString BosonScript::scriptsPath()
 {
-  if(!display())
+  QString path = KGlobal::dirs()->findResourceDir("data", "boson/themes/scripts/ai.py");
+  if(path.isNull())
   {
-    boError() << k_funcinfo << "NULL display" << endl;
-    return 0;
+    boWarning() << k_funcinfo << "No ai.py script file found!" << endl;
+    return QString::null;
   }
-
-  return display()->light(id);
+  path += "boson/themes/scripts/";
+  return path;
+  // TODO: maybe cache the path?
 }
 
+
+/*****  Event methods  *****/
+int BosonScript::addEventHandler(const QString& eventname, const QString& functionname, const QString& args)
+{
+  return interface()->addEventHandler(eventname, functionname, args);
+}
+
+void BosonScript::removeEventHandler(int id)
+{
+  interface()->removeEventHandler(id);
+}
+
+
 /*****  Player methods  *****/
-
 bool BosonScript::areEnemies(int playerId1, int playerId2)
 {
   if(!game())
@@ -196,6 +191,25 @@ QValueList<int> BosonScript::allPlayers()
   return players;
 }
 
+bool BosonScript::isNeutral(int playerId)
+{
+  if(!game())
+  {
+    boError() << k_funcinfo << "NULL game" << endl;
+    return false;  // What to return here?
+  }
+
+  Player* p = (Player*)(game()->findPlayer(playerId));
+
+  if(!p)
+  {
+    boError() << k_funcinfo << "No player with id " << playerId << endl;
+    return false;
+  }
+
+  return p->isNeutralPlayer();
+}
+
 /*****  Resource methods  *****/
 unsigned long int BosonScript::minerals(int playerId)
 {
@@ -228,7 +242,7 @@ void BosonScript::addMinerals(int playerId, int amount)
   QDataStream stream(b, IO_WriteOnly);
   stream << (Q_UINT32)playerId;
   stream << (Q_INT32)amount;
-  game()->sendMessage(b, BosonMessage::IdModifyMinerals);
+  game()->sendMessage(b, BosonMessageIds::IdModifyMinerals);
 }
 
 unsigned long int BosonScript::oil(int playerId)
@@ -262,15 +276,15 @@ void BosonScript::addOil(int playerId, int amount)
   QDataStream stream(b, IO_WriteOnly);
   stream << (Q_UINT32)playerId;
   stream << (Q_INT32)amount;
-  game()->sendMessage(b, BosonMessage::IdModifyOil);
+  game()->sendMessage(b, BosonMessageIds::IdModifyOil);
 }
 
-QValueList<QPoint> BosonScript::nearestMineralLocations(int playerId, int x, int y, unsigned int n, unsigned int radius)
+QValueList<BoVector2Fixed> BosonScript::nearestMineralLocations(int playerId, int x, int y, unsigned int n, unsigned int radius)
 {
   if(!game())
   {
     boError() << k_funcinfo << "NULL game" << endl;
-    return QValueList<QPoint>();
+    return QValueList<BoVector2Fixed>();
   }
 
   Player* p = (Player*)(game()->findPlayer(playerId));
@@ -278,18 +292,18 @@ QValueList<QPoint> BosonScript::nearestMineralLocations(int playerId, int x, int
   if(!p)
   {
     boError() << k_funcinfo << "No player with id " << playerId << endl;
-    return QValueList<QPoint>();
+    return QValueList<BoVector2Fixed>();
   }
 
-  return BosonPath::findLocations(p, x, y, n, radius, BosonPath::Minerals);
+  return canvas()->pathfinder()->findLocations(p, x, y, n, radius, BosonPath::Minerals);
 }
 
-QValueList<QPoint> BosonScript::nearestOilLocations(int playerId, int x, int y, unsigned int n, unsigned int radius)
+QValueList<BoVector2Fixed> BosonScript::nearestOilLocations(int playerId, int x, int y, unsigned int n, unsigned int radius)
 {
   if(!game())
   {
     boError() << k_funcinfo << "NULL game" << endl;
-    return QValueList<QPoint>();
+    return QValueList<BoVector2Fixed>();
   }
 
   Player* p = (Player*)(game()->findPlayer(playerId));
@@ -297,49 +311,45 @@ QValueList<QPoint> BosonScript::nearestOilLocations(int playerId, int x, int y,
   if(!p)
   {
     boError() << k_funcinfo << "No player with id " << playerId << endl;
-    return QValueList<QPoint>();
+    return QValueList<BoVector2Fixed>();
   }
 
-  return BosonPath::findLocations(p, x, y, n, radius, BosonPath::Oil);
+  return canvas()->pathfinder()->findLocations(p, x, y, n, radius, BosonPath::Oil);
 }
 
 
 /*****  Unit methods  *****/
-void BosonScript::moveUnit(int player, int id, int x, int y)
+void BosonScript::moveUnit(int player, int id, float x, float y)
 {
   QByteArray b;
   QDataStream stream(b, IO_WriteOnly);
 
-  // tell the clients we want to move units:
-  stream << (Q_UINT32)BosonMessage::MoveMove;
-  // We want to move without attacking
-  stream << (Q_UINT8)0;
-  // tell them where to move to:
-  stream << QPoint(x, y);
-  // tell them how many units:
-  stream << (Q_UINT32)1;
-  // Unit id
-  stream << (Q_ULONG)id;
+  QValueList<Q_ULONG> moveUnits;
+  moveUnits.append((Q_ULONG)id);
+  BosonMessageMoveMove message(false, BoVector2Fixed(x, y), moveUnits);
+  if(!message.save(stream))
+  {
+    boError() << k_funcinfo << "failed saving message " << message.messageId() << " to stream" << endl;
+    return;
+  }
 
   QDataStream msg(b, IO_ReadOnly);
   sendInput(player, msg);
 }
 
-void BosonScript::moveUnitWithAttacking(int player, int id, int x, int y)
+void BosonScript::moveUnitWithAttacking(int player, int id, float x, float y)
 {
   QByteArray b;
   QDataStream stream(b, IO_WriteOnly);
 
-  // tell the clients we want to move units:
-  stream << (Q_UINT32)BosonMessage::MoveMove;
-  // We want to move with attacking
-  stream << (Q_UINT8)1;
-  // tell them where to move to:
-  stream << QPoint(x, y);
-  // tell them how many units:
-  stream << (Q_UINT32)1;
-  // Unit id
-  stream << (Q_ULONG)id;
+  QValueList<Q_ULONG> moveUnits;
+  moveUnits.append((Q_ULONG)id);
+  BosonMessageMoveMove message(true, BoVector2Fixed(x, y), moveUnits);
+  if(!message.save(stream))
+  {
+    boError() << k_funcinfo << "failed saving message " << message.messageId() << " to stream" << endl;
+    return;
+  }
 
   QDataStream msg(b, IO_ReadOnly);
   sendInput(player, msg);
@@ -350,13 +360,14 @@ void BosonScript::attack(int player, int attackerId, int targetId)
   QByteArray b;
   QDataStream stream(b, IO_WriteOnly);
 
-  // tell the clients we want to attack:
-  stream << (Q_UINT32)BosonMessage::MoveAttack;
-  // tell them which unit to attack:
-  stream << (Q_ULONG)targetId;
-  // tell them how many units attack:
-  stream << (Q_UINT32)1;
-  stream << (Q_ULONG)attackerId;
+  QValueList<Q_ULONG> units;
+  units.append((Q_ULONG)attackerId);
+  BosonMessageMoveAttack message(targetId, units);
+  if(!message.save(stream))
+  {
+    boError() << k_funcinfo << "failed saving message " << message.messageId() << " to stream" << endl;
+    return;
+  }
 
   QDataStream msg(b, IO_ReadOnly);
   sendInput(player, msg);
@@ -367,23 +378,26 @@ void BosonScript::stopUnit(int player, int id)
   QByteArray b;
   QDataStream stream(b, IO_WriteOnly);
 
-  // tell the clients we want to move units:
-  stream << (Q_UINT32)BosonMessage::MoveStop;
-  // tell them how many units:
-  stream << (Q_UINT32)1;
-  stream << (Q_ULONG)id;
+  QValueList<Q_ULONG> units;
+  units.append((Q_ULONG)id);
+  BosonMessageMoveStop message(units);
+  if(!message.save(stream))
+  {
+    boError() << k_funcinfo << "failed saving message " << message.messageId() << " to stream" << endl;
+    return;
+  }
 
   QDataStream msg(b, IO_WriteOnly);
   sendInput(player, msg);
 }
 
-void BosonScript::mineUnit(int player, int id, int x, int y)
+void BosonScript::mineUnit(int player, int id, float x, float y)
 {
   // First we have to find resource mine
   // TODO: it sucks to do this here, perhaps we could have something like a
   //  MoveMineAt message which orders unit to move at specific point, not at
   //  some unit (then Boson would take care of finding resource mine unit).
-  Unit* resourceUnit = canvas()->findUnitAt(BoVector3(x, y, 0.0));
+  Unit* resourceUnit = canvas()->findUnitAt(BoVector3Fixed(x, y, 0.0));
   if(!resourceUnit)
   {
     boError() << k_funcinfo << "No units found at (" << x << "; " << y << ")" << endl;
@@ -400,9 +414,12 @@ void BosonScript::mineUnit(int player, int id, int x, int y)
   QByteArray b;
   QDataStream stream(b, IO_WriteOnly);
 
-  stream << (Q_UINT32)BosonMessage::MoveMine;
-  stream << (Q_ULONG)id;
-  stream << (Q_ULONG)resourceUnit->id();
+  BosonMessageMoveMine message(id, resourceUnit->id());
+  if(!message.save(stream))
+  {
+    boError() << k_funcinfo << "failed saving message " << message.messageId() << " to stream" << endl;
+    return;
+  }
 
   QDataStream msg(b, IO_ReadOnly);
   sendInput(player, msg);
@@ -413,28 +430,32 @@ void BosonScript::setUnitRotation(int player, int id, float rotation)
   QByteArray b;
   QDataStream stream(b, IO_WriteOnly);
 
-  stream << (Q_UINT32)BosonMessage::MoveRotate;
-  stream << (Q_UINT32)player;
-  stream << (Q_ULONG)id;
-  stream << rotation;
+  BosonMessageMoveRotate message(id, player, rotation);
+  if(!message.save(stream))
+  {
+    boError() << k_funcinfo << "failed saving message " << message.messageId() << " to stream" << endl;
+    return;
+  }
 
   QDataStream msg(b, IO_ReadOnly);
   sendInput(player, msg);
 }
 
-void BosonScript::dropBomb(int player, int id, int weapon, int x, int y)
+void BosonScript::dropBomb(int player, int id, int weapon, float x, float y)
 {
   QByteArray b;
   QDataStream stream(b, IO_WriteOnly);
 
-  stream << (Q_UINT32)BosonMessage::MoveDropBomb;
-  // tell place
-  stream << (Q_INT32)x;
-  stream << (Q_INT32)y;
-  // tell them how many units attack:
-  stream << (Q_UINT32)1;
-  stream << (Q_UINT32)id;
-  stream << (Q_UINT32)weapon;
+  QValueList<Q_ULONG> units;
+  QValueList<Q_ULONG> weapons;
+  units.append((Q_ULONG)id);
+  weapons.append((Q_ULONG)weapon);
+  BosonMessageMoveDropBomb message(BoVector2Fixed(x, y), units, weapons);
+  if(!message.save(stream))
+  {
+    boError() << k_funcinfo << "failed saving message " << message.messageId() << " to stream" << endl;
+    return;
+  }
 
   QDataStream msg(b, IO_ReadOnly);
   sendInput(player, msg);
@@ -445,17 +466,18 @@ void BosonScript::produceUnit(int player, int factory, int production)
   QByteArray b;
   QDataStream stream(b, IO_WriteOnly);
 
-  stream << (Q_UINT32)BosonMessage::MoveProduce;
-  stream << (Q_UINT32)ProduceUnit;
-  stream << (Q_UINT32)player;
-  stream << (Q_ULONG)factory;
-  stream << (Q_UINT32)production;
+  BosonMessageMoveProduce message((Q_UINT32)ProduceUnit, player, factory, production);
+  if(!message.save(stream))
+  {
+    boError() << k_funcinfo << "failed saving message " << message.messageId() << " to stream" << endl;
+    return;
+  }
 
   QDataStream msg(b, IO_ReadOnly);
   sendInput(player, msg);
 }
 
-void BosonScript::spawnUnit(int player, int type, int x, int y)
+void BosonScript::spawnUnit(int player, int type, float x, float y)
 {
   if(!game())
   {
@@ -465,17 +487,20 @@ void BosonScript::spawnUnit(int player, int type, int x, int y)
   QByteArray b;
   QDataStream stream(b, IO_WriteOnly);
 
-  stream << (Q_UINT32)BosonMessage::MovePlaceUnit;
+#warning AB: This is an editor message. it should not be here.
+  boWarning() << k_funcinfo << "FIXME" << endl;
+#if 0
+  stream << (Q_UINT32)BosonMessageIds::MovePlaceUnit;
   stream << (Q_UINT32)player;
   stream << (Q_UINT32)type;
-  stream << (Q_INT32)x;
-  stream << (Q_INT32)y;
+  stream << BoVector2Fixed(x, y);
+#endif
 
   QDataStream msg(b, IO_ReadOnly);
   sendInput(player, msg);
 }
 
-void BosonScript::teleportUnit(int player, int id, int x, int y)
+void BosonScript::teleportUnit(int player, int id, float x, float y)
 {
   if(!game())
   {
@@ -485,11 +510,67 @@ void BosonScript::teleportUnit(int player, int id, int x, int y)
   QByteArray b;
   QDataStream stream(b, IO_WriteOnly);
 
-  stream << (Q_UINT32)BosonMessage::MoveTeleport;
-  stream << (Q_UINT32)player;
-  stream << (Q_UINT32)id;
-  stream << (Q_INT32)x;
-  stream << (Q_INT32)y;
+  BosonMessageMoveTeleport message(id, player, BoVector2Fixed(x, y));
+  if(!message.save(stream))
+  {
+    boError() << k_funcinfo << "failed saving message " << message.messageId() << " to stream" << endl;
+    return;
+  }
+
+  QDataStream msg(b, IO_ReadOnly);
+  sendInput(player, msg);
+}
+
+void BosonScript::placeProduction(int player, int factoryid, float x, float y)
+{
+  if(!game())
+  {
+    boError() << k_funcinfo << "NULL game" << endl;
+    return;
+  }
+
+  Player* p = (Player*)(game()->findPlayer(player));
+
+  if(!p)
+  {
+    boError() << k_funcinfo << "No player with id " << player << endl;
+    return;
+  }
+
+  Unit* u = game()->findUnit(factoryid, p);
+  if(!u)
+  {
+    boError() << k_funcinfo << "No unit with id" << factoryid << endl;
+    return;
+  }
+
+  ProductionPlugin* prod = (ProductionPlugin*)u->plugin(UnitPlugin::Production);
+  if(!prod)
+  {
+    boError() << k_funcinfo << "Unit " << factoryid << " doesn't have production plugin!" << endl;
+    return;
+  }
+  if(prod->completedProductionType() != ProduceUnit)
+  {
+    boError() << k_lineinfo << "not producing unit!" << endl;
+    return;
+  }
+  int unitType = prod->completedProductionId();
+  if (unitType <= 0) {
+    // hope this is working...
+    boWarning() << k_lineinfo << "not yet completed" << endl;
+    return;
+  }
+
+  QByteArray b;
+  QDataStream stream(b, IO_WriteOnly);
+
+  BosonMessageMoveBuild message((Q_UINT32)ProduceUnit, p->id(), u->id(), BoVector2Fixed(x, y));
+  if(!message.save(stream))
+  {
+    boError() << k_funcinfo << "failed saving message " << message.messageId() << " to stream" << endl;
+    return;
+  }
 
   QDataStream msg(b, IO_ReadOnly);
   sendInput(player, msg);
@@ -524,7 +605,7 @@ QValueList<int> BosonScript::unitsInRect(int x1, int y1, int x2, int y2)
     return list;
   }
   BosonCollisions* c = canvas()->collisions();
-  BoItemList* l = c->collisionsAtCells(QRect(QPoint(x1, y1), QPoint(x2, y2)));
+  BoItemList* l = c->collisionsAtCells(BoRectFixed(BoVector2Fixed(x1, y1), BoVector2Fixed(x2, y2)));
   for(BoItemList::Iterator it = l->begin(); it != l->end(); ++it)
   {
     if(RTTI::isUnit((*it)->rtti()))
@@ -545,22 +626,22 @@ bool BosonScript::cellOccupied(int x, int y)
   return canvas()->cellOccupied(x, y);
 }
 
-QPoint BosonScript::unitPosition(int id)
+BoVector2Fixed BosonScript::unitPosition(int id)
 {
   if(!game())
   {
     boError() << k_funcinfo << "NULL game" << endl;
-    return QPoint(-1, -1);
+    return BoVector2Fixed(-1, -1);
   }
 
   Unit* u = game()->findUnit(id, 0);
   if(!u)
   {
     boError() << k_funcinfo << "No unit with id" << id << endl;
-    return QPoint(-1, -1);
+    return BoVector2Fixed(-1, -1);
   }
 
-  return QPoint((int)u->x(), (int)u->y());
+  return BoVector2Fixed(u->x(), u->y());
 }
 
 int BosonScript::unitOwner(int id)
@@ -635,6 +716,76 @@ bool BosonScript::isUnitMobile(int id)
   return u->isMobile();
 }
 
+bool BosonScript::isUnitTypeMobile(int playerid, int type)
+{
+  if(!game())
+  {
+    boError() << k_funcinfo << "NULL game" << endl;
+    return false;
+  }
+
+  Player* p = (Player*)(game()->findPlayer(playerid));
+
+  if(!p)
+  {
+    boError() << k_funcinfo << "No player with id " << playerid << endl;
+    return false;
+  }
+
+  const UnitProperties* prop = p->speciesTheme()->unitProperties(type);
+  if(!prop)
+  {
+    boError() << k_funcinfo << "No unit properties with typeid " << type << " for player with id " << playerid << endl;
+    return false;
+  }
+
+  return prop->isMobile();
+}
+
+bool BosonScript::isUnitAircraft(int id)
+{
+  if(!game())
+  {
+    boError() << k_funcinfo << "NULL game" << endl;
+    return false;
+  }
+
+  Unit* u = game()->findUnit(id, 0);
+  if(!u)
+  {
+    boError() << k_funcinfo << "No unit with id" << id << endl;
+    return false;
+  }
+
+  return u->unitProperties()->isAircraft();
+}
+
+bool BosonScript::isUnitTypeAircraft(int playerid, int type)
+{
+  if(!game())
+  {
+    boError() << k_funcinfo << "NULL game" << endl;
+    return false;
+  }
+
+  Player* p = (Player*)(game()->findPlayer(playerid));
+
+  if(!p)
+  {
+    boError() << k_funcinfo << "No player with id " << playerid << endl;
+    return false;
+  }
+
+  const UnitProperties* prop = p->speciesTheme()->unitProperties(type);
+  if(!prop)
+  {
+    boError() << k_funcinfo << "No unit properties with typeid " << type << " for player with id " << playerid << endl;
+    return false;
+  }
+
+  return prop->isAircraft();
+}
+
 bool BosonScript::canUnitShoot(int id)
 {
   if(!game())
@@ -989,270 +1140,192 @@ int BosonScript::playerUnitsOfTypeCount(int playerid, int type)
 
 void BosonScript::setCameraRotation(float r)
 {
-  autoCamera()->setRotation(r);
+  interface()->setCameraRotation(r);
 }
 
-void BosonScript::setCameraRadius(float r)
+void BosonScript::setCameraXRotation(float r)
 {
-  autoCamera()->setRadius(r);
+  interface()->setCameraXRotation(r);
 }
 
-void BosonScript::setCameraZ(float z)
+void BosonScript::setCameraDistance(float dist)
 {
-  autoCamera()->setZ(z);
+  interface()->setCameraDistance(dist);
 }
 
 void BosonScript::setCameraMoveMode(int mode)
 {
-  boDebug() << k_funcinfo << "mode: " << mode << endl;
-  autoCamera()->setMoveMode((BoAutoCamera::MoveMode)mode);
+  interface()->setCameraMoveMode(mode);
 }
 
-void BosonScript::setCameraLookAt(const BoVector3& pos)
+void BosonScript::setCameraInterpolationMode(int mode)
 {
-  autoCamera()->setLookAt(pos);
+  interface()->setCameraInterpolationMode(mode);
 }
 
-void BosonScript::setCameraPos(const BoVector3& pos)
+void BosonScript::setCameraPos(const BoVector3Float& pos)
 {
-  autoCamera()->setCameraPos(pos);
+  interface()->setCameraPos(pos);
 }
 
-void BosonScript::setCameraUp(const BoVector3& up)
+void BosonScript::setCameraLookAt(const BoVector3Float& pos)
 {
-  autoCamera()->setUp(up);
+  interface()->setCameraLookAt(pos);
+}
+
+void BosonScript::setCameraUp(const BoVector3Float& up)
+{
+  interface()->setCameraUp(up);
+}
+
+void BosonScript::addCameraLookAtPoint(const BoVector3Float& pos, float time)
+{
+  interface()->addCameraLookAtPoint(pos, time);
+}
+
+void BosonScript::addCameraPosPoint(const BoVector3Float& pos, float time)
+{
+  interface()->addCameraPosPoint(pos, time);
+}
+
+void BosonScript::addCameraUpPoint(const BoVector3Float& up, float time)
+{
+  interface()->addCameraUpPoint(up, time);
 }
 
 void BosonScript::setCameraLimits(bool on)
 {
-  camera()->setUseLimits(on);
+  interface()->setUseCameraLimits(on);
 }
 
 void BosonScript::setCameraFreeMode(bool on)
 {
-  camera()->setFreeMovement(on);
+  interface()->setCameraFreeMovement(on);
 }
 
 void BosonScript::commitCameraChanges(int ticks)
 {
-  autoCamera()->commitChanges(ticks);
+  interface()->commitCameraChanges(ticks);
 }
 
-BoVector3 BosonScript::cameraLookAt()
+BoVector3Float BosonScript::cameraLookAt()
 {
-  return camera()->lookAt();
+  return interface()->cameraLookAt();
 }
 
-BoVector3 BosonScript::cameraPos()
+BoVector3Float BosonScript::cameraPos()
 {
-  return camera()->cameraPos();
+  return interface()->cameraPos();
 }
 
-BoVector3 BosonScript::cameraUp()
+BoVector3Float BosonScript::cameraUp()
 {
-  return camera()->up();
+  return interface()->cameraUp();
 }
 
 float BosonScript::cameraRotation()
 {
-  return camera()->rotation();
+  return interface()->cameraRotation();
 }
 
-float BosonScript::cameraRadius()
+float BosonScript::cameraXRotation()
 {
-  return camera()->radius();
+  return interface()->cameraXRotation();
 }
 
-float BosonScript::cameraZ()
+float BosonScript::cameraDistance()
 {
-  return camera()->z();
+  return interface()->cameraDistance();
 }
 
 /*****  Light methods  *****/
-BoVector4 BosonScript::lightPos(int id)
+BoVector4Float BosonScript::lightPos(int id)
 {
-  BoLight* l = light(id);
-  if(!l)
-  {
-    boError() << k_funcinfo << "No light with id " << id << endl;
-    return BoVector4();
-  }
-
-  return l->position();
+  return interface()->lightPos(id);
 }
 
-BoVector4 BosonScript::lightAmbient(int id)
+BoVector4Float BosonScript::lightAmbient(int id)
 {
-  BoLight* l = light(id);
-  if(!l)
-  {
-    boError() << k_funcinfo << "No light with id " << id << endl;
-    return BoVector4();
-  }
-
-  return l->ambient();
+  return interface()->lightAmbient(id);
 }
 
-BoVector4 BosonScript::lightDiffuse(int id)
+BoVector4Float BosonScript::lightDiffuse(int id)
 {
-  BoLight* l = light(id);
-  if(!l)
-  {
-    boError() << k_funcinfo << "No light with id " << id << endl;
-    return BoVector4();
-  }
-
-  return l->diffuse();
+  return interface()->lightDiffuse(id);
 }
 
-BoVector4 BosonScript::lightSpecular(int id)
+BoVector4Float BosonScript::lightSpecular(int id)
 {
-  BoLight* l = light(id);
-  if(!l)
-  {
-    boError() << k_funcinfo << "No light with id " << id << endl;
-    return BoVector4();
-  }
-
-  return l->specular();
+  return interface()->lightSpecular(id);
 }
 
-BoVector3 BosonScript::lightAttenuation(int id)
+BoVector3Float BosonScript::lightAttenuation(int id)
 {
-  BoLight* l = light(id);
-  if(!l)
-  {
-    boError() << k_funcinfo << "No light with id " << id << endl;
-    return BoVector3();
-  }
-
-  return l->attenuation();
+  return interface()->lightAttenuation(id);
 }
 
 bool BosonScript::lightEnabled(int id)
 {
-  BoLight* l = light(id);
-  if(!l)
-  {
-    boError() << k_funcinfo << "No light with id " << id << endl;
-    return false;
-  }
-
-  return l->isEnabled();
+  return interface()->lightEnabled(id);
 }
 
-void BosonScript::setLightPos(int id, BoVector4 pos)
+void BosonScript::setLightPos(int id, BoVector4Float pos)
 {
-  BoLight* l = light(id);
-  if(!l)
-  {
-    boError() << k_funcinfo << "No light with id " << id << endl;
-    return;
-  }
-
-  l->setPosition(pos);
+  interface()->setLightPos(id, pos);
 }
 
-void BosonScript::setLightAmbient(int id, BoVector4 a)
+void BosonScript::setLightAmbient(int id, BoVector4Float a)
 {
-  BoLight* l = light(id);
-  if(!l)
-  {
-    boError() << k_funcinfo << "No light with id " << id << endl;
-    return;
-  }
-
-  l->setAmbient(a);
+  interface()->setLightAmbient(id, a);
 }
 
-void BosonScript::setLightDiffuse(int id, BoVector4 d)
+void BosonScript::setLightDiffuse(int id, BoVector4Float d)
 {
-  BoLight* l = light(id);
-  if(!l)
-  {
-    boError() << k_funcinfo << "No light with id " << id << endl;
-    return;
-  }
-
-  l->setDiffuse(d);
+  interface()->setLightDiffuse(id, d);
 }
 
-void BosonScript::setLightSpecular(int id, BoVector4 s)
+void BosonScript::setLightSpecular(int id, BoVector4Float s)
 {
-  BoLight* l = light(id);
-  if(!l)
-  {
-    boError() << k_funcinfo << "No light with id " << id << endl;
-    return;
-  }
-
-  l->setSpecular(s);
+  interface()->setLightSpecular(id, s);
 }
 
-void BosonScript::setLightAttenuation(int id, BoVector3 a)
+void BosonScript::setLightAttenuation(int id, BoVector3Float a)
 {
-  BoLight* l = light(id);
-  if(!l)
-  {
-    boError() << k_funcinfo << "No light with id " << id << endl;
-    return;
-  }
-
-  l->setAttenuation(a);
+  interface()->setLightAttenuation(id, a);
 }
 
 void BosonScript::setLightEnabled(int id, bool enable)
 {
-  BoLight* l = light(id);
-  if(!l)
-  {
-    boError() << k_funcinfo << "No light with id " << id << endl;
-    return;
-  }
-
-  l->setEnabled(enable);
+  interface()->setLightEnabled(id, enable);
 }
 
 int BosonScript::addLight()
 {
-  if(!display())
-  {
-    boError() << k_funcinfo << "NULL display" << endl;
-    return -1;
-  }
-  BoLight* l = display()->newLight();
-  if(!l)
-  {
-    return -1;
-  }
-  return l->id();
+  return interface()->addLight();
 }
 
 void BosonScript::removeLight(int id)
 {
-  if(!display())
-  {
-    boError() << k_funcinfo << "NULL display" << endl;
-    return;
-  }
-  display()->removeLight(id);
+  interface()->removeLight(id);
 }
 
 /*****  AI methods  *****/
 float BosonScript::aiDelay()
 {
-  return boConfig->aiDelay();
+  return (float)boConfig->doubleValue("AIDelay");
 }
 
 /*****  Other methods  *****/
 void BosonScript::startBenchmark()
 {
-  boProfiling->startBenchmark();
+// AB: benchmark feature got lost
+//  boProfiling->startBenchmark();
 }
 
 void BosonScript::endBenchmark(const QString& name)
 {
-  boProfiling->endBenchmark(name);
+// AB: benchmark feature got lost
+//  boProfiling->endBenchmark(name);
 }
 
 void BosonScript::setRandomSeed(long int seed)
@@ -1265,8 +1338,8 @@ void BosonScript::findPath(int x1, int y1, int x2, int y2)
 #ifdef PATHFINDER_TNG
  boDebug() << k_funcinfo << "Trying searching script path" << endl;
  BosonPathInfo i;
- i.start = QPoint(x1, y1);
- i.dest = QPoint(x2, y2);
+ i.start = BoVector2Fixed(x1, y1);
+ i.dest = BoVector2Fixed(x2, y2);
  boDebug() << k_funcinfo << "Let's go!" << endl;
  canvas()->pathfinder()->findPath(&i);
  boDebug() << k_funcinfo << "script path searching complete" << endl;
@@ -1275,7 +1348,7 @@ void BosonScript::findPath(int x1, int y1, int x2, int y2)
 #endif
 }
 
-void BosonScript::addEffect(int player, unsigned int id, BoVector3 pos, float zrot)
+void BosonScript::addEffect(unsigned int id, BoVector3Fixed pos, bofixed zrot)
 {
   if(!game())
   {
@@ -1283,22 +1356,202 @@ void BosonScript::addEffect(int player, unsigned int id, BoVector3 pos, float zr
     return;
   }
 
-  Player* p = (Player*)(game()->findPlayer(player));
+#warning FIXME: addEffect
+  boError() << k_funcinfo << "currently not supported" << endl;
+#if 0
+  const BosonEffectProperties* prop = boEffectPropertiesManager->effectProperties(id);
+  if(!prop)
+  {
+    boError() << k_funcinfo << "No effect properties with id " << id << endl;
+    return;
+  }
+  QPtrList<BosonEffect> list = BosonEffectProperties::newEffects(prop, pos, BoVector3Fixed(0, 0, zrot));
+  BosonCanvas* c = boGame->canvasNonConst();
+  c->addEffects(list);
+#endif
+}
+
+void BosonScript::addEffectToUnit(int unitid, unsigned int effectid)
+{
+  addEffectToUnit(unitid, effectid, BoVector3Fixed());
+}
+
+void BosonScript::addEffectToUnit(int unitid, unsigned int effectid, BoVector3Fixed offset, bofixed zrot)
+{
+  if(!game())
+  {
+    boError() << k_funcinfo << "NULL game" << endl;
+    return;
+  }
+
+  Unit* u = game()->findUnit(unitid, 0);
+  if(!u)
+  {
+    boError() << k_funcinfo << "No unit with id" << unitid << endl;
+    return;
+  }
+
+#warning FIXME: addEffectToUnit
+  boError() << k_funcinfo << "currently not supported" << endl;
+
+#if 0
+  const BosonEffectProperties* prop = boEffectPropertiesManager->effectProperties(effectid);
+  if(!prop)
+  {
+    boError() << k_funcinfo << "No effect properties with id " << effectid << endl;
+    return;
+  }
+  BoVector3Fixed pos(u->centerX(), u->centerY(), u->z());
+  pos += offset;
+  BoVector3Fixed rot(u->xRotation(), u->yRotation(), u->rotation());
+  rot.setZ(rot.z() + zrot);
+  QPtrList<BosonEffect> list = BosonEffectProperties::newEffects(prop, pos, rot);
+  QPtrListIterator<BosonEffect> it(list);
+  while(it.current())
+  {
+    u->addEffect(*it);
+    ++it;
+  }
+#endif
+}
+
+void BosonScript::advanceEffects(int ticks)
+{
+  if(!game())
+  {
+    boError() << k_funcinfo << "NULL game" << endl;
+    return;
+  }
+
+#warning fixme: advanceEffects
+  boError() << k_funcinfo << "currently not supported" << endl;
+#if 0
+  QPtrListIterator<BosonEffect> it(*game()->canvas()->effects());
+  while(it.current())
+  {
+    BosonEffect* e = it.current();
+    for(int i = 0; i < ticks; i++)
+    {
+      // This code is taken from BoCanvasAdvance::updateEffects()
+      if(!e->hasStarted())
+      {
+        e->update(0.05f);
+      }
+      else
+      {
+        e->markUpdate(0.05f);
+      }
+    }
+    ++it;
+  }
+#endif
+}
+
+void BosonScript::setWind(const BoVector3Float& wind)
+{
+#if 0
+  BosonEffectPropertiesParticle::setWind(wind);
+#else
+#warning FIXME
+#endif
+}
+
+static BoVector3Float g_wind(0.2, 0.1, 0.0);
+const BoVector3Float& BosonScript::wind()
+{
+#if 0
+  return BosonEffectPropertiesParticle::wind();
+#else
+#warning FIXME
+  return g_wind;
+#endif
+}
+
+void BosonScript::unfogPlayer(int playerid)
+{
+  if(!game())
+  {
+    boError() << k_funcinfo << "NULL game" << endl;
+    return;
+  }
+  if(!game()->playField())
+  {
+    boError() << k_funcinfo << "NULL playField" << endl;
+    return;
+  }
+  BosonMap* map = game()->playField()->map();
+  if(!map)
+  {
+    boError() << k_funcinfo << "NULL map" << endl;
+    return;
+  }
+
+  Player* p = (Player*)(game()->findPlayer(playerid));
 
   if(!p)
   {
-    boError() << k_funcinfo << "No player with id " << player << endl;
+    boError() << k_funcinfo << "No player with id " << playerid << endl;
     return;
   }
-
-  const BosonEffectProperties* prop = p->speciesTheme()->effectProperties(id);
-  if(!prop)
+  for(unsigned int x = 0; x < map->width(); x++)
   {
-    boError() << k_funcinfo << "No effect properties with id " << id << " for player " << player << endl;
-    return;
+    for(unsigned int y = 0; y < map->height(); y++)
+    {
+      p->unfog(x, y);
+    }
   }
-  QPtrList<BosonEffect> list = BosonEffectProperties::newEffects(prop, pos, BoVector3(0, 0, zrot));
-  BosonCanvas* c = boGame->canvasNonConst();
-  c->addEffects(list);
 }
 
+void BosonScript::unfogAllPlayers()
+{
+  if(!game())
+  {
+    boError() << k_funcinfo << "NULL game" << endl;
+    return;
+  }
+  if(!game()->playField())
+  {
+    boError() << k_funcinfo << "NULL playField" << endl;
+    return;
+  }
+  BosonMap* map = game()->playField()->map();
+  if(!map)
+  {
+    boError() << k_funcinfo << "NULL map" << endl;
+    return;
+  }
+  QPtrList<KPlayer> list = *game()->playerList();
+  for(unsigned int i = 0; i < list.count(); i++)
+  {
+    Player* p = (Player*)list.at(i);
+    for(unsigned int x = 0; x < map->width(); x++)
+    {
+      for(unsigned int y = 0; y < map->height(); y++)
+      {
+        p->unfog(x, y);
+      }
+    }
+  }
+}
+
+void BosonScript::setAcceptUserInput(bool accept)
+{
+  interface()->setAcceptUserInput(accept);
+}
+
+void BosonScript::addChatMessage(const QString& from, const QString& message)
+{
+  if(!game())
+  {
+    boError() << k_funcinfo << "NULL game" << endl;
+    return;
+  }
+
+  game()->slotAddChatSystemMessage(from, message);
+}
+
+
+/*
+ * vim: et sw=2
+ */
+
