@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2003-2004 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2003-2005 Rivo Laks (rivolaks@hot.ee)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,31 +22,49 @@
 
 #include "bosonscript.h"
 
+#include <qstring.h>
+#include <qdatastream.h>
+
 typedef struct _object PyObject;
 struct PyMethodDef;
 typedef struct _ts PyThreadState;
-class QString;
+
 
 class PythonScript : public BosonScript
 {
   public:
-    PythonScript(Player* p);
+    PythonScript(int playerId = -1);
     virtual ~PythonScript();
 
-    virtual void loadScript(QString file);
+    virtual bool loadScript(QString file);
+    virtual bool loadScriptFromString(const QString& string);
 
-    virtual void advance();
-    virtual void init();
+    virtual bool advance();
+    virtual bool init();
+    virtual void setPlayerId(int id);
 
-    virtual void callFunction(QString function);
-    virtual void callFunction(QString function, PyObject* args);
+    virtual bool load(QDataStream& stream);
+    virtual bool save(QDataStream& stream);
+
+    virtual void callFunction(const QString& function);
+    void callFunction(const QString& function, PyObject* args);
+    virtual int callFunctionWithReturn(const QString& function);
+    int callFunctionWithReturn(const QString& function, PyObject* args);
 
     virtual void execLine(const QString& line);
 
+    virtual void callEventHandler(const BoEvent* e, const QString& function, const QString& args);
+
+
+    // Events
+    static PyObject* py_addEventHandler(PyObject* self, PyObject* args);
+    static PyObject* py_removeEventHandler(PyObject* self, PyObject* args);
+
 
     // Players
     static PyObject* py_areEnemies(PyObject* self, PyObject* args);
     static PyObject* py_allPlayers(PyObject* self, PyObject* args);
+    static PyObject* py_isNeutral(PyObject* self, PyObject* args);
 
 
     // Resources
@@ -69,6 +87,7 @@ class PythonScript : public BosonScript
     static PyObject* py_produceUnit(PyObject* self, PyObject* args);
     static PyObject* py_spawnUnit(PyObject* self, PyObject* args);
     static PyObject* py_teleportUnit(PyObject* self, PyObject* args);
+    static PyObject* py_placeProduction(PyObject* self, PyObject* args);
 
     static PyObject* py_unitsOnCell(PyObject* self, PyObject* args);
     static PyObject* py_unitsInRect(PyObject* self, PyObject* args);
@@ -79,6 +98,9 @@ class PythonScript : public BosonScript
     static PyObject* py_unitType(PyObject* self, PyObject* args);
     static PyObject* py_unitWork(PyObject* self, PyObject* args);
     static PyObject* py_isUnitMobile(PyObject* self, PyObject* args);
+    static PyObject* py_isUnitTypeMobile(PyObject* self, PyObject* args);
+    static PyObject* py_isUnitAircraft(PyObject* self, PyObject* args);
+    static PyObject* py_isUnitTypeAircraft(PyObject* self, PyObject* args);
     static PyObject* py_canUnitShoot(PyObject* self, PyObject* args);
     static PyObject* py_canUnitTypeShoot(PyObject* self, PyObject* args);
     static PyObject* py_canUnitProduce(PyObject* self, PyObject* args);
@@ -98,14 +120,18 @@ class PythonScript : public BosonScript
 
     // Camera
     static PyObject* py_setCameraRotation(PyObject* self, PyObject* args);
-    static PyObject* py_setCameraRadius(PyObject* self, PyObject* args);
-    static PyObject* py_setCameraZ(PyObject* self, PyObject* args);
+    static PyObject* py_setCameraXRotation(PyObject* self, PyObject* args);
+    static PyObject* py_setCameraDistance(PyObject* self, PyObject* args);
     static PyObject* py_setCameraMoveMode(PyObject* self, PyObject* args);
+    static PyObject* py_setCameraInterpolationMode(PyObject* self, PyObject* args);
     static PyObject* py_setCameraLookAt(PyObject* self, PyObject* args);
     static PyObject* py_setCameraPos(PyObject* self, PyObject* args);
     static PyObject* py_setCameraUp(PyObject* self, PyObject* args);
     static PyObject* py_setCameraLimits(PyObject* self, PyObject* args);
     static PyObject* py_setCameraFreeMode(PyObject* self, PyObject* args);
+    static PyObject* py_addCameraLookAtPoint(PyObject* self, PyObject* args);
+    static PyObject* py_addCameraUpPoint(PyObject* self, PyObject* args);
+    static PyObject* py_addCameraPosPoint(PyObject* self, PyObject* args);
 
     static PyObject* py_commitCameraChanges(PyObject* self, PyObject* args);
 
@@ -113,8 +139,8 @@ class PythonScript : public BosonScript
     static PyObject* py_cameraPos(PyObject* self, PyObject* args);
     static PyObject* py_cameraUp(PyObject* self, PyObject* args);
     static PyObject* py_cameraRotation(PyObject* self, PyObject* args);
-    static PyObject* py_cameraRadius(PyObject* self, PyObject* args);
-    static PyObject* py_cameraZ(PyObject* self, PyObject* args);
+    static PyObject* py_cameraXRotation(PyObject* self, PyObject* args);
+    static PyObject* py_cameraDistance(PyObject* self, PyObject* args);
 
 
     // Lights
@@ -146,6 +172,14 @@ class PythonScript : public BosonScript
     static PyObject* py_setRandomSeed(PyObject* self, PyObject* args);
     static PyObject* py_findPath(PyObject* self, PyObject* args);
     static PyObject* py_addEffect(PyObject* self, PyObject* args);
+    static PyObject* py_addEffectToUnit(PyObject* self, PyObject* args);
+    static PyObject* py_advanceEffects(PyObject* self, PyObject* args);
+    static PyObject* py_wind(PyObject* self, PyObject* args);
+    static PyObject* py_setWind(PyObject* self, PyObject* args);
+    static PyObject* py_unfogPlayer(PyObject* self, PyObject* args);
+    static PyObject* py_unfogAllPlayers(PyObject* self, PyObject* args);
+    static PyObject* py_setAcceptUserInput(PyObject* self, PyObject* args);
+    static PyObject* py_addChatMessage(PyObject* self, PyObject* args);
 
   protected:
     static PyObject* QValueListToPyList(QValueList<int>* list);
@@ -156,9 +190,14 @@ class PythonScript : public BosonScript
     void getPythonLock();
     void freePythonLock();
 
+    PyObject* saveModule(PyObject* module) const;
+    void loadModule(PyObject* module, PyObject* data);
+
   private:
+    PyObject* mMainModule;
     PyObject* mDict;
     PyThreadState* mInterpreter;
+    QString mLoadedScripts;
 
     static PyMethodDef mCallbacks[];
     static bool mScriptingInited;
