@@ -1,6 +1,8 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 1999-2000,2001-2004 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 1999-2000 Thomas Capricelli (capricel@email.enst.fr)
+    Copyright (C) 2001-2005 Andreas Beckermann (b_mann@gmx.de)
+    Copyright (C) 2001-2005 Rivo Laks (rivolaks@hot.ee)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,6 +24,8 @@
 #include "unitbase.h"
 #include "global.h"
 
+#include <qvaluevector.h>
+
 class Player;
 class BosonCanvas;
 class BoItemList;
@@ -31,16 +35,22 @@ class Facility;
 class ProductionPlugin;
 class UnitPlugin;
 class RepairPlugin;
-class BosonEffect;
 class BosonWeapon;
+class BosonMoveData;
 class BosonPathInfo;
-class BoVector3;
+template<class T> class BoVector2;
+template<class T> class BoVector3;
+template<class T> class BoRect;
+typedef BoVector2<bofixed> BoVector2Fixed;
+typedef BoVector3<bofixed> BoVector3Fixed;
+typedef BoRect<bofixed> BoRectFixed;
 template<class T> class QValueList;
 template<class T> class QPtrList;
 class QDomElement;
 
 class KGameUnitDebug;
 
+class UnitPrivate;
 /**
  * Implementation of the visual parts of a unit. As far as possible all stuff
  * should go to UnitBase directly - except the visual stuff. (UPDATE 01/12/27:
@@ -55,38 +65,50 @@ class Unit : public UnitBase
 {
 public:
 	enum PropertyIds {
-		// properties in Unit
-		IdWaypoints = UnitBase::IdLast + 2,
-		IdWantedRotation = UnitBase::IdLast + 6,
-		IdPathPoints = UnitBase::IdLast + 10,
+		// properties in Unit. IDs from 1024 to 1279 may be used here
+		// (1024+0 .. 1024+255)
+		IdWaypoints = 1024 + 2,
+		IdWantedRotation = 1024 + 6,
+		IdPathPoints = 1024 + 10,
 
-		// properties in MobileUnit
-
-		// properties in Facility
-		IdConstructionStep = UnitBase::IdLast + 100,
+		// properties in MobileUnit or Facility.
+		// IDs from 1280 to 1535 may be used here
+		// (1280+0 .. 1280+255)
+		IdConstructionStep = 1280 + 100,
 
 		// properties in UnitPlugins and derived classes
-		IdProductionState = UnitBase::IdLast + 300,
-		IdResourcesMined = UnitBase::IdLast + 301,
-		IdResourcesX = UnitBase::IdLast + 302,
-		IdResourcesY = UnitBase::IdLast + 303,
-		IdHarvestingType = UnitBase::IdLast + 304,
-		IdBombingPosX = UnitBase::IdLast + 305,
-		IdBombingPosY = UnitBase::IdLast + 306,
-		IdMinePlacingCounter = UnitBase::IdLast + 307,
-		IdResourceMineMinerals = UnitBase::IdLast + 308,
-		IdResourceMineOil = UnitBase::IdLast + 309
+		// IDs from 1536 to 4095 may be used here
+		// (1536+0 .. 1536+3071)
+		IdProductionState = 1536 + 0,
+		IdResourcesMined = 1536 + 1,
+		IdResourcesX = 1536 + 2,
+		IdResourcesY = 1536 + 3,
+		IdHarvestingType = 1536 + 4,
+		IdBombingTargetX = 1536 + 5,
+		IdBombingTargetY = 1536 + 6,
+		IdMinePlacingCounter = 1536 + 7,
+		IdResourceMineMinerals = 1536 + 8,
+		IdResourceMineOil = 1536 + 9,
+		IdBombingDropDist = 1536 + 10,
+		IdBombingLastDistFromDropPoint = 1536 + 11
 	};
 
 	Unit(const UnitProperties* prop, Player* owner, BosonCanvas* canvas);
 	virtual ~Unit();
 
+	virtual bool init();
+
 	static void initStatic();
 
 	inline virtual int rtti() const { return UnitBase::rtti(); }
 
+	void setMoveData(BosonMoveData* moveData);
+	BosonMoveData* moveData() const;
+
 	virtual void setHealth(unsigned long int h);
 
+	virtual void setSightRange(unsigned long int r);
+
 	/**
 	 * @return The current plugin, used in e.g. @ref advance or NULL if
 	 * none. Always NULL if @ref work != WorkPlugin
@@ -94,7 +116,7 @@ public:
 	inline UnitPlugin* currentPlugin() const { return mCurrentPlugin; }
 
 	/**
-	 * @return The @refUnitPlugin::pluginType of the @ref currentPlugin or
+	 * @return The @ref UnitPlugin::pluginType of the @ref currentPlugin or
 	 * 0 if there is no plugin.
 	 **/
 	int currentPluginType() const;
@@ -107,6 +129,16 @@ public:
 	virtual void setWork(WorkType w);
 	void setPluginWork(int pluginType);
 
+	/**
+	 * Called when @p unit is destroyed. If the unit is a target of this
+	 * unit, it can now decide to reset the target.
+	 *
+	 * Note that @p unit usually will remain on the map as a wreckage for a
+	 * certain time. @ref itemRemoved is called when it is finally removed
+	 * completely.
+	 **/
+	virtual void unitDestroyed(Unit* unit);
+
 	/**
 	 * We are about to remove @p item from the game. Make sure that this
 	 * unit does not use that item in any way (e.g. as a target)
@@ -123,29 +155,14 @@ public:
 	 * Moves unit by specified amount and also calculates new z-coordinate and
 	 * rotation.
 	 * This should only be called by BosonCanvas' advance methods.
-	 *
-	 * Note that we use float all over in boson, since mesa uses float
-	 * internally. We won't gain precision by double but we will lose some
-	 * performance
 	 **/
-	virtual void moveBy(float x, float y, float z);
+	virtual void moveBy(bofixed x, bofixed y, bofixed z);
 
 	/**
-	 * There is not much to do here. Keep the stuff in this function as
-	 * simple as possible as it's called for <em>every</em> unit on
-	 * <em>every</em> advance call. An example of what could be done here is
-	 * to increase the realod state.
-	 *
-	 * The really interesting things get done in the advanceXYZ() functions
-	 * below. They are called from @ref BosonCanvas::slotAdvance(). For
-	 * every @ref UnitBase::WorkType there is at least one advance function which
-	 * implements its behaviour.
-	 *
-	 * Please note that the @ref UnitPlugin::advance methods get called
-	 * here, too!
-	 * @param advanceCount Used by @ref UnitPlugin::advance
+	 * Reload shields and weapons. @p count specifies how much reloading
+	 * should be done (note that this is <em>not</em> the advanceCallsCount
 	 **/
-	virtual void advance(unsigned int advanceCount);
+	virtual void reload(unsigned int count);
 
 	/**
 	 * Call the advance*() function that is currently used. The advance
@@ -158,9 +175,9 @@ public:
 	 * for a lot of units and with this we avoid any switch on @ref
 	 * advanceWork
 	 **/
-	inline virtual void advanceFunction(unsigned int advanceCount)
+	inline virtual void advanceFunction(unsigned int advanceCallsCount)
 	{
-		(this->*mAdvanceFunction)(advanceCount);
+		(this->*mAdvanceFunction)(advanceCallsCount);
 	}
 
 	/**
@@ -172,9 +189,9 @@ public:
 	 *
 	 * See also @ref Boson::advanceFlag and @ref Boson::toggleAdvanceFlag
 	 **/
-	inline virtual void advanceFunction2(unsigned int advanceCount)
+	inline virtual void advanceFunction2(unsigned int advanceCallsCount)
 	{
-		(this->*mAdvanceFunction2)(advanceCount);
+		(this->*mAdvanceFunction2)(advanceCallsCount);
 	}
 
 
@@ -206,9 +223,9 @@ public:
 	 * be called after @ref advanceMoveInternal, so you should never call @ref
 	 * advanceMoveInternal directly, but rather use this.
 	 **/
-	void advanceMove(unsigned int advanceCount)
+	void advanceMove(unsigned int advanceCallsCount)
 	{
-		advanceMoveInternal(advanceCount);
+		advanceMoveInternal(advanceCallsCount);
 		advanceMoveCheck();
 	}
 
@@ -216,7 +233,7 @@ public:
 	/**
 	 * Attack a unit. The target was set before using @ref setTarget
 	 **/
-	virtual void advanceAttack(unsigned int advanceCount);
+	virtual void advanceAttack(unsigned int advanceCallsCount);
 
 	/**
 	 * Follow another unit.
@@ -224,12 +241,20 @@ public:
 	 **/
 	virtual void advanceFollow(unsigned int) { }
 
+	/**
+	 * This is called when the unit is supposed to do nothing.
+	 *
+	 * Note that the game may decide _not_ to call this advance function at
+	 * all, as it is a noop anyway. Do NOT implement anything here.
+	 **/
+	virtual void advanceNone(unsigned int advanceCallsCount);
+
 	/**
 	 * This is called when there is nothing else to do for this unit.
 	 * Usually the unit will check for enemy units in range and fire at
 	 * them.
 	 **/
-	virtual void advanceNone(unsigned int advanceCount);
+	virtual void advanceIdle(unsigned int advanceCallsCount);
 
 	/**
 	 * Move the construction animation one step forward. Does nothing by
@@ -243,12 +268,12 @@ public:
 	 *
 	 * Should do - just like all advance*() methods - as little as possible.
 	 **/
-	virtual void advanceDestroyed(unsigned int advanceCount);
+	virtual void advanceDestroyed(unsigned int advanceCallsCount);
 
 	/**
 	 * Call the @ref UnitPlugin::advance method of the @ref currentPlugin
 	 **/
-	virtual void advancePlugin(unsigned int advanceCount);
+	virtual void advancePlugin(unsigned int advanceCallsCount);
 
 	/**
 	 * Called when unit has to turn to some direction. It smootly turns unit
@@ -271,9 +296,9 @@ public:
 	/**
 	 * Add pos to the waypoint list.
 	 **/
-	void addWaypoint(const QPoint& pos);
+	void addWaypoint(const BoVector2Fixed& pos);
 
-	const QPoint& currentWaypoint() const;
+	const BoVector2Fixed& currentWaypoint() const;
 	unsigned int waypointCount() const;
 	/**
 	 * Removes all waypoints from the list.
@@ -288,12 +313,12 @@ public:
 	/**
 	 * @return A list of all waypoints for debugging
 	 **/
-	const QValueList<QPoint>& waypointList() const;
+	const QValueList<BoVector2Fixed>& waypointList() const;
 
 	/**
 	 * @return List of path-points
 	 **/
-	const QValueList<QPoint>& pathPointList() const;
+	const QValueList<BoVector2Fixed>& pathPointList() const;
 
 	/**
 	 * Move this unit to a specified point. Also make sure that previous
@@ -302,7 +327,14 @@ public:
 	 * @param pos The point on the canvas to move to.
 	 * @param attack If this is true, unit will stop and attack any enemy units in range while moving
 	 **/
-	virtual void moveTo(const QPoint& pos, bool attack = false);
+	virtual void moveTo(const BoVector2Fixed& pos, bool attack = false);
+
+	/**
+	 * Moves this unit to the target.
+	 * range specifies distance between target and this unit in cells. If range
+	 * is 0, unit tries to move next to the target.
+	 **/
+	bool moveTo(BosonItem* target, int range = 0);
 
 
 	// TODO: maybe make this protected?
@@ -312,11 +344,11 @@ public:
 	 * point. You shouldn't use this directly, but instead use 'client methods',
 	 * which will then call this one.
 	 *
-	 * Moves unit's _center_ exactly to given position. If range is not 0, unit
-	 * will move until it's less that range _cells_ away from destination.
+	 * Moves unit's _center_ exactly to given position. If range is not -1, unit
+	 * will move until it's at most that range _cells_ away from destination.
 	 * Destination is in canvas coords.
 	 **/
-	bool moveTo(float x, float y, int range = 0);
+	bool moveTo(bofixed x, bofixed y, int range = -1);
 
 	/**
 	 * Turns unit smoothly to given degrees
@@ -331,7 +363,7 @@ public:
 
 	/**
 	 * Just stop moving. Don't call this if you don't want to stop attacking
-	 * as well! This sets @ref work to @ref WorkNone
+	 * as well! This sets @ref work to @ref WorkIdle
 	 **/
 	virtual void stopMoving();
 	void stopAttacking();
@@ -339,8 +371,6 @@ public:
 	virtual bool saveAsXML(QDomElement& root);
 	virtual bool loadFromXML(const QDomElement& root);
 
-	void updateSelectBox();
-
 	/**
 	 * @return All units except this that are @ref range or less cells away from this unit
 	 * Note that it doesn't include units on cells that are not visible
@@ -365,13 +395,13 @@ public:
 	 * If unit isn't moving, returned value is undefined.
 	 * Returned value is in canvas coordinates.
 	 **/
-	int destinationX() const;
+	bofixed destinationX() const;
 	/**
 	 * @return Y-coordinate of unit's current destination (where it's moving).
 	 * If unit isn't moving, returned value is undefined.
 	 * Returned value is in canvas coordinates.
 	 **/
-	int destinationY() const;
+	bofixed destinationY() const;
 	/**
 	 * @return Current moving range of the unit (how close to destination point it
 	 *  must move).
@@ -406,21 +436,6 @@ public:
 	 **/
 	bool isNextTo(Unit* unit) const;
 
-	void playSound(UnitSoundEvent event);
-
-	/**
-	 * @return List of active effects this unit has.
-	 * This may include e.g. smoke for factories.
-	 **/
-	virtual const QPtrList<BosonEffect>* effects() const;
-
-	/**
-	 * Clear the effects list. Note that the effects are
-	 * <em>not</em> deleted - @ref BosonCanvas will take care of this
-	 * anyway. Just the @ref effects list is meant to be cleared.
-	 **/
-	virtual void clearEffects();
-
 	void loadWeapons();
 
 	bool canShootAt(Unit* u);
@@ -439,24 +454,34 @@ public:
 	/**
 	 * @return Square of distance between center points of this unit and u
 	 **/
-	int distance(const Unit* u) const;
+	bofixed distance(const Unit* u) const;
 	/**
 	 * @return Square of distance between center point of this unit and pos
 	 **/
-	int distance(const BoVector3& pos) const;
+	bofixed distance(const BoVector3Fixed& pos) const;
 
 	/**
 	 * @return Weapon with given id for this unit. 0 if it doesn't exist.
 	 **/
 	BosonWeapon* weapon(unsigned long int id) const;
 
+	/**
+	 * @return maximum range of weapons of this unit e.g. range of weapon with the longest range
+	 **/
+	unsigned long int maxWeaponRange() const;
+	unsigned long int maxAirWeaponRange() const;
+	unsigned long int maxLandWeaponRange() const;
+
 
 	virtual void setMovingStatus(MovingStatus m);
 
+	virtual void addUpgrade(const UpgradeProperties* upgrade);
+	virtual void removeUpgrade(const UpgradeProperties* upgrade);
+
+	virtual void flyInCircle() {}
+
 
 protected:
-	void setEffects(const QPtrList<BosonEffect>& list);
-
 	void shootAt(BosonWeapon* w, Unit* target);
 
 	/**
@@ -465,16 +490,6 @@ protected:
 	 **/
 	QValueList<Unit*> unitCollisions(bool exact = false);
 
-	/**
-	 * Finds new path to destination.
-	 * Destination must have been set before in variables movedestx and
-	 * movedesty - usually using @ref moveTo
-	 *
-	 * This is in Unit instead of @ref MobileUnit so that we can apply a
-	 * path to newly constructed units of factories.
-	 **/
-	virtual void newPath();
-
 	/**
 	 * Move the unit. By default this does nothing. Reimplemented in @ref
 	 * MobileUnit
@@ -498,7 +513,7 @@ protected:
 	 * Adds so-called path point for this unit.
 	 * Path points are like waypoints, except that they are internal.
 	 **/
-	void addPathPoint(const QPoint& pos);
+	void addPathPoint(const BoVector2Fixed& pos);
 	/**
 	 * @return Number of path-points
 	 **/
@@ -506,7 +521,7 @@ protected:
 	/**
 	 * @return Current path-point (where unit should go next)
 	 **/
-	const QPoint& currentPathPoint();
+	const BoVector2Fixed& currentPathPoint();
 	/**
 	 * Clears list of path-points
 	 **/
@@ -527,17 +542,23 @@ protected:
 	 * implemented. moveZ is simply set to the highest z value of the
 	 * corners of all cells it occupies.
 	 **/
-	void updateZ(float moveByX, float moveByY, float* moveZ, float* rotateX, float* rotateY);
+	void updateZ(bofixed moveByX, bofixed moveByY, bofixed* moveZ, bofixed* rotateX, bofixed* rotateY);
 
 	virtual const QColor* teamColor() const;
 
+	void recalculateMaxWeaponRange();
+
+	// TODO: move to BosonCanvas or to somewhere else
+	bool cellOccupied(int x, int y, bool ignoremoving = false) const;
+
+	virtual int getAnimationMode() const;
+
 
 private:
-	typedef void (Unit::*MemberFunction)(unsigned int advanceCount);
+	typedef void (Unit::*MemberFunction)(unsigned int advanceCallsCount);
 	void setAdvanceFunction(MemberFunction, bool advanceFlag);
 
 private:
-	class UnitPrivate;
 	UnitPrivate* d;
 	UnitPlugin* mCurrentPlugin;
 
@@ -546,7 +567,7 @@ private:
 	// now we come to the interesting slightly hackish performance part :-)
 	// we store a pointer to the advance*() method that is currently used.
 	// we do NULL checking on this pointer - but it *must not* happen, that
-	// this pointer is null (advanceNone instead).
+	// this pointer is null (advanceIdle instead).
 	// we store 2 pointers - see advanceFunction2() on this.
 	MemberFunction mAdvanceFunction;
 	MemberFunction mAdvanceFunction2;
@@ -568,6 +589,8 @@ public:
 	MobileUnit(const UnitProperties* prop, Player* owner, BosonCanvas* canvas);
 	virtual ~MobileUnit();
 
+	virtual bool init();
+
 	/**
 	 * Turn to direction. This sets a new frame according to the new
 	 * direction.
@@ -580,7 +603,9 @@ public:
 	void turnTo();
 
 
-	virtual void advanceFollow(unsigned int advanceCount);
+	virtual void advanceFollow(unsigned int advanceCallsCount);
+
+	virtual void advanceIdle(unsigned int advanceCallsCount);
 
 
 	/**
@@ -589,8 +614,6 @@ public:
 	 **/
 	void repairAt(Facility* repairYard);
 
-	virtual QRect boundingRect() const;
-
 	virtual bool saveAsXML(QDomElement& root);
 	virtual bool loadFromXML(const QDomElement& root);
 
@@ -605,18 +628,32 @@ public:
 	 *
 	 * @return How much is moved (will be <= maxdist)
 	 **/
-	float moveTowardsPoint(const QPoint& p, float x, float y, float maxdist, float &xspeed, float &yspeed);
-
-	virtual void newPath();
+	bofixed moveTowardsPoint(const BoVector2Fixed& p, bofixed x, bofixed y, bofixed maxdist, bofixed &xspeed, bofixed &yspeed);
 
 	/**
-	 * Check if pathpoint p marks end of the path. If yes, then it stops unit,
-	 * turns to random direction and true, otherwise returns false.
+	 * @return How fast this mobile unit accelerates.
 	 **/
-	bool checkPathPoint(const QPoint& p);
+	bofixed maxAccelerationSpeed() const;
+
+	/**
+	 * @return How fast this mobile unit decelerates.
+	 **/
+	bofixed maxDecelerationSpeed() const;
+
+	virtual void addUpgrade(const UpgradeProperties* upgrade);
+	virtual void removeUpgrade(const UpgradeProperties* upgrade);
+
+	static void initCellIntersectionTable();
+
+	virtual void flyInCircle();
 
 protected:
-	virtual void advanceMoveInternal(unsigned int advanceCount); // move one step futher to path
+	virtual void advanceMoveInternal(unsigned int advanceCallsCount); // move one step futher to path
+
+	// TODO: rename?
+	void advanceMoveLeader(unsigned int advanceCallsCount);
+	void advanceMoveFollowing(unsigned int advanceCallsCount);
+	void advanceMoveFlying(unsigned int advanceCallsCount);
 
 	/**
 	 * Note: this is not actually an advance*() method, like @ref
@@ -627,12 +664,35 @@ protected:
 	 **/
 	virtual void advanceMoveCheck();
 
+	/**
+	 * Finds new path to destination.
+	 * Destination must have been set before in @ref pathInfo
+	 *
+	 * This is in Unit instead of @ref MobileUnit so that we can apply a
+	 * path to newly constructed units of factories.
+	 **/
+	virtual bool newPath();
+
+	int selectNextPathPoint(int xpos, int ypos);
+	void avoidance();
+	bool canGoToCurrentPathPoint(int xpos, int ypos);
+	void currentPathPointChanged(int unitx, int unity);
+
+private:
+	void changeUpgrades(const UpgradeProperties* upgrade, bool add);
+
 private:
 	// a d pointer is probably not very good here - far too much memory consumption
 	// same apllies to Unit and UnitBase. But it speeds up compiling as we don't
 	// have to change the headers every time...
 	class MobileUnitPrivate;
 	MobileUnitPrivate* d;
+
+	BoUpgradeableProperty<bofixed> mMaxSpeed;
+	BoUpgradeableProperty<bofixed> mMaxAccelerationSpeed;
+	BoUpgradeableProperty<bofixed> mMaxDecelerationSpeed;
+
+	static QValueVector<BoVector2Fixed> mCellIntersectionTable[11][11];
 };
 
 // if you add class members - ONLY KGameProperties!! otherwise Player::load and
@@ -646,6 +706,8 @@ public:
 	Facility(const UnitProperties* prop, Player* owner, BosonCanvas* canvas);
 	virtual ~Facility();
 
+	virtual bool init();
+
 	/**
 	 * The construction steps are the number of frames until the complete
 	 * pixmap of the facility is shown.
@@ -688,14 +750,14 @@ public:
 	 * Does nothing if @ref isConstructionComplete is false - otherwise the
 	 * same as @ref Unit::moveTo
 	 **/
-	virtual void moveTo(float x, float y, int range = 0);
+	virtual void moveTo(bofixed x, bofixed y, int range = 0);
 
 	/**
 	 * Advance the construction animation. This is usually called when
 	 * placing the unit until the construction is completed. See @ref
 	 * isConstructionComplete
 	 **/
-	virtual void advanceConstruction(unsigned int advanceCount);
+	virtual void advanceConstruction(unsigned int advanceCallsCount);
 
 	/**
 	 * @return NULL if the facility has not yet been fully constructed,
@@ -706,6 +768,9 @@ public:
 	virtual bool saveAsXML(QDomElement& root);
 	virtual bool loadFromXML(const QDomElement& root);
 
+protected:
+	virtual int getAnimationMode() const;
+
 private:
 	class FacilityPrivate;
 	FacilityPrivate* d;
