@@ -1,6 +1,8 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 1999-2000,2001-2004 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 1999-2000 Thomas Capricelli (capricel@email.enst.fr)
+    Copyright (C) 2001-2005 Andreas Beckermann (b_mann@gmx.de)
+    Copyright (C) 2001-2005 Rivo Laks (rivolaks@hot.ee)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,113 +21,94 @@
 #include "unitproperties.h"
 #include "defines.h"
 
+#include "../bomemory/bodummymemory.h"
 #include "speciestheme.h"
 #include "pluginproperties.h"
-#include "bosoneffect.h"
-#include "bosoneffectproperties.h"
 #include "bosonweapon.h"
 #include "bosonconfig.h"
 #include "bodebug.h"
-#include "boaction.h"
+#include "bosonprofiling.h"
+#include "upgradeproperties.h"
 
 #include <ksimpleconfig.h>
 #include <klocale.h>
+#include <kmdcodec.h>
 
-class UnitProperties::UnitPropertiesPrivate
+#include <qfile.h>
+
+class UnitPropertiesPrivate
 {
 public:
 	UnitPropertiesPrivate()
 	{
 	}
 
+	QCString mMD5;
 	QString mName;
+	QString mDescription;
 	QString mUnitPath; // the path to the unit files
 	QValueList<unsigned long int> mRequirements;
 
-
 	QPtrList<PluginProperties> mPlugins;
 
 	QMap<QString, QString> mTextureNames;
 	QMap<int, QString> mSounds;
 
-	QIntDict<BoAction> mActions;
+	QMap<int, QString> mActionStrings;
 
-	QPtrList<BosonEffectProperties> mDestroyedEffects;
 	QValueList<unsigned long int> mDestroyedEffectIds;
-	QPtrList<BosonEffectProperties> mConstructedEffects;
 	QValueList<unsigned long int> mConstructedEffectIds;
-	QPtrList<BosonEffectProperties> mExplodingFragmentFlyEffects;
 	QValueList<unsigned long int> mExplodingFragmentFlyEffectIds;
-	QPtrList<BosonEffectProperties> mExplodingFragmentHitEffects;
 	QValueList<unsigned long int> mExplodingFragmentHitEffectIds;
-	BoVector3 mHitPoint;  // FIXME: better name
-};
+	BoVector3Fixed mHitPoint;  // FIXME: better name
 
-class UnitProperties::MobileProperties
-{
-public:
-	MobileProperties()
-	{
-	}
-
-	float mSpeed;
-	float mAccelerationSpeed;
-	float mDecelerationSpeed;
-	int mRotationSpeed;
-	bool mCanGoOnLand;
-	bool mCanGoOnWater;
-};
-
-class UnitProperties::FacilityProperties
-{
-public:
-	FacilityProperties()
-	{
-	}
-
-	unsigned int mConstructionFrames;
+	BoUpgradesCollection mUpgradesCollection;
 };
 
 
-UnitProperties::UnitProperties(bool fullmode)
-{
- init();
- mFullMode = fullmode;
-}
-
-UnitProperties::UnitProperties(SpeciesTheme* theme)
+UnitProperties::UnitProperties(SpeciesTheme* theme, bool fullMode)
+	: BoBaseValueCollection(),
+	mHealth(this, "Health", "MaxValue"),
+	mSightRange(this, "SightRange", "MaxValue"),
+	mProductionTime(this, "ProductionTime", "MaxValue"),
+	mMineralCost(this, "MineralCost", "MaxValue"),
+	mOilCost(this, "OilCost", "MaxValue"),
+	mArmor(this, "Armor", "MaxValue"),
+	mShields(this, "Shields", "MaxValue"),
+	mSpeed(this, "Speed", "MaxValue"),
+	mAccelerationSpeed(this, "AccelerationSpeed", "MaxValue"),
+	mDecelerationSpeed(this, "DecelerationSpeed", "MaxValue")
 {
  init();
  mTheme = theme;
-}
-
-UnitProperties::UnitProperties(SpeciesTheme* theme, const QString& fileName, bool fullload)
-{
- init();
- mTheme = theme;
- loadUnitType(fileName, fullload);
+ mFullMode = fullMode;
 }
 
 void UnitProperties::init()
 {
  d = new UnitPropertiesPrivate;
+ mFullMode = true;
  mTheme = 0;
- mMobileProperties = 0;
- mFacilityProperties = 0;
+ mIsFacility = false;
  d->mPlugins.setAutoDelete(true);
 }
 
 UnitProperties::~UnitProperties()
 {
- delete mMobileProperties;
- delete mFacilityProperties;
  delete d;
 }
 
-void UnitProperties::loadUnitType(const QString& fileName, bool fullmode)
+bool UnitProperties::loadUnitType(const QString& fileName, bool fullmode)
 {
  mFullMode = fullmode;
  bool isFacility;
+ QFile file(fileName);
+ if (!file.open(IO_ReadOnly)) {
+	boError() << k_funcinfo << "could not open " << fileName << endl;
+	return false;
+ }
+ KMD5 md5(file.readAll());
+ d->mMD5 = md5.hexDigest();
  KSimpleConfig conf(fileName);
  conf.setGroup(QString::fromLatin1("Boson Unit"));
 
@@ -141,57 +124,62 @@ void UnitProperties::loadUnitType(const QString& fileName, bool fullmode)
 	boWarning() << k_funcinfo << "Invalid TerrainType value: " << mTerrain << " for unit " << typeId() << ", defaulting to 0" << endl;
 	mTerrain = (TerrainType)0;
  }
- mUnitWidth = (unsigned int)(conf.readDoubleNumEntry("UnitWidth", 1.0) * BO_TILE_SIZE);
- mUnitHeight = (unsigned int)(conf.readDoubleNumEntry("UnitHeight", 1.0) * BO_TILE_SIZE);
- mUnitDepth = (float)conf.readDoubleNumEntry("UnitDepth", 1.0);
+ mUnitWidth = conf.readDoubleNumEntry("UnitWidth", 1.0);
+ mUnitHeight = (conf.readDoubleNumEntry("UnitHeight", 1.0));
+ mUnitDepth = conf.readDoubleNumEntry("UnitDepth", 1.0);
  d->mName = conf.readEntry("Name", i18n("Unknown"));
- mHealth = conf.readUnsignedLongNumEntry("Health", 100);
- mMineralCost = conf.readUnsignedLongNumEntry("MineralCost", 100);
- mOilCost = conf.readUnsignedLongNumEntry("OilCost", 0);
- mSightRange = conf.readUnsignedLongNumEntry("SightRange", 5);
- mProductionTime = conf.readUnsignedNumEntry("ProductionTime", 100);
- mShields = conf.readUnsignedLongNumEntry("Shield", 0);
- mArmor = conf.readUnsignedLongNumEntry("Armor", 0);
+ d->mDescription = conf.readEntry("Description", QString::null);
+ insertULongBaseValue(conf.readUnsignedLongNumEntry("Health", 100), "Health", "MaxValue");
+ insertULongBaseValue(conf.readUnsignedLongNumEntry("SightRange", 5), "SightRange", "MaxValue");
+ // We convert this from seconds to advance calls
+ insertULongBaseValue((unsigned long int)(conf.readDoubleNumEntry("ProductionTime", 5) * 20.0f), "ProductionTime", "MaxValue");
+ insertULongBaseValue(conf.readUnsignedLongNumEntry("MineralCost", 100), "MineralCost", "MaxValue");
+ insertULongBaseValue(conf.readUnsignedLongNumEntry("OilCost", 0), "OilCost", "MaxValue");
+ insertULongBaseValue(conf.readUnsignedLongNumEntry("Armor", 0), "Armor", "MaxValue");
+ insertULongBaseValue(conf.readUnsignedLongNumEntry("Shield", 0), "Shields", "MaxValue");
  mSupportMiniMap = conf.readBoolEntry("SupportMiniMap", false);
  isFacility = conf.readBoolEntry("IsFacility", false);
  d->mRequirements = BosonConfig::readUnsignedLongNumList(&conf, "Requirements");
+
  mExplodingDamage = conf.readLongNumEntry("ExplodingDamage", 0);
- mExplodingDamageRange = (float)(conf.readDoubleNumEntry("ExplodingDamageRange", 0));
+ mExplodingDamageRange = conf.readDoubleNumEntry("ExplodingDamageRange", 0);
  mExplodingFragmentCount = conf.readUnsignedNumEntry("ExplodingFragmentCount", 0);
  mExplodingFragmentDamage = conf.readLongNumEntry("ExplodingFragmentDamage", 10);
- mExplodingFragmentDamageRange = (float)conf.readDoubleNumEntry("ExplodingFragmentDamageRange", 0.5);
+ mExplodingFragmentDamageRange = conf.readDoubleNumEntry("ExplodingFragmentDamageRange", 0.5);
  mRemoveWreckageImmediately = conf.readBoolEntry("RemoveWreckageImmediately", false);
  d->mExplodingFragmentFlyEffectIds = BosonConfig::readUnsignedLongNumList(&conf, "ExplodingFragmentFlyEffects");
  d->mExplodingFragmentHitEffectIds = BosonConfig::readUnsignedLongNumList(&conf, "ExplodingFragmentHitEffects");
- if (mFullMode) {
-	d->mExplodingFragmentFlyEffects = BosonEffectProperties::loadEffectProperties(d->mExplodingFragmentFlyEffectIds, mTheme);
-	d->mExplodingFragmentHitEffects = BosonEffectProperties::loadEffectProperties(d->mExplodingFragmentHitEffectIds, mTheme);
- }
- d->mHitPoint = BosonConfig::readBoVector3Entry(&conf, "HitPoint");  // FIXME: better name
- d->mHitPoint.cellToCanvas();
+ d->mHitPoint = BosonConfig::readBoVector3FixedEntry(&conf, "HitPoint", BoVector3Fixed(0, 0, mUnitDepth / 2.0f));  // FIXME: better name
 
  d->mDestroyedEffectIds = BosonConfig::readUnsignedLongNumList(&conf, "DestroyedEffects");
  d->mConstructedEffectIds = BosonConfig::readUnsignedLongNumList(&conf, "ConstructedEffects");
- if (mFullMode) {
-	d->mDestroyedEffects = BosonEffectProperties::loadEffectProperties(d->mDestroyedEffectIds, mTheme);
-	d->mConstructedEffects = BosonEffectProperties::loadEffectProperties(d->mConstructedEffectIds, mTheme);
- }
+
+ mIsFacility = isFacility;
+
+ // AB: note that we need to load _both_ in order to initialize all variables.
+ loadFacilityProperties(&conf);
+ loadMobileProperties(&conf);
 
  if (isFacility) {
 	mProducer = conf.readUnsignedNumEntry("Producer", (unsigned int)CommandBunker);
-	loadFacilityProperties(&conf);
  } else {
 	mProducer = conf.readUnsignedNumEntry("Producer", (unsigned int)mTerrain);
-	loadMobileProperties(&conf);
  }
 
  loadAllPluginProperties(&conf);
  loadTextureNames(&conf);
  loadSoundNames(&conf);
  loadWeapons(&conf);
+
+ if (!loadActions(&conf)) {
+	boError() << k_funcinfo << "loading actions failed" << endl;
+	return false;
+ }
+
+ return true;
 }
 
-void UnitProperties::saveUnitType(const QString& fileName)
+bool UnitProperties::saveUnitType(const QString& fileName)
 {
  d->mUnitPath = fileName.left(fileName.length() - QString("index.unit").length());
  KSimpleConfig conf(fileName);
@@ -199,25 +187,25 @@ void UnitProperties::saveUnitType(const QString& fileName)
 
  conf.writeEntry("Id", typeId());
  conf.writeEntry("TerrainType", (int)mTerrain);
- conf.writeEntry("UnitWidth", (double)mUnitWidth / BO_TILE_SIZE);
- conf.writeEntry("UnitHeight", (double)mUnitHeight / BO_TILE_SIZE);
+ conf.writeEntry("UnitWidth", (double)mUnitWidth);
+ conf.writeEntry("UnitHeight", (double)mUnitHeight);
  conf.writeEntry("UnitDepth", (double)mUnitDepth);
  conf.writeEntry("Name", d->mName);
- conf.writeEntry("Health", mHealth);
- conf.writeEntry("MineralCost", mMineralCost);
- conf.writeEntry("OilCost", mOilCost);
- conf.writeEntry("SightRange", mSightRange);
- conf.writeEntry("ProductionTime", mProductionTime);
- conf.writeEntry("Shield", mShields);
- conf.writeEntry("Armor", mArmor);
+ conf.writeEntry("Health", ulongBaseValue("Health", "MaxValue", 100));
+ conf.writeEntry("MineralCost", ulongBaseValue("MineralCost"));
+ conf.writeEntry("OilCost", ulongBaseValue("OilCost"));
+ conf.writeEntry("SightRange", ulongBaseValue("SightRange"));
+ // This is converted from advance calls to seconds
+ conf.writeEntry("ProductionTime", ulongBaseValue("ProductionTime") / 20.0f);
+ conf.writeEntry("Shield", ulongBaseValue("Shields"));
+ conf.writeEntry("Armor", ulongBaseValue("Armor"));
  conf.writeEntry("SupportMiniMap", mSupportMiniMap);
  conf.writeEntry("IsFacility", isFacility());
  BosonConfig::writeUnsignedLongNumList(&conf, "Requirements", d->mRequirements);
  conf.writeEntry("ExplodingDamage", mExplodingDamage);
- conf.writeEntry("ExplodingDamageRange", mExplodingDamageRange);
- BoVector3 tmpHitPoint(d->mHitPoint);
- tmpHitPoint.canvasToCell();
- BosonConfig::writeEntry(&conf, "HitPoint", tmpHitPoint);
+ conf.writeEntry("ExplodingDamageRange", (double)mExplodingDamageRange);
+ BoVector3Fixed tmpHitPoint(d->mHitPoint);
+ BosonConfig::writeEntry(&conf, "HitPoint", d->mHitPoint);
  conf.writeEntry("Producer", mProducer);
 
  BosonConfig::writeUnsignedLongNumList(&conf, "DestroyedEffects", d->mDestroyedEffectIds);
@@ -232,35 +220,45 @@ void UnitProperties::saveUnitType(const QString& fileName)
  saveAllPluginProperties(&conf);  // This saves weapons too
  saveTextureNames(&conf);
  saveSoundNames(&conf);
+
+ return true;
 }
 
-void UnitProperties::loadMobileProperties(KSimpleConfig* conf)
+bool UnitProperties::loadMobileProperties(KSimpleConfig* conf)
 {
  conf->setGroup("Boson Mobile Unit");
- createMobileProperties();
- mMobileProperties->mSpeed = (float)conf->readDoubleNumEntry("Speed", 0);
- if (mMobileProperties->mSpeed < 0) {
-	boWarning() << k_funcinfo << "Invalid Speed value: " << mMobileProperties->mSpeed <<
+ // We divide speeds with 20, because speeds in config files are cells/second,
+ //  but we want cells/advance call
+ insertBoFixedBaseValue(conf->readDoubleNumEntry("Speed", 0) / 20.0f, "Speed", "MaxValue");
+ if (bofixedBaseValue("Speed") < 0) {
+	boWarning() << k_funcinfo << "Invalid Speed value: " << bofixedBaseValue("Speed") <<
 			" for unit " << typeId() << ", defaulting to 0" << endl;
-	mMobileProperties->mSpeed = 0;
+	insertBoFixedBaseValue(0, "Speed", "MaxValue");
  }
- mMobileProperties->mAccelerationSpeed = (float)conf->readDoubleNumEntry("AccelerationSpeed", 0.1);
- mMobileProperties->mDecelerationSpeed = (float)conf->readDoubleNumEntry("DecelerationSpeed", 0.2);
- mMobileProperties->mRotationSpeed = conf->readNumEntry("RotationSpeed", (int)(mMobileProperties->mSpeed * 2));
- mMobileProperties->mCanGoOnLand = conf->readBoolEntry("CanGoOnLand",
-		(isLand() || isAircraft()));
- mMobileProperties->mCanGoOnWater = conf->readBoolEntry("CanGoOnWater",
-		(isShip() || isAircraft()));
+ insertBoFixedBaseValue(conf->readDoubleNumEntry("AccelerationSpeed", 1) / 20.0f / 20.0f, "AccelerationSpeed", "MaxValue");
+ insertBoFixedBaseValue(conf->readDoubleNumEntry("DecelerationSpeed", 2) / 20.0f / 20.0f, "DecelerationSpeed", "MaxValue");
+ mRotationSpeed = (int)(conf->readNumEntry("RotationSpeed", (int)(bofixedBaseValue("Speed") * 20.0f * 90.0f)) / 20.0f);
+ mCanGoOnLand = conf->readBoolEntry("CanGoOnLand", (isLand() || isAircraft()));
+ mCanGoOnWater = conf->readBoolEntry("CanGoOnWater", (isShip() || isAircraft()));
+ mCrushDamage = conf->readUnsignedLongNumEntry("CrushDamage", 0);
+ mMaxSlope = conf->readDoubleNumEntry("MaxSlope", 30);
+ mWaterDepth = conf->readDoubleNumEntry("WaterDepth", 0.25);
+
+ // Those are relevant only for aircrafts
+ mIsHelicopter = conf->readBoolEntry("IsHelicopter", false);
+ mTurnRadius = conf->readDoubleNumEntry("TurnRadius", 5);
+ mPreferredAltitude = conf->readDoubleNumEntry("PreferredAltitude", 3);
+ return true;
 }
 
-void UnitProperties::loadFacilityProperties(KSimpleConfig* conf)
+bool UnitProperties::loadFacilityProperties(KSimpleConfig* conf)
 {
  conf->setGroup("Boson Facility");
- createFacilityProperties();
- mFacilityProperties->mConstructionFrames = conf->readUnsignedNumEntry("ConstructionSteps", 20);
+ mConstructionFrames = conf->readUnsignedNumEntry("ConstructionSteps", 20);
+ return true;
 }
 
-void UnitProperties::loadAllPluginProperties(KSimpleConfig* conf)
+bool UnitProperties::loadAllPluginProperties(KSimpleConfig* conf)
 {
  if (conf->hasGroup(ProductionProperties::propertyGroup())) {
 	loadPluginProperties(new ProductionProperties(this), conf);
@@ -272,28 +270,29 @@ void UnitProperties::loadAllPluginProperties(KSimpleConfig* conf)
 	loadPluginProperties(new HarvesterProperties(this), conf);
  }
  if (conf->hasGroup(RefineryProperties::propertyGroup())) {
-	boDebug() << k_funcinfo << "loading refinery plugin properties" << endl;
 	loadPluginProperties(new RefineryProperties(this), conf);
  }
  if (conf->hasGroup(ResourceMineProperties::propertyGroup())) {
 	loadPluginProperties(new ResourceMineProperties(this), conf);
  }
+ return true;
 }
 
-void UnitProperties::loadPluginProperties(PluginProperties* prop, KSimpleConfig* conf)
+bool UnitProperties::loadPluginProperties(PluginProperties* prop, KSimpleConfig* conf)
 {
  if (!prop || !conf) {
 	boError() << k_funcinfo << "oops" << endl;
-	return;
+	return false;
  }
  prop->loadPlugin(conf);
  d->mPlugins.append(prop);
+ return true;
 }
 
-void UnitProperties::loadTextureNames(KSimpleConfig* conf)
+bool UnitProperties::loadTextureNames(KSimpleConfig* conf)
 {
  if (!conf->hasGroup("Textures")) {
-	return;
+	return true;
  }
  d->mTextureNames.clear();
  conf->setGroup("Textures");
@@ -305,9 +304,10 @@ void UnitProperties::loadTextureNames(KSimpleConfig* conf)
 		boDebug() << "mapping " << textures[i] << "->" << longName << endl;
 	}
  }
+ return true;
 }
 
-void UnitProperties::loadSoundNames(KSimpleConfig* conf)
+bool UnitProperties::loadSoundNames(KSimpleConfig* conf)
 {
  d->mSounds.clear();
  conf->setGroup("Sounds");
@@ -317,86 +317,82 @@ void UnitProperties::loadSoundNames(KSimpleConfig* conf)
  d->mSounds.insert(SoundReportProduced, conf->readEntry("ReportProduced", "report_produced"));
  d->mSounds.insert(SoundReportDestroyed, conf->readEntry("ReportDestroyed", "report_destroyed"));
  d->mSounds.insert(SoundReportUnderAttack, conf->readEntry("ReportUnderAttack", "report_underattack"));
+ return true;
 }
 
-void UnitProperties::loadWeapons(KSimpleConfig* conf)
+bool UnitProperties::loadWeapons(KSimpleConfig* conf)
 {
  conf->setGroup("Boson Unit");
  int num = conf->readNumEntry("Weapons", 0);
  mCanShootAtAirUnits = false;
  mCanShootAtLandUnits = false;
- mMaxAirWeaponRange = 0;
- mMaxLandWeaponRange = 0;
  for (int i = 0; i < num; i++) {
 	conf->setGroup(QString("Weapon_%1").arg(i));
 	BosonWeaponProperties* p = new BosonWeaponProperties(this, i + 1);
-	p->loadPlugin(conf, mFullMode);
+	p->loadPlugin(conf);
 	d->mPlugins.append(p);
 	if (!p->autoUse()) {
 		continue;
 	}
 	if (p->canShootAtAirUnits()) {
 		mCanShootAtAirUnits = true;
-		if(p->range() > mMaxAirWeaponRange) {
-			mMaxAirWeaponRange = p->range();
-		}
 	}
 	if (p->canShootAtLandUnits()) {
 		mCanShootAtLandUnits = true;
-		if(p->range() > mMaxLandWeaponRange) {
-			mMaxLandWeaponRange = p->range();
-		}
 	}
  }
- mMaxWeaponRange = QMAX(mMaxAirWeaponRange, mMaxLandWeaponRange);
+ return true;
 }
 
-void UnitProperties::loadActions()
+bool UnitProperties::loadActions(KSimpleConfig* conf)
 {
- KSimpleConfig conf(unitPath() + "index.unit");
+ BosonProfiler prof("LoadActions");
  // Produce action first
  // Produce is special because it uses little overview as pixmap and it's
  //  tooltip text is auto-generated. Only thing we load here is hotkey
  // TODO: load hotkey
- conf.setGroup("Actions");
- mProduceAction = new BoAction(QString("ProduceAction-%1").arg(typeId()), theme()->smallOverview(typeId()),
-		"Produce"/*, hotkey*/);
+ conf->setGroup("Actions");
  if (canShoot()) {
-	d->mActions.insert(ActionAttack, theme()->action(conf.readEntry("ActionAttack", "ActionAttack")));
+	d->mActionStrings.insert(ActionAttack, conf->readEntry("ActionAttack", "ActionAttack"));
  }
  if (isMobile()) {
-	d->mActions.insert(ActionMove, theme()->action(conf.readEntry("ActionMove", "ActionMove")));
-	d->mActions.insert(ActionFollow, theme()->action(conf.readEntry("ActionFollow", "ActionFollow")));
+	d->mActionStrings.insert(ActionMove, conf->readEntry("ActionMove", "ActionMove"));
+	d->mActionStrings.insert(ActionFollow, conf->readEntry("ActionFollow", "ActionFollow"));
  }
  if (properties(PluginProperties::Harvester)) {
-	d->mActions.insert(ActionHarvest, theme()->action(conf.readEntry("ActionHarvest", "ActionHarvest")));
+	d->mActionStrings.insert(ActionHarvest, conf->readEntry("ActionHarvest", "ActionHarvest"));
  }
  if (properties(PluginProperties::Repair)) {
-	d->mActions.insert(ActionHarvest, theme()->action(conf.readEntry("ActionRepair", "ActionRepair")));
+	d->mActionStrings.insert(ActionHarvest, conf->readEntry("ActionRepair", "ActionRepair"));
  }
- if (!d->mActions.isEmpty()) {
-	d->mActions.insert(ActionStop, theme()->action(conf.readEntry("ActionStop", "ActionStop")));
+ if (!d->mActionStrings.isEmpty()) {
+	d->mActionStrings.insert(ActionStop, conf->readEntry("ActionStop", "ActionStop"));
  }
+ return true;
 }
 
-void UnitProperties::saveMobileProperties(KSimpleConfig* conf)
+bool UnitProperties::saveMobileProperties(KSimpleConfig* conf)
 {
  conf->setGroup("Boson Mobile Unit");
- conf->writeEntry("Speed", (double)mMobileProperties->mSpeed);
- conf->writeEntry("AccelerationSpeed", (double)mMobileProperties->mAccelerationSpeed);
- conf->writeEntry("DecelerationSpeed", (double)mMobileProperties->mDecelerationSpeed);
- conf->writeEntry("RotationSpeed", mMobileProperties->mRotationSpeed);
- conf->writeEntry("CanGoOnLand", mMobileProperties->mCanGoOnLand);
- conf->writeEntry("CanGoOnWater", mMobileProperties->mCanGoOnWater);
+ // We multiply speeds with 20 because speeds in config files are cells/second,
+ //  but here we have cells/advance call
+ conf->writeEntry("Speed", bofixedBaseValue("Speed") * 20.0f);
+ conf->writeEntry("AccelerationSpeed", (double)bofixedBaseValue("AccelerationSpeed") * 20.0f * 20.0f);
+ conf->writeEntry("DecelerationSpeed", (double)bofixedBaseValue("DecelerationSpeed") * 20.0f * 20.0f);
+ conf->writeEntry("RotationSpeed", mRotationSpeed * 20.0f);
+ conf->writeEntry("CanGoOnLand", mCanGoOnLand);
+ conf->writeEntry("CanGoOnWater", mCanGoOnWater);
+ return true;
 }
 
-void UnitProperties::saveFacilityProperties(KSimpleConfig* conf)
+bool UnitProperties::saveFacilityProperties(KSimpleConfig* conf)
 {
  conf->setGroup("Boson Facility");
- conf->writeEntry("ConstructionSteps", mFacilityProperties->mConstructionFrames);
+ conf->writeEntry("ConstructionSteps", mConstructionFrames);
+ return true;
 }
 
-void UnitProperties::saveAllPluginProperties(KSimpleConfig* conf)
+bool UnitProperties::saveAllPluginProperties(KSimpleConfig* conf)
 {
  int weaponcounter = 0;
  QPtrListIterator<PluginProperties> it(d->mPlugins);
@@ -410,12 +406,13 @@ void UnitProperties::saveAllPluginProperties(KSimpleConfig* conf)
  }
  conf->setGroup("Boson Unit");
  conf->writeEntry("Weapons", weaponcounter);
+ return true;
 }
 
-void UnitProperties::saveTextureNames(KSimpleConfig* conf)
+bool UnitProperties::saveTextureNames(KSimpleConfig* conf)
 {
  if (d->mTextureNames.count() == 0) {
-	return;
+	return true;
  }
  conf->setGroup("Textures");
  QMap<QString, QString>::Iterator it;
@@ -425,9 +422,10 @@ void UnitProperties::saveTextureNames(KSimpleConfig* conf)
 	conf->writeEntry(it.key(), it.data());
  }
  conf->writeEntry("Textures", textures);
+ return true;
 }
 
-void UnitProperties::saveSoundNames(KSimpleConfig* conf)
+bool UnitProperties::saveSoundNames(KSimpleConfig* conf)
 {
  conf->setGroup("Sounds");
  conf->writeEntry("OrderMove", d->mSounds[SoundOrderMove]);
@@ -436,6 +434,12 @@ void UnitProperties::saveSoundNames(KSimpleConfig* conf)
  conf->writeEntry("ReportProduced", d->mSounds[SoundReportProduced]);
  conf->writeEntry("ReportDestroyed", d->mSounds[SoundReportDestroyed]);
  conf->writeEntry("ReportUnderAttack", d->mSounds[SoundReportUnderAttack]);
+ return true;
+}
+
+const QCString& UnitProperties::md5() const
+{
+ return d->mMD5;
 }
 
 void UnitProperties::setName(const QString& n)
@@ -448,6 +452,11 @@ const QString& UnitProperties::name() const
  return d->mName;
 }
 
+const QString& UnitProperties::description() const
+{
+ return d->mDescription;
+}
+
 const QString& UnitProperties::unitPath() const
 {
  return d->mUnitPath;
@@ -478,14 +487,17 @@ QMap<int, QString> UnitProperties::sounds() const
  return d->mSounds;
 }
 
-BoAction* UnitProperties::action(UnitAction type) const
+QString UnitProperties::actionString(UnitAction type) const
 {
- return d->mActions[type];
+ if (!d->mActionStrings.contains(type)) {
+	return QString::null;
+ }
+ return d->mActionStrings[type];
 }
 
-const QIntDict<BoAction>* UnitProperties::allActions() const
+const QMap<int, QString>* UnitProperties::allActionStrings() const
 {
- return &d->mActions;
+ return &d->mActionStrings;
 }
 
 void UnitProperties::setRequirements(QValueList<unsigned long int> requirements)
@@ -493,70 +505,84 @@ void UnitProperties::setRequirements(QValueList<unsigned long int> requirements)
  d->mRequirements = requirements;
 }
 
-bool UnitProperties::isMobile() const
+bofixed UnitProperties::maxSpeed() const
 {
- return (mMobileProperties != 0);
-}
-
-bool UnitProperties::isFacility() const
-{
- return (mFacilityProperties != 0);
-}
-
-float UnitProperties::speed() const
-{
- if (!mMobileProperties) {
-	return 0;
+ if (!isMobile()) {
+	return bofixed(0);
  }
- return mMobileProperties->mSpeed;
+ return mSpeed.value(upgradesCollection());
 }
 
-float UnitProperties::accelerationSpeed() const
+bofixed UnitProperties::maxAccelerationSpeed() const
 {
- if (!mMobileProperties) {
-	return 0;
+ if (!isMobile()) {
+	return bofixed(0);
  }
- return mMobileProperties->mAccelerationSpeed;
+ return mAccelerationSpeed.value(upgradesCollection());
 }
 
-float UnitProperties::decelerationSpeed() const
+bofixed UnitProperties::maxDecelerationSpeed() const
 {
- if (!mMobileProperties) {
-	return 0;
+ if (!isMobile()) {
+	return bofixed(0);
  }
- return mMobileProperties->mDecelerationSpeed;
+ return mDecelerationSpeed.value(upgradesCollection());
 }
 
 int UnitProperties::rotationSpeed() const
 {
- if (!mMobileProperties) {
+ if (!isMobile()) {
 	return 0;
  }
- return mMobileProperties->mRotationSpeed;
+ return mRotationSpeed;
+}
+
+bool UnitProperties::isHelicopter() const
+{
+ if (!isMobile() || !isAircraft()) {
+	return false;
+ }
+ return mIsHelicopter;
+}
+
+bofixed UnitProperties::turnRadius() const
+{
+ if (!isMobile() || !isAircraft()) {
+	return 0;
+ }
+ return mTurnRadius;
+}
+
+bofixed UnitProperties::preferredAltitude() const
+{
+ if (!isMobile() || !isAircraft()) {
+	return 0;
+ }
+ return mPreferredAltitude;
 }
 
 bool UnitProperties::canGoOnLand() const
 {
- if (!mMobileProperties) {
+ if (!isMobile()) {
 	return true; // even facilities can go there.
  }
- return mMobileProperties->mCanGoOnLand;
+ return mCanGoOnLand;
 }
 
 bool UnitProperties::canGoOnWater() const
 {
- if (!mMobileProperties) {
+ if (!isMobile()) {
 	return false;
  }
- return mMobileProperties->mCanGoOnWater;
+ return mCanGoOnWater;
 }
 
 unsigned int UnitProperties::constructionSteps() const
 {
- if (!mFacilityProperties) {
+ if (!isFacility()) {
 	return 0;
  }
- return mFacilityProperties->mConstructionFrames;
+ return mConstructionFrames;
 }
 
 const PluginProperties* UnitProperties::properties(int pluginType) const
@@ -570,26 +596,6 @@ const PluginProperties* UnitProperties::properties(int pluginType) const
  return 0;
 }
 
-QPtrList<BosonEffect> UnitProperties::newDestroyedEffects(float x, float y, float z) const
-{
- return BosonEffectProperties::newEffects(&d->mDestroyedEffects, BoVector3(x, y, z));
-}
-
-QPtrList<BosonEffect> UnitProperties::newConstructedEffects(float x, float y, float z) const
-{
- return BosonEffectProperties::newEffects(&d->mConstructedEffects, BoVector3(x, y, z));
-}
-
-QPtrList<BosonEffect> UnitProperties::newExplodingFragmentFlyEffects(BoVector3 pos) const
-{
- return BosonEffectProperties::newEffects(&d->mExplodingFragmentFlyEffects, pos);
-}
-
-QPtrList<BosonEffect> UnitProperties::newExplodingFragmentHitEffects(BoVector3 pos) const
-{
- return BosonEffectProperties::newEffects(&d->mExplodingFragmentHitEffects, pos);
-}
-
 void UnitProperties::clearPlugins(bool deleteweapons)
 {
  // FIXME: deleteweapons is very ugly hack here. In unit editor, we store
@@ -608,20 +614,6 @@ void UnitProperties::clearPlugins(bool deleteweapons)
  } else {
 	d->mPlugins.clear();
  }
- delete mMobileProperties;
- mMobileProperties = 0;
- delete mFacilityProperties;
- mFacilityProperties = 0;
-}
-
-void UnitProperties::createMobileProperties()
-{
- mMobileProperties = new MobileProperties;
-}
-
-void UnitProperties::createFacilityProperties()
-{
- mFacilityProperties = new FacilityProperties;
 }
 
 void UnitProperties::addPlugin(PluginProperties* prop)
@@ -631,50 +623,29 @@ void UnitProperties::addPlugin(PluginProperties* prop)
 
 void UnitProperties::setConstructionSteps(unsigned int steps)
 {
- if (mFacilityProperties) {
-	mFacilityProperties->mConstructionFrames = steps;
- }
-}
-
-void UnitProperties::setSpeed(float speed)
-{
- if (mMobileProperties) {
-	mMobileProperties->mSpeed = speed;
- }
-}
-
-void UnitProperties::setAccelerationSpeed(float speed)
-{
- if (mMobileProperties) {
-	mMobileProperties->mAccelerationSpeed = speed;
- }
-}
-
-void UnitProperties::setDecelerationSpeed(float speed)
-{
- if (mMobileProperties) {
-	mMobileProperties->mDecelerationSpeed = speed;
+ if (isFacility()) {
+	mConstructionFrames = steps;
  }
 }
 
 void UnitProperties::setRotationSpeed(int speed)
 {
- if (mMobileProperties) {
-	mMobileProperties->mRotationSpeed = speed;
+ if (isMobile()) {
+	mRotationSpeed = speed;
  }
 }
 
 void UnitProperties::setCanGoOnLand(bool c)
 {
- if (mMobileProperties) {
-	mMobileProperties->mCanGoOnLand = c;
+ if (isMobile()) {
+	mCanGoOnLand = c;
  }
 }
 
 void UnitProperties::setCanGoOnWater(bool c)
 {
- if (mMobileProperties) {
-	mMobileProperties->mCanGoOnWater = c;
+ if (isMobile()) {
+	mCanGoOnWater = c;
  }
 }
 
@@ -693,7 +664,7 @@ void UnitProperties::setDestroyedEffectIds(QValueList<unsigned long int> ids)
  d->mDestroyedEffectIds = ids;
 }
 
-QValueList<unsigned long int> UnitProperties::destroyedEffectIds() const
+const QValueList<unsigned long int>& UnitProperties::destroyedEffectIds() const
 {
  return d->mDestroyedEffectIds;
 }
@@ -703,11 +674,21 @@ void UnitProperties::setConstructedEffectIds(QValueList<unsigned long int> ids)
  d->mConstructedEffectIds = ids;
 }
 
-QValueList<unsigned long int> UnitProperties::constructedEffectIds() const
+const QValueList<unsigned long int>& UnitProperties::constructedEffectIds() const
 {
  return d->mConstructedEffectIds;
 }
 
+const QValueList<unsigned long int>& UnitProperties::explodingFragmentFlyEffectIds() const
+{
+ return d->mExplodingFragmentFlyEffectIds;
+}
+
+const QValueList<unsigned long int>& UnitProperties::explodingFragmentHitEffectIds() const
+{
+ return d->mExplodingFragmentHitEffectIds;
+}
+
 void UnitProperties::reset()
 {
  if (mFullMode) {
@@ -719,17 +700,20 @@ void UnitProperties::reset()
  d->mUnitPath = "";
  mTypeId = 0;
  mTerrain = (TerrainType)0;
- mUnitWidth = (unsigned int)(1.0 * BO_TILE_SIZE);
- mUnitHeight = (unsigned int)(1.0 * BO_TILE_SIZE);
+ mUnitWidth = 1.0f;
+ mUnitHeight = 1.0f;
  mUnitDepth = 1.0;
  d->mName = i18n("Unknown");
- mHealth = 100;
- mMineralCost = 100;
- mOilCost = 0;
- mSightRange = 5;
- mProductionTime = 100;
- mShields = 0;
- mArmor = 0;
+ insertULongBaseValue(100, "Health", "MaxValue");
+ insertULongBaseValue(5, "SightRange", "MaxValue");
+ insertULongBaseValue(100, "ProductionTime", "MaxValue");
+ insertULongBaseValue(100, "MineralCost", "MaxValue");
+ insertULongBaseValue(0, "OilCost", "MaxValue");
+ insertULongBaseValue(0, "Armor", "MaxValue");
+ insertULongBaseValue(0, "Shields", "MaxValue");
+ insertBoFixedBaseValue(0, "Speed", "MaxValue");
+ insertBoFixedBaseValue(2.0f / 20.0f / 20.0f, "AccelerationSpeed", "MaxValue");
+ insertBoFixedBaseValue(4.0f / 20.0f / 20.0f, "DecelerationSpeed", "MaxValue");
  mSupportMiniMap = false;
  d->mRequirements.clear();
  d->mDestroyedEffectIds.clear();
@@ -738,19 +722,11 @@ void UnitProperties::reset()
  mProducer = 0;
  mExplodingDamage = 0;
  mExplodingDamageRange = 0;
- // Delete old mobile/facility properties
- delete mMobileProperties;
- mMobileProperties = 0;
- delete mFacilityProperties;
- mFacilityProperties = 0;
  // Mobile stuff (because unit is mobile by default)
- createMobileProperties();
- mMobileProperties->mSpeed = 0; // Hmm, this doesn't make any sense IMO
- mMobileProperties->mAccelerationSpeed = 0.5;
- mMobileProperties->mDecelerationSpeed = 1.0;
- mMobileProperties->mRotationSpeed = (int)(2 * mMobileProperties->mSpeed);
- mMobileProperties->mCanGoOnLand = true;
- mMobileProperties->mCanGoOnWater = false;
+ mIsFacility = false;
+ mRotationSpeed = (int)(45.0f * bofixedBaseValue("Speed"));
+ mCanGoOnLand = true;
+ mCanGoOnWater = false;
  // Sounds
  d->mSounds.clear();
  d->mSounds.insert(SoundOrderMove, "order_move");
@@ -763,17 +739,17 @@ void UnitProperties::reset()
  d->mTextureNames.clear();
 }
 
-void UnitProperties::setHitPoint(const BoVector3& hitpoint)
+void UnitProperties::setHitPoint(const BoVector3Fixed& hitpoint)
 {
  d->mHitPoint = hitpoint;
 }
 
-const BoVector3& UnitProperties::hitPoint() const
+const BoVector3Fixed& UnitProperties::hitPoint() const
 {
  return d->mHitPoint;
 }
 
-const BosonWeaponProperties* UnitProperties::weaponProperties(unsigned long int id) const
+BosonWeaponProperties* UnitProperties::nonConstWeaponProperties(unsigned long int id) const
 {
  QPtrListIterator<PluginProperties> it(d->mPlugins);
  while (it.current()) {
@@ -784,6 +760,62 @@ const BosonWeaponProperties* UnitProperties::weaponProperties(unsigned long int
 	}
 	++it;
  }
- return 0l;
+ return 0;
+}
+
+const BosonWeaponProperties* UnitProperties::weaponProperties(unsigned long int id) const
+{
+ return nonConstWeaponProperties(id);
+}
+
+const BoUpgradesCollection& UnitProperties::upgradesCollection() const
+{
+ return d->mUpgradesCollection;
+}
+
+bool UnitProperties::saveUpgradesAsXML(QDomElement& root) const
+{
+ return upgradesCollection().saveAsXML(root);
+}
+
+bool UnitProperties::loadUpgradesFromXML(const QDomElement& root)
+{
+ if (!mTheme) {
+	BO_NULL_ERROR(mTheme);
+	return false;
+ }
+ return d->mUpgradesCollection.loadFromXML(mTheme, root);
+}
+
+void UnitProperties::clearUpgrades()
+{
+ d->mUpgradesCollection.clearUpgrades();
+}
+
+void UnitProperties::addUpgrade(const UpgradeProperties* upgrade)
+{
+ d->mUpgradesCollection.addUpgrade(upgrade);
+}
+
+void UnitProperties::removeUpgrade(const UpgradeProperties* upgrade)
+{
+ d->mUpgradesCollection.removeUpgrade(upgrade);
+}
+
+void UnitProperties::removeUpgrade(unsigned long int id)
+{
+ removeUpgrade(d->mUpgradesCollection.findUpgrade(id));
+}
+
+bool UnitProperties::canGo(int x, int y)
+{
+ if (isAircraft()) {
+	// Aircrafts can go everywhere
+	return true;
+ } else {
+	// TODO: implement this (we could use map's width here...Z)
+	return true;
+	//return moveData()->cellPassable[...];
+ }
 }
 
